@@ -16,9 +16,10 @@
 # This script will extract translatable strings from input files and write
 # to output in gettext .pot format.
 #
+from __future__ import print_function
+
 import sys, os, re, getopt
-if sys.version_info < (2, 4, 0):
-    from sets import Set as set
+import io
 
 def relativePath(path, base):
     '''return relative path from top source dir'''
@@ -26,7 +27,7 @@ def relativePath(path, base):
     path1 = os.path.normpath(os.path.realpath(path)).split(os.sep)
     path2 = os.path.normpath(os.path.realpath(base)).split(os.sep)
     if path1[:len(path2)] != path2:
-        print "Path %s is not under top source directory" % path
+        print("Path %s is not under top source directory" % path)
     path3 = os.path.join(*path1[len(path2):]);
     # replace all \ by / such that we get the same comments on Windows and *nix
     path3 = path3.replace('\\', '/')
@@ -37,13 +38,13 @@ def writeString(outfile, infile, basefile, lineno, string):
     string = string.replace('\\', '\\\\').replace('"', '')
     if string == "":
         return
-    print >> outfile, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-        (relativePath(infile, basefile), lineno, string)
+    print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+        (relativePath(infile, basefile), lineno, string), file=outfile)
 
 
 def ui_l10n(input_files, output, base):
     '''Generate pot file from lib/ui/*'''
-    output = open(output, 'w')
+    output = io.open(output, 'w', encoding='utf_8')
     Submenu = re.compile(r'^[^#]*Submenu\s+"([^"]*)"', re.IGNORECASE)
     Popupmenu = re.compile(r'^[^#]*PopupMenu\s+"[^"]+"\s+"([^"]*)"', re.IGNORECASE)
     IconPalette = re.compile(r'^[^#]*IconPalette\s+"[^"]+"\s+"([^"]*)"', re.IGNORECASE)
@@ -51,7 +52,7 @@ def ui_l10n(input_files, output, base):
     Item = re.compile(r'[^#]*Item\s+"([^"]*)"', re.IGNORECASE)
     TableInsert = re.compile(r'[^#]*TableInsert\s+"([^"]*)"', re.IGNORECASE)
     for src in input_files:
-        input = open(src)
+        input = io.open(src, encoding='utf_8')
         for lineno, line in enumerate(input.readlines()):
             if Submenu.match(line):
                 (string,) = Submenu.match(line).groups()
@@ -70,8 +71,8 @@ def ui_l10n(input_files, output, base):
                 continue
             string = string.replace('"', '')
             if string != "":
-                print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                    (relativePath(src, base), lineno+1, string)
+                print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                    (relativePath(src, base), lineno+1, string), file=output)
         input.close()
     output.close()
 
@@ -84,6 +85,7 @@ def layouts_l10n(input_files, output, base, layouttranslations):
     # match LabelString, EndLabelString, LabelStringAppendix and maybe others but no comments
     LabelString = re.compile(r'^[^#]*LabelString\S*\s+(.*\S)\s*$', re.IGNORECASE)
     MenuString = re.compile(r'^[^#]*MenuString\S*\s+(.*\S)\s*$', re.IGNORECASE)
+    OutlinerName = re.compile(r'^[^#]*OutlinerName\s+(\S+|\"[^\"]*\")\s+(\S+|\"[^\"]*\")\s*$', re.IGNORECASE)
     Tooltip = re.compile(r'^\s*Tooltip\S*\s+(.*\S)\s*$', re.IGNORECASE)
     GuiName = re.compile(r'^\s*GuiName\s+(.*\S)\s*$', re.IGNORECASE)
     ListName = re.compile(r'^\s*ListName\s+(.*\S)\s*$', re.IGNORECASE)
@@ -124,7 +126,7 @@ def layouts_l10n(input_files, output, base, layouttranslations):
 
         # read old translations if available
         try:
-            input = open(output)
+            input = io.open(output, encoding='utf_8')
             lang = ''
             for line in input.readlines():
                 res = Comment.search(line)
@@ -146,25 +148,25 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                     continue
                 res = KeyValPair.search(line)
                 if res and lang != '':
-                    key = res.group(1).decode('utf-8')
-                    val = res.group(2).decode('utf-8')
+                    key = res.group(1)
+                    val = res.group(2)
                     key = key.replace('\\"', '"').replace('\\\\', '\\')
                     val = val.replace('\\"', '"').replace('\\\\', '\\')
                     oldtrans[lang][key] = val
                     keyset.add(key)
                     continue
-                print "Error: Unable to handle line:"
-                print line
+                print("Error: Unable to handle line:")
+                print(line)
         except IOError:
-            print "Warning: Unable to open %s for reading." % output
-            print "         Old translations will be lost."
+            print("Warning: Unable to open %s for reading." % output)
+            print("         Old translations will be lost.")
 
         # walon is not a known document language
         # FIXME: Do not hardcode, read from lib/languages!
         if 'wa' in languages:
             languages.remove('wa')
 
-    out = open(output, 'w')
+    out = io.open(output, 'w', encoding='utf_8')
     for src in input_files:
         readingDescription = False
         readingI18nPreamble = False
@@ -177,7 +179,7 @@ def layouts_l10n(input_files, output, base, layouttranslations):
         descStartLine = -1
         descLines = []
         lineno = 0
-        for line in open(src).readlines():
+        for line in io.open(src, encoding='utf_8').readlines():
             lineno += 1
             res = ClassDescription.search(line)
             if res != None:
@@ -250,6 +252,12 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                 if not layouttranslations:
                     writeString(out, src, base, lineno, string)
                 continue
+            res = OutlinerName.search(line)
+            if res != None:
+                string = res.group(2)
+                if not layouttranslations:
+                    writeString(out, src, base, lineno, string)
+                continue
             res = Tooltip.search(line)
             if res != None:
                 string = res.group(1)
@@ -374,16 +382,16 @@ def layouts_l10n(input_files, output, base, layouttranslations):
 
         ContextRe = re.compile(r'(.*)(\[\[.*\]\])')
 
-        print >> out, '''# This file has been automatically generated by po/lyx_pot.py.
+        print(u'''# This file has been automatically generated by po/lyx_pot.py.
 # PLEASE MODIFY ONLY THE LAGUAGES HAVING NO .po FILE! If you want to regenerate
 # this file from the translations, run `make ../lib/layouttranslations' in po.
 # Python polib library is needed for building the output file.
 #
 # This file should remain fixed during minor LyX releases.
-# For more comments see README.localization file.'''
+# For more comments see README.localization file.''', file=out)
         for lang in languages:
-            print >> out, '\nTranslation %s' % lang
-            if lang in oldtrans.keys():
+            print(u'\nTranslation %s' % lang, file=out)
+            if lang in list(oldtrans.keys()):
                 trans = oldtrans[lang]
             else:
                 trans = dict()
@@ -402,14 +410,13 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                         if val != key:
                             trans[key] = val
             for key in keys:
-                if key in trans.keys():
+                if key in list(trans.keys()):
                     val = trans[key].replace('\\', '\\\\').replace('"', '\\"')
                     res = ContextRe.search(val)
                     if res != None:
                         val = res.group(1)
                     key = key.replace('\\', '\\\\').replace('"', '\\"')
-                    print >> out, '\t"%s" "%s"' % \
-                             (key.encode('utf-8'), val.encode('utf-8'))
+                    print(u'\t"%s" "%s"' % (key, val), file=out)
                 # also print untranslated entries to help translators
                 elif not lang in oldlanguages:
                     key = key.replace('\\', '\\\\').replace('"', '\\"')
@@ -418,20 +425,19 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                         val = res.group(1)
                     else:
                         val = key
-                    print >> out, '\t"%s" "%s"' % \
-                             (key.encode('utf-8'), val.encode('utf-8'))
-            print >> out, 'End'
+                    print(u'\t"%s" "%s"' % (key, val), file=out)
+            print(u'End', file=out)
 
     out.close()
 
 
 def qt4_l10n(input_files, output, base):
     '''Generate pot file from src/frontends/qt4/ui/*.ui'''
-    output = open(output, 'w')
+    output = io.open(output, 'w', encoding='utf_8')
     pat = re.compile(r'\s*<string>(.*)</string>')
     prop = re.compile(r'\s*<property.*name.*=.*shortcut')
     for src in input_files:
-        input = open(src)
+        input = io.open(src, encoding='utf_8')
         skipNextLine = False
         for lineno, line in enumerate(input.readlines()):
             # skip the line after <property name=shortcut>
@@ -448,55 +454,55 @@ def qt4_l10n(input_files, output, base):
                 string = string.replace('&lt;', '<').replace('&gt;', '>')
                 string = string.replace('\\', '\\\\').replace('"', r'\"')
                 string = string.replace('&#x0a;', r'\n')
-                print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                    (relativePath(src, base), lineno+1, string)
+                print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                    (relativePath(src, base), lineno+1, string), file=output)
         input.close()
     output.close()
 
 
 def languages_l10n(input_files, output, base):
     '''Generate pot file from lib/languages'''
-    out = open(output, 'w')
+    out = io.open(output, 'w', encoding='utf_8')
     GuiName = re.compile(r'^[^#]*GuiName\s+(.*)', re.IGNORECASE)
-    
+
     for src in input_files:
         descStartLine = -1
         descLines = []
         lineno = 0
-        for line in open(src).readlines():
+        for line in io.open(src, encoding='utf_8').readlines():
             lineno += 1
             res = GuiName.search(line)
             if res != None:
                 string = res.group(1)
                 writeString(out, src, base, lineno, string)
                 continue
-               
+
     out.close()
 
 
 def latexfonts_l10n(input_files, output, base):
     '''Generate pot file from lib/latexfonts'''
-    out = open(output, 'w')
+    out = io.open(output, 'w', encoding='utf_8')
     GuiName = re.compile(r'^[^#]*GuiName\s+(.*)', re.IGNORECASE)
-    
+
     for src in input_files:
         descStartLine = -1
         descLines = []
         lineno = 0
-        for line in open(src).readlines():
+        for line in io.open(src, encoding='utf_8').readlines():
             lineno += 1
             res = GuiName.search(line)
             if res != None:
                 string = res.group(1)
                 writeString(out, src, base, lineno, string)
                 continue
-               
+
     out.close()
 
 
 def external_l10n(input_files, output, base):
     '''Generate pot file from lib/external_templates'''
-    output = open(output, 'w')
+    output = io.open(output, 'w', encoding='utf_8')
     Template = re.compile(r'^Template\s+(.*)', re.IGNORECASE)
     GuiName = re.compile(r'\s*GuiName\s+(.*)', re.IGNORECASE)
     HelpTextStart = re.compile(r'\s*HelpText\s', re.IGNORECASE)
@@ -504,7 +510,7 @@ def external_l10n(input_files, output, base):
     HelpTextEnd = re.compile(r'\s*HelpTextEnd\s', re.IGNORECASE)
     i = -1
     for src in input_files:
-        input = open(src)
+        input = io.open(src, encoding='utf_8')
         inHelp = False
         hadHelp = False
         prev_help_string = ''
@@ -516,7 +522,7 @@ def external_l10n(input_files, output, base):
             elif inHelp:
                 if HelpTextEnd.match(line):
                     if hadHelp:
-                        print >> output, '\nmsgstr ""\n'
+                        print(u'\nmsgstr ""\n', file=output)
                     inHelp = False
                     hadHelp = False
                     prev_help_string = ''
@@ -524,11 +530,11 @@ def external_l10n(input_files, output, base):
                     (help_string,) = HelpTextSection.match(line).groups()
                     help_string = help_string.replace('"', '')
                     if help_string != "" and prev_help_string == '':
-                        print >> output, '#: %s:%d\nmsgid ""\n"%s\\n"' % \
-                            (relativePath(src, base), lineno+1, help_string)
+                        print(u'#: %s:%d\nmsgid ""\n"%s\\n"' % \
+                            (relativePath(src, base), lineno+1, help_string), file=output)
                         hadHelp = True
                     elif help_string != "":
-                        print >> output, '"%s\\n"' % help_string
+                        print(u'"%s\\n"' % help_string, file=output)
                     prev_help_string = help_string
             elif HelpTextStart.match(line):
                 inHelp = True
@@ -537,18 +543,18 @@ def external_l10n(input_files, output, base):
                 continue
             string = string.replace('"', '')
             if string != "" and not inHelp:
-                print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                    (relativePath(src, base), lineno+1, string)
+                print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                    (relativePath(src, base), lineno+1, string), file=output)
         input.close()
     output.close()
 
 
 def formats_l10n(input_files, output, base):
     '''Generate pot file from configure.py'''
-    output = open(output, 'w')
+    output = io.open(output, 'w', encoding='utf_8')
     GuiName = re.compile(r'.*\\Format\s+\S+\s+\S+\s+"([^"]*)"\s+(\S*)\s+.*', re.IGNORECASE)
     GuiName2 = re.compile(r'.*\\Format\s+\S+\s+\S+\s+([^"]\S+)\s+(\S*)\s+.*', re.IGNORECASE)
-    input = open(input_files[0])
+    input = io.open(input_files[0], encoding='utf_8')
     for lineno, line in enumerate(input.readlines()):
         label = ""
         labelsc = ""
@@ -564,31 +570,31 @@ def formats_l10n(input_files, output, base):
         if shortcut != "":
             labelsc = label + "|" + shortcut
         if label != "":
-            print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                (relativePath(input_files[0], base), lineno+1, label)
+            print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                (relativePath(input_files[0], base), lineno+1, label), file=output)
         if labelsc != "":
-            print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                (relativePath(input_files[0], base), lineno+1, labelsc)
+            print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                (relativePath(input_files[0], base), lineno+1, labelsc), file=output)
     input.close()
     output.close()
 
 
 def encodings_l10n(input_files, output, base):
     '''Generate pot file from lib/encodings'''
-    output = open(output, 'w')
+    output = io.open(output, 'w', encoding='utf_8')
     # assuming only one encodings file
     #                 Encoding utf8      utf8    "Unicode (utf8)" UTF-8    variable inputenc
     reg = re.compile('Encoding [\w-]+\s+[\w-]+\s+"([\w \-\(\)]+)"\s+[\w-]+\s+(fixed|variable|variableunsafe)\s+\w+.*')
-    input = open(input_files[0])
+    input = io.open(input_files[0], encoding='utf_8')
     for lineno, line in enumerate(input.readlines()):
         if not line.startswith('Encoding'):
             continue
         if reg.match(line):
-            print >> output, '#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
-                (relativePath(input_files[0], base), lineno+1, reg.match(line).groups()[0])
+            print(u'#: %s:%d\nmsgid "%s"\nmsgstr ""\n' % \
+                (relativePath(input_files[0], base), lineno+1, reg.match(line).groups()[0]), file=output)
         else:
-            print "Error: Unable to handle line:"
-            print line
+            print("Error: Unable to handle line:")
+            print(line)
             # No need to abort if the parsing fails
             # sys.exit(1)
     input.close()
@@ -628,7 +634,7 @@ if __name__ == '__main__':
         ['help', 'type=', 'output=', 'base=', 'src_file='])
     for (opt, value) in optlist:
         if opt in ['-h', '--help']:
-            print Usage
+            print(Usage)
             sys.exit(0)
         elif opt in ['-o', '--output']:
             output = value
@@ -637,10 +643,10 @@ if __name__ == '__main__':
         elif opt in ['-t', '--type']:
             input_type = value
         elif opt in ['-s', '--src_file']:
-            input_files = [f.strip() for f in open(value)]
+            input_files = [f.strip() for f in io.open(value, encoding='utf_8')]
 
     if input_type not in ['ui', 'layouts', 'layouttranslations', 'qt4', 'languages', 'latexfonts', 'encodings', 'external', 'formats'] or output is None:
-        print 'Wrong input type or output filename.'
+        print('Wrong input type or output filename.')
         sys.exit(1)
 
     input_files += args
