@@ -466,13 +466,21 @@ docstring parseOptions(docstring const & format, string & optkey,
 
 } // anon namespace
 
-
+/* FIXME
+Bug #9131 revealed an oddity in how we are generating citation information
+when more than one key is given. We end up building a longer and longer format 
+string as we go, which we then have to re-parse, over and over and over again,
+rather than generating the information for the individual keys and then putting
+all of that together. We do that to deal with the way separators work, from what
+I can tell, but it still feels like a hack. Fixing this would require quite a
+bit of work, however.
+*/
 docstring BibTeXInfo::expandFormat(docstring const & format,
 		BibTeXInfo const * const xref, int & counter, Buffer const & buf,
 		docstring before, docstring after, docstring dialog, bool next) const
 {
 	// incorrect use of macros could put us in an infinite loop
-	static int max_passes = 5000;
+	static int const max_passes = 5000;
 	// the use of overly large keys can lead to performance problems, due
 	// to eventual attempts to convert LaTeX macros to unicode. See bug
 	// #8944. This is perhaps not the best solution, but it will have to
@@ -628,19 +636,24 @@ docstring const & BibTeXInfo::getInfo(BibTeXInfo const * const xref,
 	info_ = expandFormat(format, xref, counter, buf,
 		docstring(), docstring(), docstring(), false);
 
-	if (!info_.empty()) {
-		info_richtext_ = convertLaTeXCommands(processRichtext(info_, true));
-		info_ = convertLaTeXCommands(processRichtext(info_, false));
-		if (richtext)
-			return info_richtext_;
+	if (info_.empty()) {
+		// this probably shouldn't happen
+		return info_;
 	}
+
+	if (richtext) {
+		info_richtext_ = convertLaTeXCommands(processRichtext(info_, true));
+		return info_richtext_;
+	}
+
+	info_ = convertLaTeXCommands(processRichtext(info_, false));
 	return info_;
 }
 
 
 docstring const BibTeXInfo::getLabel(BibTeXInfo const * const xref,
 	Buffer const & buf, docstring const & format, bool richtext,
-	docstring const & before, docstring const & after,
+	docstring const & before, docstring const & after, 
 	docstring const & dialog, bool next) const
 {
 	docstring loclabel;
@@ -743,8 +756,7 @@ docstring BibTeXInfo::getValueForKey(string const & oldkey, Buffer const & buf,
 		ret = html::cleanAttr(ret);
 
 	// make sure it is not too big
-	if (ret.size() > maxsize)
-		ret = ret.substr(0, maxsize - 3) + from_ascii("...");
+	support::truncateWithEllipsis(ret, maxsize);
 	return ret;
 }
 
@@ -915,14 +927,9 @@ docstring const BiblioInfo::getLabel(vector<docstring> keys,
 			before, after, dialog, key + 1 != ken);
 	}
 
-	if (ret.size() > max_size) {
-		ret.resize(max_size - 3);
-		ret += "...";
-	} else if (too_many_keys) {
-		if (ret.size() > max_size - 3)
-			ret.resize(max_size - 3);
-		ret += "...";
-	}
+	if (too_many_keys)
+		ret.push_back(0x2026);//HORIZONTAL ELLIPSIS
+	support::truncateWithEllipsis(ret, max_size);
 	return ret;
 }
 
@@ -992,9 +999,9 @@ void BiblioInfo::collectCitedEntries(Buffer const & buf)
 	// FIXME We may want to collect these differently, in the first case,
 	// so that we might have them in order of appearance.
 	set<docstring> citekeys;
-	Toc const & toc = buf.tocBackend().toc("citation");
-	Toc::const_iterator it = toc.begin();
-	Toc::const_iterator const en = toc.end();
+	shared_ptr<Toc const> toc = buf.tocBackend().toc("citation");
+	Toc::const_iterator it = toc->begin();
+	Toc::const_iterator const en = toc->end();
 	for (; it != en; ++it) {
 		if (it->str().empty())
 			continue;
