@@ -66,6 +66,7 @@ class WorkAreaManager;
 }
 
 namespace support {
+class DocFileName;
 class FileName;
 class FileNameList;
 }
@@ -134,7 +135,10 @@ public:
 		ExportTexPathHasSpaces,
 		ExportConverterError,
 		// preview
+		// Implies ExportSuccess.
 		PreviewSuccess,
+		// The exported file exists but there was an error when opening
+		// it in a viewer.
 		PreviewError
 	};
 
@@ -236,6 +240,8 @@ public:
 	/// read the header, returns number of unknown tokens
 	int readHeader(Lexer & lex);
 
+	double fontScalingFactor() const;
+
 private:
 	///
 	typedef std::map<Buffer const *, Buffer *> BufferMap;
@@ -262,6 +268,8 @@ private:
 	/// of the temporary file to be read
 	ReadStatus convertLyXFormat(support::FileName const & fn,
 		support::FileName & tmpfile, int from_format);
+	/// get appropriate name for backing up files from older versions
+	support::FileName getBackupName() const;
 	//@}
 
 public:
@@ -398,6 +406,27 @@ public:
 	/// It is always an absolute path.
 	std::string filePath() const;
 
+	/** Contructs a file name of a referenced file (child doc, included graphics etc).
+	 *  Absolute names are returned as is. If the name is relative, it is
+	 *  interpreted relative to filePath() if the file exists, otherwise
+	 *  relative to the original path where the document was last saved.
+	 *  The original path may be different from filePath() if the document was
+	 *  later manually moved to a different location.
+	 */
+	support::DocFileName getReferencedFileName(std::string const & fn) const;
+
+	/** Returns the path where a local layout file lives.
+	 *  An empty string is returned for standard system and user layouts.
+	 *  If possible, it is always relative to the buffer path.
+	 */
+	std::string layoutPos() const;
+
+	/** Set the path to a local layout file.
+	 *  This must be an absolute path but, if possible, it is always
+	 *  stored as relative to the buffer path.
+	 */
+	void setLayoutPos(std::string const & path);
+
 	/** A transformed version of the file name, adequate for LaTeX.
 	    \param no_path optional if \c true then the path is stripped.
 	*/
@@ -594,9 +623,10 @@ public:
 
 	/// get source code (latex/docbook) for some paragraphs, or all paragraphs
 	/// including preamble
-	void getSourceCode(odocstream & os, std::string const format,
-			   pit_type par_begin, pit_type par_end, OutputWhat output,
-			   bool master) const;
+	/// returns NULL if Id to Row conversion is unsupported
+	std::auto_ptr<TexRow> getSourceCode(odocstream & os,
+			std::string const & format, pit_type par_begin,
+			pit_type par_end, OutputWhat output, bool master) const;
 
 	/// Access to error list.
 	/// This method is used only for GUI visualisation of Buffer related
@@ -640,21 +670,22 @@ public:
 
 	///
 	ExportStatus doExport(std::string const & target, bool put_in_tempdir) const;
+	/// Export buffer to format \p format and open the result in a suitable viewer.
+	/// Note: This has nothing to do with preview of graphics or math formulas.
+	ExportStatus preview(std::string const & format) const;
+	/// true if there was a previous preview this session of this buffer and
+	/// there was an error on the previous preview of this buffer.
+	bool lastPreviewError() const;
+
+private:
 	///
 	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
 		std::string & result_file) const;
-	///
-	ExportStatus preview(std::string const & format) const;
-
-private:
 	/// target is a format name optionally followed by a space
 	/// and a destination file-name
 	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
 		bool includeall, std::string & result_file) const;
 	///
-	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
-		bool includeall) const;
-	///
 	ExportStatus preview(std::string const & format, bool includeall = false) const;
 	///
 	void setMathFlavor(OutputParams & op) const;
@@ -713,6 +744,9 @@ public:
 	/// In all other cases, this is a no-op and name is returned unchanged.
 	/// If a non-empty ext is given, the existence of name.ext is checked
 	/// but the returned path will not contain this extension.
+	/// Similarly, when loading a document that was moved from the location
+	/// where it was saved, return the correct path relative to the new
+	/// location.
 	std::string includedFilePath(std::string const & name,
 				std::string const & ext = empty_string()) const;
 
