@@ -61,6 +61,7 @@
 #include "support/limited_stack.h"
 #include "support/lstrings.h"
 #include "support/lyxalgo.h"
+#include "support/TempFile.h"
 
 #include "frontends/alert.h"
 #include "frontends/Clipboard.h"
@@ -117,22 +118,23 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 	pit_type pit = cur.pit();
 	pos_type pos = cur.pos();
 	bool need_update = false;
-	InsetText * target_inset = cur.inset().asInsetText();
-	if (!target_inset) {
-		InsetTabular * it = cur.inset().asInsetTabular();
-		target_inset = it? it->cell(cur.idx())->asInsetText() : 0;
-	}
-	LASSERT(target_inset, return PasteReturnValue(pit, pos, need_update));
-	ParagraphList & pars = target_inset->paragraphs();
 
 	if (parlist.empty())
 		return PasteReturnValue(pit, pos, need_update);
 
-	BOOST_ASSERT (pos <= pars[pit].size());
+	InsetText * target_inset = cur.inset().asInsetText();
+	if (!target_inset) {
+		InsetTabular * it = cur.inset().asInsetTabular();
+		target_inset = it ? it->cell(cur.idx())->asInsetText() : 0;
+	}
+	LASSERT(target_inset, return PasteReturnValue(pit, pos, need_update));
+
+	ParagraphList & pars = target_inset->paragraphs();
+	LASSERT(pos <= pars[pit].size(),
+			return PasteReturnValue(pit, pos, need_update));
 
 	// Make a copy of the CaP paragraphs.
 	ParagraphList insertion = parlist;
-	DocumentClassConstPtr newDocClass = buffer.params().documentClassPtr();
 
 	// Now remove all out of the pars which is NOT allowed in the
 	// new environment and set also another font if that is required.
@@ -156,6 +158,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 	}
 
 	// set the paragraphs to plain layout if necessary
+	DocumentClassConstPtr newDocClass = buffer.params().documentClassPtr();
 	if (cur.inset().usePlainLayout()) {
 		bool forcePlainLayout = cur.inset().forcePlainLayout();
 		Layout const & plainLayout = newDocClass->plainLayout();
@@ -167,7 +170,8 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 			if (forcePlainLayout || parLayout == defaultLayout)
 				par->setLayout(plainLayout);
 		}
-	} else { // check if we need to reset from plain layout
+	} else {
+		// check if we need to reset from plain layout
 		Layout const & defaultLayout = newDocClass->defaultLayout();
 		Layout const & plainLayout = newDocClass->plainLayout();
 		ParagraphList::iterator const end = insertion.end();
@@ -221,7 +225,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 					tmpbuf->eraseChar(i--, false);
 		}
 
-		tmpbuf->setChange(Change(buffer.params().trackChanges ?
+		tmpbuf->setChange(Change(buffer.params().track_changes ?
 					 Change::INSERTED : Change::UNCHANGED));
 	}
 
@@ -423,7 +427,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 			pars[last_paste].makeSameLayout(pars[last_paste + 1]);
 			mergeParagraph(buffer.params(), pars, last_paste);
 		} else {
-			pars[last_paste + 1].stripLeadingSpaces(buffer.params().trackChanges);
+			pars[last_paste + 1].stripLeadingSpaces(buffer.params().track_changes);
 			++last_paste;
 		}
 	}
@@ -444,17 +448,17 @@ PitPosPair eraseSelectionHelper(BufferParams const & params,
 
 	// Start and end is inside same paragraph
 	if (endpit == pit_type(pars.size()) || startpit == endpit) {
-		endpos -= pars[startpit].eraseChars(startpos, endpos, params.trackChanges);
+		endpos -= pars[startpit].eraseChars(startpos, endpos, params.track_changes);
 		return PitPosPair(endpit, endpos);
 	}
 
 	for (pit_type pit = startpit; pit != endpit + 1;) {
 		pos_type const left  = (pit == startpit ? startpos : 0);
 		pos_type right = (pit == endpit ? endpos : pars[pit].size() + 1);
-		bool const merge = pars[pit].isMergedOnEndOfParDeletion(params.trackChanges);
+		bool const merge = pars[pit].isMergedOnEndOfParDeletion(params.track_changes);
 
 		// Logically erase only, including the end-of-paragraph character
-		pars[pit].eraseChars(left, right, params.trackChanges);
+		pars[pit].eraseChars(left, right, params.track_changes);
 
 		// Separate handling of paragraph break:
 		if (merge && pit != endpit &&
@@ -482,8 +486,11 @@ Buffer * copyToTempBuffer(ParagraphList const & paragraphs, DocumentClassConstPt
 	// to be so, but the alternative is to construct a new one of these (with a
 	// new temporary directory, etc) every time, and then to destroy it. So maybe
 	// it's worth just keeping this one around.
+	// FIXME THREAD
+	static TempFile tempfile("clipboard.internal");
+	tempfile.setAutoRemove(false);
 	static Buffer * staticbuffer = theBufferList().newInternalBuffer(
-		FileName::tempName("clipboard.internal").absFileName());
+			tempfile.name().absFileName());
 
 	// These two things only really need doing the first time.
 	staticbuffer->setUnnamed(true);
@@ -804,22 +811,20 @@ vector<docstring> availableSelections(Buffer const * buf)
 		// we do not use cit-> here because gcc 2.9x does not
 		// like it (JMarc)
 		ParagraphList const & pars = (*cit).first;
-		docstring asciiSel;
+		docstring textSel;
 		ParagraphList::const_iterator pit = pars.begin();
 		ParagraphList::const_iterator pend = pars.end();
 		for (; pit != pend; ++pit) {
-			Paragraph par(*pit, 0, 26);
+			Paragraph par(*pit, 0, 46);
 			// adapt paragraph to current buffer.
 			par.setBuffer(const_cast<Buffer &>(*buf));
-			asciiSel += par.asString(AS_STR_INSETS);
-			if (asciiSel.size() > 25) {
-				asciiSel.replace(22, docstring::npos,
-						 from_ascii("..."));
+			textSel += par.asString(AS_STR_INSETS);
+			if (textSel.size() > 45) {
+				support::truncateWithEllipsis(textSel,45);
 				break;
 			}
 		}
-
-		selList.push_back(asciiSel);
+		selList.push_back(textSel);
 	}
 
 	return selList;
@@ -884,7 +889,7 @@ void cutSelection(Cursor & cur, bool doclear, bool realcut)
 
 		// sometimes necessary
 		if (doclear
-			&& text->paragraphs()[begpit].stripLeadingSpaces(bp.trackChanges))
+			&& text->paragraphs()[begpit].stripLeadingSpaces(bp.track_changes))
 			cur.fixIfBroken();
 
 		// need a valid cursor. (Lgb)
@@ -1255,7 +1260,7 @@ void replaceSelectionWithString(Cursor & cur, docstring const & str)
 	docstring::const_iterator cit = str.begin();
 	docstring::const_iterator end = str.end();
 	for (; cit != end; ++cit, ++pos)
-		par.insertChar(pos, *cit, font, cur.buffer()->params().trackChanges);
+		par.insertChar(pos, *cit, font, cur.buffer()->params().track_changes);
 
 	// Cut the selection
 	cutSelection(cur, true, false);
