@@ -24,22 +24,20 @@ using namespace std;
 namespace lyx {
 
 
-Floating::Floating()
-{}
-
-
 Floating::Floating(string const & type, string const & placement,
 		   string const & ext, string const & within,
 		   string const & style, string const & name,
-		   string const & listName, std::string const & listCmd, 
-		   string const & refPrefix,
-		   string const & htmlTag, string const & htmlAttrib, 
-		   string const & htmlStyle, bool usesfloat, bool ispredefined)
+		   string const & listName, std::string const & listCmd,
+		   string const & refPrefix, std::string const & allowedplacement,
+		   string const & htmlTag, string const & htmlAttrib,
+		   string const & htmlStyle, bool usesfloat, bool ispredefined,
+		   bool allowswide, bool allowssideways)
 	: floattype_(type), placement_(placement), ext_(ext), within_(within),
 	  style_(style), name_(name), listname_(listName), listcommand_(listCmd),
-	  refprefix_(refPrefix), usesfloatpkg_(usesfloat),
-	  ispredefined_(ispredefined), html_tag_(htmlTag),
-	  html_attrib_(htmlAttrib), html_style_(htmlStyle)
+	  refprefix_(refPrefix), allowedplacement_(allowedplacement),
+	  usesfloatpkg_(usesfloat), ispredefined_(ispredefined),
+	  allowswide_(allowswide), allowssideways_(allowssideways),
+	  html_tag_(htmlTag), html_attrib_(htmlAttrib), html_style_(htmlStyle)
 {}
 
 
