@@ -80,7 +80,6 @@ public:
 		RC_EXAMPLEPATH,
 		RC_EXPORT_OVERWRITE,
 		RC_FONT_ENCODING,
-		RC_FORCE_PAINT_SINGLE_CHAR,
 		RC_FILEFORMAT,
 		RC_FORWARD_SEARCH_DVI,
 		RC_FORWARD_SEARCH_PDF,
@@ -120,6 +119,7 @@ public:
 		RC_MAKE_BACKUP,
 		RC_MARK_FOREIGN_LANGUAGE,
 		RC_MOUSE_WHEEL_SPEED,
+		RC_MOUSE_MIDDLEBUTTON_PASTE,
 		RC_NOMENCL_COMMAND,
 		RC_NUMLASTFILES,
 		RC_OPEN_BUFFERS_IN_TABS,
@@ -129,26 +129,11 @@ public:
 		RC_PREVIEW,
 		RC_PREVIEW_HASHED_LABELS,
 		RC_PREVIEW_SCALE_FACTOR,
-		RC_PRINTCOLLCOPIESFLAG,
-		RC_PRINTCOPIESFLAG,
-		RC_PRINTER,
-		RC_PRINTEVENPAGEFLAG,
-		RC_PRINTEXSTRAOPTIONS,
-		RC_PRINTFILEEXTENSION,
 		RC_PRINTLANDSCAPEFLAG,
-		RC_PRINTODDPAGEFLAG,
-		RC_PRINTPAGERANGEFLAG,
 		RC_PRINTPAPERDIMENSIONFLAG,
 		RC_PRINTPAPERFLAG,
-		RC_PRINTREVERSEFLAG,
-		RC_PRINTSPOOL_COMMAND,
-		RC_PRINTSPOOL_PRINTERPREFIX,
-		RC_PRINTTOFILE,
-		RC_PRINTTOPRINTER,
-		RC_PRINT_ADAPTOUTPUT,
-		RC_PRINT_COMMAND,
-		RC_RTL_SUPPORT,
 		RC_SAVE_COMPRESSED,
+		RC_SAVE_ORIGIN,
 		RC_SCREEN_DPI,
 		RC_SCREEN_FONT_ROMAN,
 		RC_SCREEN_FONT_ROMAN_FOUNDRY,
@@ -237,38 +222,8 @@ public:
 	///
 	std::string ui_file;
 	///
-	std::string printer;
-	///
-	std::string print_command;
-	///
-	std::string print_evenpage_flag;
-	///
-	std::string print_oddpage_flag;
-	///
-	std::string print_pagerange_flag;
-	///
-	std::string print_copies_flag;
-	///
-	std::string print_collcopies_flag;
-	///
-	std::string print_reverse_flag;
-	///
 	std::string print_landscape_flag;
 	///
-	std::string print_to_printer;
-	///
-	bool print_adapt_output;
-	///
-	std::string print_to_file;
-	///
-	std::string print_file_extension;
-	///
-	std::string print_extra_options;
-	///
-	std::string print_spool_command;
-	///
-	std::string print_spool_printerprefix;
-	///
 	std::string print_paper_flag;
 	///
 	std::string print_paper_dimension_flag;
@@ -307,6 +262,8 @@ public:
 	std::string hunspelldir_path;
 	///
 	bool auto_region_delete;
+	/// enable middle-mouse-button paste
+	bool mouse_middlebutton_paste;
 	/// flag telling whether lastfiles should be checked for existance
 	bool auto_reset_options;
 	///
@@ -319,6 +276,8 @@ public:
 	bool load_session;
 	/// do we save new documents as compressed by default
 	bool save_compressed;
+	/// whether or not to save the document dir in the file
+	bool save_origin;
 	/// shall a backup file be created
 	bool make_backup;
 	/// A directory for storing backup files
@@ -411,8 +370,6 @@ public:
 	};
 	///
 	LangPackageSelection language_package_selection;
-	///
-	bool rtl_support;
 	/// bidi cursor movement: true = visual, false = logical
 	bool visual_cursor;
 	///
@@ -552,25 +509,12 @@ public:
 	///
 	ScrollWheelZoom scroll_wheel_zoom;
 	///
-	bool force_paint_single_char;
-	///
 	int cursor_width;
 	/// One of: yes, no, ask
 	std::string close_buffer_with_last_view;
 };
 
 
-/** \c LyXRC_PreviewStatus is a wrapper for LyXRC::PreviewStatus.
- *  It can be forward-declared and passed as a function argument without
- *  having to expose LyXRC.h.
- */
-class LyXRC_PreviewStatus {
-	LyXRC::PreviewStatus val_;
-public:
-	LyXRC_PreviewStatus(LyXRC::PreviewStatus val) : val_(val) {}
-	operator LyXRC::PreviewStatus() const { return val_; }
-};
-
 void actOnUpdatedPrefs(LyXRC const & lyxrc_orig, LyXRC const & lyxrc_new);
 
 ///
