@@ -49,7 +49,6 @@ class MetricsInfo;
 class OutputParams;
 class PainterInfo;
 class ParagraphParameters;
-class TexRow;
 class Toc;
 class WordLangTuple;
 class XHTMLStream;
@@ -76,7 +75,7 @@ public:
 		return first == s.first && last == s.last;
 	}
 
-	inline bool inside(pos_type p) const
+	inline bool contains(pos_type p) const
 	{
 		return first <= p && p <= last;
 	}
@@ -90,15 +89,15 @@ public:
 	inline FontSpan intersect(FontSpan const & f) const
 	{
 		FontSpan result = FontSpan();
-		if (inside(f.first))
+		if (contains(f.first))
 			result.first = f.first;
-		else if (f.inside(first))
+		else if (f.contains(first))
 			result.first = first;
 		else
 			return result;
-		if (inside(f.last))
+		if (contains(f.last))
 			result.last = f.last;
-		else if (f.inside(last))
+		else if (f.contains(last))
 			result.last = last;
 		return result;
 	}
@@ -127,7 +126,8 @@ enum AsStringParameter
 	AS_STR_LABEL = 1, ///< Prefix with paragraph label.
 	AS_STR_INSETS = 2, ///< Go into insets.
 	AS_STR_NEWLINES = 4, ///< Get also newline characters.
-	AS_STR_SKIPDELETE = 8 ///< Skip deleted text in change tracking.
+	AS_STR_SKIPDELETE = 8, ///< Skip deleted text in change tracking.
+	AS_STR_PLAINTEXT = 16 ///< Don't export formatting when descending into insets.
 };
 
 
@@ -173,16 +173,15 @@ public:
 	///		asString(AS_STR_LABEL | AS_STR_INSETS)
 	///		asString(AS_STR_INSETS)
 	docstring asString(int options = AS_STR_NONE) const;
-	///
-	docstring asString(pos_type beg, pos_type end,
-		int options = AS_STR_NONE) const;
-	///
-	void forToc(docstring &, size_t maxlen) const;
 
-	/// Extract only the explicitly visible text (without any formatting),
-	/// descending into insets
-	docstring stringify(pos_type beg, pos_type end, int options,
-		OutputParams const & runparams) const;
+	/// Convert the paragraph to a string.
+	/// \note If options includes AS_STR_PLAINTEXT, then runparams must be != 0
+	docstring asString(pos_type beg, pos_type end,
+			   int options = AS_STR_NONE,
+			   const OutputParams *runparams = 0) const;
+	///
+	void forOutliner(docstring &, size_t const maxlen,
+					 bool const shorten = true) const;
 
 	///
 	void write(std::ostream &, BufferParams const &,
@@ -405,6 +404,8 @@ public:
 	bool isInset(pos_type pos) const;
 	///
 	bool isNewline(pos_type pos) const;
+	///
+	bool isEnvSeparator(pos_type pos) const;
 	/// return true if the char is a word separator
 	bool isSeparator(pos_type pos) const;
 	///
@@ -430,8 +431,6 @@ public:
 	/// return true if we allow this par to stay empty
 	bool allowEmpty() const;
 	///
-	char_type transformChar(char_type c, pos_type pos) const;
-	///
 	ParagraphParameters & params();
 	///
 	ParagraphParameters const & params() const;
@@ -484,10 +483,9 @@ public:
 	/// \return true if one of the tested positions is misspelled.
 	bool isMisspelled(pos_type pos, bool check_boundary = false) const;
 
-	/// \return true if both positions are inside the same
-	/// spell range - i.e. the same word.
-	/// use it for positions inside misspelled range only.
-	bool isSameSpellRange(pos_type pos1, pos_type pos2) const;
+	/// \return the spell range (misspelled area) around position.
+	/// Range is empty if word at position is correctly spelled.
+	FontSpan const & getSpellRange(pos_type pos) const;
 
 	/// spell check of whole paragraph
 	/// remember results until call of requestSpellCheck()
