@@ -92,19 +92,23 @@ size_t ParagraphMetrics::computeRowSignature(Row const & row,
 	for (pos_type i = row.pos(); i < row.endpos(); ++i) {
 		char_type const b[] = { par_->getChar(i) };
 		crc.process_bytes(b, sizeof(char_type));
-		if (bparams.trackChanges) {
+		if (bparams.track_changes) {
 			Change change = par_->lookupChange(i);
 			char_type const b[] = { static_cast<char_type>(change.type) };
 			// 1 byte is enough to encode Change::Type
 			crc.process_bytes(b, 1);
-		}			
+		}
 	}
 
+	pos_type const b1[] = { row.sel_beg, row.sel_end };
+	crc.process_bytes(b1, sizeof(b1));
+
 	Dimension const & d = row.dimension();
-	char_type const b[] = { static_cast<char_type>(row.sel_beg),
-				static_cast<char_type>(row.sel_end),
-		row.begin_margin_sel, row.end_margin_sel, d.wid, d.asc, d.des};
-	crc.process_bytes(b, sizeof(b));
+	int const b2[] = { row.begin_margin_sel,
+	                   row.end_margin_sel,
+	                   d.wid, d.asc, d.des };
+	crc.process_bytes(b2, sizeof(b2));
+	crc.process_bytes(&row.separator, sizeof(row.separator));
 
 	return crc.checksum();
 }
@@ -116,24 +120,6 @@ void ParagraphMetrics::setPosition(int position)
 }
 
 
-Dimension const & ParagraphMetrics::insetDimension(Inset const * inset) const
-{
-	InsetDims::const_iterator it = inset_dims_.find(inset);
-	if (it != inset_dims_.end())
-		return it->second;
-
-	static Dimension dummy;
-	return dummy;
-}
-
-
-void ParagraphMetrics::setInsetDimension(Inset const * inset,
-		Dimension const & dim)
-{
-	inset_dims_[inset] = dim;
-}
-
-
 Row & ParagraphMetrics::getRow(pos_type pos, bool boundary)
 {
 	LBUFERR(!rows().empty());
@@ -190,8 +176,7 @@ void ParagraphMetrics::dump() const
 {
 	lyxerr << "Paragraph::dump: rows.size(): " << rows_.size() << endl;
 	for (size_t i = 0; i != rows_.size(); ++i) {
-		lyxerr << "  row " << i << ":   ";
-		rows_[i].dump();
+		lyxerr << "  row " << i << ":   " << rows_[i];
 	}
 }
 
@@ -210,37 +195,7 @@ int ParagraphMetrics::rightMargin(BufferView const & bv) const
 }
 
 
-int ParagraphMetrics::singleWidth(pos_type pos, Font const & font) const
-{
-	// The most special cases are handled first.
-	if (Inset const * inset = par_->getInset(pos))
-		return insetDimension(inset).wid;
-
-	char_type c = par_->getChar(pos);
-
-	if (c == '\t')
-		return 4 * theFontMetrics(font).width(' ');
-	
-	if (!isPrintable(c))
-		return theFontMetrics(font).width(c);
-
-	Language const * language = font.language();
-	if (language->rightToLeft()) {
-		if (language->lang() == "arabic_arabtex" ||
-			language->lang() == "arabic_arabi" ||
-			language->lang() == "farsi") {
-				if (Encodings::isArabicComposeChar(c))
-					return 0;
-				c = par_->transformChar(c, pos);
-		} else if (language->lang() == "hebrew" &&
-				Encodings::isHebrewComposeChar(c)) {
-			return 0;
-		}
-	}
-	return theFontMetrics(font).width(c);
-}
-
-
+// FIXME: this code seems bogus. Audit and rewrite (see bug #9860).
 bool ParagraphMetrics::hfillExpansion(Row const & row, pos_type pos) const
 {
 	if (!par_->isHfill(pos))
@@ -269,7 +224,7 @@ bool ParagraphMetrics::hfillExpansion(Row const & row, pos_type pos) const
 	// the specified position that is neither a newline nor an hfill,
 	// the hfill will be expanded, otherwise it won't
 	for (pos_type i = row.pos(); i < pos; i++) {
-		if (!par_->isNewline(i) && !par_->isHfill(i))
+		if (!par_->isNewline(i) && !par_->isEnvSeparator(i) && !par_->isHfill(i))
 			return true;
 	}
 	return false;
