@@ -69,7 +69,6 @@
 #include "support/lyxalgo.h"
 #include "support/textutils.h"
 
-#include <limits>
 #include <sstream>
 
 
@@ -156,7 +155,7 @@ void breakParagraphConservative(BufferParams const & bparams,
 		}
 		// Move over the end-of-par change information
 		tmp.setChange(tmp.size(), par.lookupChange(par.size()));
-		par.setChange(par.size(), Change(bparams.trackChanges ?
+		par.setChange(par.size(), Change(bparams.track_changes ?
 					   Change::INSERTED : Change::UNCHANGED));
 	}
 }
@@ -197,7 +196,7 @@ void mergeParagraph(BufferParams const & bparams,
 
 
 Text::Text(InsetText * owner, bool use_default_layout)
-	: owner_(owner), autoBreakRows_(false), undo_counter_(0)
+	: owner_(owner)
 {
 	pars_.push_back(Paragraph());
 	Paragraph & par = pars_.back();
@@ -211,7 +210,7 @@ Text::Text(InsetText * owner, bool use_default_layout)
 
 
 Text::Text(InsetText * owner, Text const & text)
-	: owner_(owner), autoBreakRows_(text.autoBreakRows_), undo_counter_(0)
+	: owner_(owner)
 {
 	pars_ = text.pars_;
 	ParagraphList::iterator const end = pars_.end();
@@ -361,7 +360,7 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 	string const & token, Font & font, Change & change, ErrorList & errorList)
 {
 	Buffer * buf = const_cast<Buffer *>(&owner_->buffer());
-	BufferParams const & bp = buf->params();
+	BufferParams & bp = buf->params();
 
 	if (token[0] != '\\') {
 		docstring dstr = lex.getDocString();
@@ -485,18 +484,42 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 	} else if (token == "\\color") {
 		lex.next();
 		setLyXColor(lex.getString(), font.fontInfo());
-	} else if (token == "\\SpecialChar") {
+	} else if (token == "\\SpecialChar" ||
+	           (token == "\\SpecialCharNoPassThru" &&
+	            !par.layout().pass_thru && !inset().isPassThru())) {
 		auto_ptr<Inset> inset;
 		inset.reset(new InsetSpecialChar);
 		inset->read(lex);
 		inset->setBuffer(*buf);
 		par.insertInset(par.size(), inset.release(), font, change);
+	} else if (token == "\\SpecialCharNoPassThru") {
+		lex.next();
+		docstring const s = ltrim(lex.getDocString(), "\\");
+		par.insert(par.size(), s, font, change);
 	} else if (token == "\\IPAChar") {
 		auto_ptr<Inset> inset;
 		inset.reset(new InsetIPAChar);
 		inset->read(lex);
 		inset->setBuffer(*buf);
 		par.insertInset(par.size(), inset.release(), font, change);
+	} else if (token == "\\twohyphens" || token == "\\threehyphens") {
+		// Ideally, this should be done by lyx2lyx, but lyx2lyx does not know the
+		// running font and does not know anything about layouts (and CopyStyle).
+		Layout const & layout(par.layout());
+		FontInfo info = font.fontInfo();
+		info.realize(layout.resfont);
+		if (layout.pass_thru || inset().isPassThru() ||
+		    info.family() == TYPEWRITER_FAMILY) {
+			if (token == "\\twohyphens")
+				par.insert(par.size(), from_ascii("--"), font, change);
+			else
+				par.insert(par.size(), from_ascii("---"), font, change);
+		} else {
+			if (token == "\\twohyphens")
+				par.insertChar(par.size(), 0x2013, font, change);
+			else
+				par.insertChar(par.size(), 0x2014, font, change);
+		}
 	} else if (token == "\\backslash") {
 		par.appendChar('\\', font, change);
 	} else if (token == "\\LyXTable") {
@@ -511,18 +534,25 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 		int aid;
 		time_t ct;
 		is >> aid >> ct;
-		BufferParams::AuthorMap const & am = bp.author_map;
+		BufferParams::AuthorMap const & am = bp.author_map_;
 		if (am.find(aid) == am.end()) {
-			errorList.push_back(ErrorItem(_("Change tracking error"),
-					    bformat(_("Unknown author index for change: %1$d\n"), aid),
-					    par.id(), 0, par.size()));
-			change = Change(Change::UNCHANGED);
-		} else {
-			if (token == "\\change_inserted")
-				change = Change(Change::INSERTED, am.find(aid)->second, ct);
-			else
-				change = Change(Change::DELETED, am.find(aid)->second, ct);
+			errorList.push_back(ErrorItem(
+				_("Change tracking author index missing"),
+				bformat(_("A change tracking author information for index "
+				          "%1$d is missing. This can happen after a wrong "
+				          "merge by a version control system. In this case, "
+				          "either fix the merge, or have this information "
+				          "missing until the corresponding tracked changes "
+				          "are merged or this user edits the file again.\n"),
+				        aid),
+				par.id(), par.size(), par.size() + 1
+				));
+			bp.addAuthor(Author(aid));
 		}
+		if (token == "\\change_inserted")
+			change = Change(Change::INSERTED, am.find(aid)->second, ct);
+		else
+			change = Change(Change::DELETED, am.find(aid)->second, ct);
 	} else {
 		lex.eatLine();
 		errorList.push_back(ErrorItem(_("Unknown token"),
@@ -690,7 +720,7 @@ static void breakParagraph(Text & text, pit_type par_offset, pos_type pos,
 
 	// Move over the end-of-par change information
 	tmp->setChange(tmp->size(), par.lookupChange(par.size()));
-	par.setChange(par.size(), Change(bparams.trackChanges ?
+	par.setChange(par.size(), Change(bparams.track_changes ?
 					   Change::INSERTED : Change::UNCHANGED));
 
 	if (pos) {
@@ -748,7 +778,7 @@ void Text::breakParagraph(Cursor & cur, bool inverse_logic)
 	// Always break behind a space
 	// It is better to erase the space (Dekel)
 	if (cur.pos() != cur.lastpos() && cpar.isLineSeparator(cur.pos()))
-		cpar.eraseChar(cur.pos(), cur.buffer()->params().trackChanges);
+		cpar.eraseChar(cur.pos(), cur.buffer()->params().track_changes);
 
 	// What should the layout for the new paragraph be?
 	bool keep_layout = layout.isEnvironment()
@@ -783,7 +813,7 @@ void Text::breakParagraph(Cursor & cur, bool inverse_logic)
 	}
 
 	while (!pars_[next_par].empty() && pars_[next_par].isNewline(0)) {
-		if (!pars_[next_par].eraseChar(0, cur.buffer()->params().trackChanges))
+		if (!pars_[next_par].eraseChar(0, cur.buffer()->params().track_changes))
 			break; // the character couldn't be deleted physically due to change tracking
 	}
 
@@ -814,7 +844,7 @@ void Text::insertStringAsLines(Cursor & cur, docstring const & str,
 	    cit != str.end(); ++cit) {
 		Paragraph & par = pars_[pit];
 		if (*cit == '\n') {
-			if (autoBreakRows_ && (!par.empty() || par.allowEmpty())) {
+			if (inset().allowMultiPar() && (!par.empty() || par.allowEmpty())) {
 				lyx::breakParagraph(*this, pit, pos,
 					par.layout().isEnvironment());
 				++pit;
@@ -823,18 +853,18 @@ void Text::insertStringAsLines(Cursor & cur, docstring const & str,
 			} else {
 				continue;
 			}
-			// do not insert consecutive spaces if !free_spacing
+		// do not insert consecutive spaces if !free_spacing
 		} else if ((*cit == ' ' || *cit == '\t') &&
 			   space_inserted && !par.isFreeSpacing()) {
 			continue;
 		} else if (*cit == '\t') {
 			if (!par.isFreeSpacing()) {
 				// tabs are like spaces here
-				par.insertChar(pos, ' ', font, bparams.trackChanges);
+				par.insertChar(pos, ' ', font, bparams.track_changes);
 				++pos;
 				space_inserted = true;
 			} else {
-				par.insertChar(pos, *cit, font, bparams.trackChanges);
+				par.insertChar(pos, *cit, font, bparams.track_changes);
 				++pos;
 				space_inserted = true;
 			}
@@ -843,7 +873,7 @@ void Text::insertStringAsLines(Cursor & cur, docstring const & str,
 			continue;
 		} else {
 			// just insert the character
-			par.insertChar(pos, *cit, font, bparams.trackChanges);
+			par.insertChar(pos, *cit, font, bparams.track_changes);
 			++pos;
 			space_inserted = (*cit == ' ');
 		}
@@ -920,6 +950,7 @@ void Text::insertChar(Cursor & cur, char_type c)
 				if (contains(number_unary_operators, c) &&
 				    (cur.pos() == 1
 				     || par.isSeparator(cur.pos() - 2)
+				     || par.isEnvSeparator(cur.pos() - 2)
 				     || par.isNewline(cur.pos() - 2))
 				  ) {
 					setCharFont(pit, cur.pos() - 1, cur.current_font,
@@ -1017,14 +1048,36 @@ void Text::insertChar(Cursor & cur, char_type c)
 		}
 	}
 
-	par.insertChar(cur.pos(), c, cur.current_font,
-		cur.buffer()->params().trackChanges);
+	pos_type pos = cur.pos();
+	if (!cur.paragraph().isPassThru() && owner_->lyxCode() != IPA_CODE &&
+	    cur.real_current_font.fontInfo().family() != TYPEWRITER_FAMILY &&
+	    c == '-' && pos > 0) {
+		if (par.getChar(pos - 1) == '-') {
+			// convert "--" to endash
+			par.eraseChar(pos - 1, cur.buffer()->params().track_changes);
+			c = 0x2013;
+			pos--;
+		} else if (par.getChar(pos - 1) == 0x2013) {
+			// convert "---" to emdash
+			par.eraseChar(pos - 1, cur.buffer()->params().track_changes);
+			c = 0x2014;
+			pos--;
+		} else if (par.getChar(pos - 1) == 0x2014) {
+			// convert "----" to "-"
+			par.eraseChar(pos - 1, cur.buffer()->params().track_changes);
+			c = '-';
+			pos--;
+		}
+	}
+
+	par.insertChar(pos, c, cur.current_font,
+		cur.buffer()->params().track_changes);
 	cur.checkBufferStructure();
 
 //		cur.screenUpdateFlags(Update::Force);
 	bool boundary = cur.boundary()
-		|| tm.isRTLBoundary(cur.pit(), cur.pos() + 1);
-	setCursor(cur, cur.pit(), cur.pos() + 1, false, boundary);
+		|| tm.isRTLBoundary(cur.pit(), pos + 1);
+	setCursor(cur, cur.pit(), pos + 1, false, boundary);
 	charInserted(cur);
 }
 
@@ -1033,15 +1086,6 @@ void Text::charInserted(Cursor & cur)
 {
 	Paragraph & par = cur.paragraph();
 
-	// Here we call finishUndo for every 20 characters inserted.
-	// This is from my experience how emacs does it. (Lgb)
-	if (undo_counter_ < 20) {
-		++undo_counter_;
-	} else {
-		cur.finishUndo();
-		undo_counter_ = 0;
-	}
-
 	// register word if a non-letter was entered
 	if (cur.pos() > 1
 	    && !par.isWordSeparator(cur.pos() - 2)
@@ -1076,7 +1120,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 	if (lyxrc.mac_like_cursor_movement) {
 		// Skip through trailing punctuation and spaces.
 		while (pos != lastpos && (par.isChar(pos) || par.isSpace(pos)))
-                        ++pos;
+			++pos;
 
 		// Skip over either a non-char inset or a full word
 		if (pos != lastpos && par.isWordSeparator(pos))
@@ -1149,7 +1193,6 @@ bool Text::cursorVisLeftOneWord(Cursor & cur)
 	LBUFERR(this == cur.text());
 
 	pos_type left_pos, right_pos;
-	bool left_is_letter, right_is_letter;
 
 	Cursor temp_cur = cur;
 
@@ -1158,9 +1201,9 @@ bool Text::cursorVisLeftOneWord(Cursor & cur)
 
 		// collect some information about current cursor position
 		temp_cur.getSurroundingPos(left_pos, right_pos);
-		left_is_letter =
+		bool left_is_letter =
 			(left_pos > -1 ? !temp_cur.paragraph().isWordSeparator(left_pos) : false);
-		right_is_letter =
+		bool right_is_letter =
 			(right_pos > -1 ? !temp_cur.paragraph().isWordSeparator(right_pos) : false);
 
 		// if we're not at a letter/non-letter boundary, continue moving
@@ -1186,7 +1229,6 @@ bool Text::cursorVisRightOneWord(Cursor & cur)
 	LBUFERR(this == cur.text());
 
 	pos_type left_pos, right_pos;
-	bool left_is_letter, right_is_letter;
 
 	Cursor temp_cur = cur;
 
@@ -1195,9 +1237,9 @@ bool Text::cursorVisRightOneWord(Cursor & cur)
 
 		// collect some information about current cursor position
 		temp_cur.getSurroundingPos(left_pos, right_pos);
-		left_is_letter =
+		bool left_is_letter =
 			(left_pos > -1 ? !temp_cur.paragraph().isWordSeparator(left_pos) : false);
-		right_is_letter =
+		bool right_is_letter =
 			(right_pos > -1 ? !temp_cur.paragraph().isWordSeparator(right_pos) : false);
 
 		// if we're not at a letter/non-letter boundary, continue moving
@@ -1224,7 +1266,7 @@ void Text::selectWord(Cursor & cur, word_location loc)
 {
 	LBUFERR(this == cur.text());
 	CursorSlice from = cur.top();
-	CursorSlice to = cur.top();
+	CursorSlice to;
 	getWord(from, to, loc);
 	if (cur.top() != from)
 		setCursor(cur, from.pit(), from.pos());
@@ -1365,7 +1407,7 @@ void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 	}
 
 	// finally, invoke the DEPM
-	deleteEmptyParagraphMechanism(begPit, endPit, cur.buffer()->params().trackChanges);
+	deleteEmptyParagraphMechanism(begPit, endPit, cur.buffer()->params().track_changes);
 
 	cur.finishUndo();
 	cur.clearSelection();
@@ -1379,7 +1421,7 @@ void Text::acceptChanges()
 {
 	BufferParams const & bparams = owner_->buffer().params();
 	lyx::acceptChanges(pars_, bparams);
-	deleteEmptyParagraphMechanism(0, pars_.size() - 1, bparams.trackChanges);
+	deleteEmptyParagraphMechanism(0, pars_.size() - 1, bparams.track_changes);
 }
 
 
@@ -1415,7 +1457,7 @@ void Text::rejectChanges()
 	}
 
 	// finally, invoke the DEPM
-	deleteEmptyParagraphMechanism(0, pars_size - 1, bparams.trackChanges);
+	deleteEmptyParagraphMechanism(0, pars_size - 1, bparams.track_changes);
 }
 
 
@@ -1452,20 +1494,19 @@ void Text::deleteWordBackward(Cursor & cur)
 
 
 // Kill to end of line.
-void Text::changeCase(Cursor & cur, TextCase action)
+void Text::changeCase(Cursor & cur, TextCase action, bool partial)
 {
 	LBUFERR(this == cur.text());
 	CursorSlice from;
 	CursorSlice to;
 
-	bool gotsel = false;
-	if (cur.selection()) {
+	bool const gotsel = cur.selection();
+	if (gotsel) {
 		from = cur.selBegin();
 		to = cur.selEnd();
-		gotsel = true;
 	} else {
 		from = cur.top();
-		getWord(from, to, PARTIAL_WORD);
+		getWord(from, to, partial ? PARTIAL_WORD : WHOLE_WORD);
 		cursorForwardOneWord(cur);
 	}
 
@@ -1519,7 +1560,7 @@ bool Text::handleBibitems(Cursor & cur)
 	// if a bibitem is deleted, merge with previous paragraph
 	// if this is a bibliography item as well
 	if (cur.pit() > 0 && par.layout() == prevpar.layout()) {
-		cur.recordUndo(ATOMIC_UNDO, prevcur.pit());
+		cur.recordUndo(prevcur.pit());
 		mergeParagraph(bufparams, cur.text()->paragraphs(),
 							prevcur.pit());
 		cur.forceBufferUpdate();
@@ -1545,7 +1586,7 @@ bool Text::erase(Cursor & cur)
 		// any paragraphs
 		cur.recordUndo(DELETE_UNDO);
 		bool const was_inset = cur.paragraph().isInset(cur.pos());
-		if(!par.eraseChar(cur.pos(), cur.buffer()->params().trackChanges))
+		if(!par.eraseChar(cur.pos(), cur.buffer()->params().track_changes))
 			// the character has been logically deleted only => skip it
 			cur.top().forwardPos();
 
@@ -1558,7 +1599,7 @@ bool Text::erase(Cursor & cur)
 		if (cur.pit() == cur.lastpit())
 			return dissolveInset(cur);
 
-		if (!par.isMergedOnEndOfParDeletion(cur.buffer()->params().trackChanges)) {
+		if (!par.isMergedOnEndOfParDeletion(cur.buffer()->params().track_changes)) {
 			par.setChange(cur.pos(), Change(Change::DELETED));
 			cur.forwardPos();
 			needsUpdate = true;
@@ -1601,14 +1642,14 @@ bool Text::backspacePos0(Cursor & cur)
 	// is it an empty paragraph?
 	if (cur.lastpos() == 0
 	    || (cur.lastpos() == 1 && par.isSeparator(0))) {
-		cur.recordUndo(ATOMIC_UNDO, prevcur.pit(), cur.pit());
+		cur.recordUndo(prevcur.pit());
 		plist.erase(lyx::next(plist.begin(), cur.pit()));
 		needsUpdate = true;
 	}
 	// is previous par empty?
 	else if (prevcur.lastpos() == 0
 		 || (prevcur.lastpos() == 1 && prevpar.isSeparator(0))) {
-		cur.recordUndo(ATOMIC_UNDO, prevcur.pit(), cur.pit());
+		cur.recordUndo(prevcur.pit());
 		plist.erase(lyx::next(plist.begin(), prevcur.pit()));
 		needsUpdate = true;
 	}
@@ -1620,7 +1661,7 @@ bool Text::backspacePos0(Cursor & cur)
 	else if (par.layout() == prevpar.layout()
 		 || tclass.isDefaultLayout(par.layout())
 		 || tclass.isPlainLayout(par.layout())) {
-		cur.recordUndo(ATOMIC_UNDO, prevcur.pit());
+		cur.recordUndo(prevcur.pit());
 		mergeParagraph(bufparams, plist, prevcur.pit());
 		needsUpdate = true;
 	}
@@ -1645,8 +1686,8 @@ bool Text::backspace(Cursor & cur)
 		Cursor prev_cur = cur;
 		--prev_cur.pit();
 
-		if (!prev_cur.paragraph().isMergedOnEndOfParDeletion(cur.buffer()->params().trackChanges)) {
-			cur.recordUndo(ATOMIC_UNDO, prev_cur.pit(), prev_cur.pit());
+		if (!prev_cur.paragraph().isMergedOnEndOfParDeletion(cur.buffer()->params().track_changes)) {
+			cur.recordUndo(prev_cur.pit(), prev_cur.pit());
 			prev_cur.paragraph().setChange(prev_cur.lastpos(), Change(Change::DELETED));
 			setCursorIntern(cur, prev_cur.pit(), prev_cur.lastpos());
 			return true;
@@ -1666,7 +1707,7 @@ bool Text::backspace(Cursor & cur)
 		setCursorIntern(cur, cur.pit(), cur.pos() - 1,
 				false, cur.boundary());
 		bool const was_inset = cur.paragraph().isInset(cur.pos());
-		cur.paragraph().eraseChar(cur.pos(), cur.buffer()->params().trackChanges);
+		cur.paragraph().eraseChar(cur.pos(), cur.buffer()->params().track_changes);
 		if (was_inset)
 			cur.forceBufferUpdate();
 		else
@@ -1708,7 +1749,7 @@ bool Text::dissolveInset(Cursor & cur)
 		spos += cur.pos();
 	spit += cur.pit();
 	Buffer & b = *cur.buffer();
-	cur.paragraph().eraseChar(cur.pos(), b.params().trackChanges);
+	cur.paragraph().eraseChar(cur.pos(), b.params().track_changes);
 
 	if (!plist.empty()) {
 		// see bug 7319
@@ -1843,7 +1884,7 @@ docstring Text::currentState(Cursor const & cur) const
 	Paragraph const & par = cur.paragraph();
 	odocstringstream os;
 
-	if (buf.params().trackChanges)
+	if (buf.params().track_changes)
 		os << _("[Change Tracking] ");
 
 	Change change = par.lookupChange(cur.pos());
@@ -2009,16 +2050,25 @@ docstring Text::asString(pit_type beg, pit_type end, int options) const
 }
 
 
-void Text::forToc(docstring & os, size_t maxlen, bool shorten) const
+void Text::shortenForOutliner(docstring & str, size_t const maxlen)
 {
-	if (maxlen == 0)
-		maxlen = std::numeric_limits<std::size_t>::max();
-	else
-		LASSERT(maxlen >= 8, maxlen = TOC_ENTRY_LENGTH);
-	for (size_t i = 0; i != pars_.size() && os.length() < maxlen; ++i)
-		pars_[i].forToc(os, maxlen);
-	if (shorten && os.length() >= maxlen)
-		os = os.substr(0, maxlen - 3) + from_ascii("...");
+	support::truncateWithEllipsis(str, maxlen);
+	docstring::iterator it = str.begin();
+	docstring::iterator end = str.end();
+	for (; it != end; ++it)
+		if ((*it) == L'\n' || (*it) == L'\t')
+			(*it) = L' ';	
+}
+
+
+void Text::forOutliner(docstring & os, size_t const maxlen,
+					   bool const shorten) const
+{
+	size_t tmplen = shorten ? maxlen + 1 : maxlen;
+	for (size_t i = 0; i != pars_.size() && os.length() < tmplen; ++i)
+		pars_[i].forOutliner(os, tmplen, false);
+	if (shorten)
+		shortenForOutliner(os, maxlen);
 }
 
 
@@ -2064,7 +2114,7 @@ void Text::charsTranspose(Cursor & cur)
 
 	// And finally, we are ready to perform the transposition.
 	// Track the changes if Change Tracking is enabled.
-	bool const trackChanges = cur.buffer()->params().trackChanges;
+	bool const trackChanges = cur.buffer()->params().track_changes;
 
 	cur.recordUndo();
 
