@@ -17,7 +17,6 @@
 
 #include "Text.h"
 
-#include "Bidi.h"
 #include "BranchList.h"
 #include "FloatList.h"
 #include "FuncStatus.h"
@@ -45,8 +44,10 @@
 #include "SpellChecker.h"
 #include "TextClass.h"
 #include "TextMetrics.h"
+#include "Thesaurus.h"
 #include "WordLangTuple.h"
 
+#include "frontends/alert.h"
 #include "frontends/Application.h"
 #include "frontends/Clipboard.h"
 #include "frontends/Selection.h"
@@ -131,7 +132,7 @@ static void finishChange(Cursor & cur, bool selecting)
 }
 
 
-static void mathDispatch(Cursor & cur, FuncRequest const & cmd, bool display)
+static void mathDispatch(Cursor & cur, FuncRequest const & cmd)
 {
 	cur.recordUndo();
 	docstring sel = cur.selectionAsString(false);
@@ -151,45 +152,39 @@ static void mathDispatch(Cursor & cur, FuncRequest const & cmd, bool display)
 		LATTEST(old_pos == cur.pos());
 #endif
 		cur.nextInset()->edit(cur, true);
-		// don't do that also for LFUN_MATH_MODE
-		// unless you want end up with always changing
-		// to mathrm when opening an inlined inset --
-		// I really hate "LyXfunc overloading"...
-		if (display)
-			cur.dispatch(FuncRequest(LFUN_MATH_DISPLAY));
-		// Avoid an unnecessary undo step if cmd.argument
-		// is empty
-		if (!cmd.argument().empty())
-			cur.dispatch(FuncRequest(LFUN_MATH_INSERT,
-						 cmd.argument()));
+		if (cmd.action() != LFUN_MATH_MODE)
+			// LFUN_MATH_MODE has a different meaning in math mode
+			cur.dispatch(cmd);
 	} else {
-		// create a macro if we see "\\newcommand"
-		// somewhere, and an ordinary formula
-		// otherwise
-		if (sel.find(from_ascii("\\newcommand")) == string::npos
-				&& sel.find(from_ascii("\\newlyxcommand")) == string::npos
-				&& sel.find(from_ascii("\\def")) == string::npos)
-		{
-			InsetMathHull * formula = new InsetMathHull(cur.buffer());
-			string const selstr = to_utf8(sel);
-			istringstream is(selstr);
-			Lexer lex;
+		InsetMathHull * formula = new InsetMathHull(cur.buffer());
+		string const selstr = to_utf8(sel);
+		istringstream is(selstr);
+		Lexer lex;
+		lex.setStream(is);
+		if (!formula->readQuiet(lex)) {
+			// No valid formula, let's try with delims
+			is.str("$" + selstr + "$");
 			lex.setStream(is);
 			if (!formula->readQuiet(lex)) {
-				// No valid formula, let's try with delims
-				is.str("$" + selstr + "$");
-				lex.setStream(is);
-				if (!formula->readQuiet(lex)) {
-					// Still not valid, leave it as is
-					valid = false;
-					delete formula;
-					cur.insert(sel);
-				} else
-					cur.insert(formula);
-			} else
-				cur.insert(formula);
-		} else {
-			cur.insert(new MathMacroTemplate(cur.buffer(), sel));
+				// Still not valid, leave it as is
+				valid = false;
+				delete formula;
+				cur.insert(sel);
+			}
+		}
+		if (valid) {
+			cur.insert(formula);
+			cur.nextInset()->edit(cur, true);
+			LASSERT(cur.inMathed(), return);
+			cur.pos() = 0;
+			cur.resetAnchor();
+			cur.setSelection(true);
+			cur.pos() = cur.lastpos();
+			if (cmd.action() != LFUN_MATH_MODE)
+				// LFUN_MATH_MODE has a different meaning in math mode
+				cur.dispatch(cmd);
+			cur.clearSelection();
+			cur.pos() = cur.lastpos();
 		}
 	}
 	if (valid)
@@ -264,6 +259,42 @@ static bool doInsertInset(Cursor & cur, Text * text,
 		}
 		return true;
 	}
+	else if (cmd.action() == LFUN_ARGUMENT_INSERT) {
+		bool cotextinsert = false;
+		InsetArgument const * const ia = static_cast<InsetArgument const *>(inset);
+		Layout const & lay = cur.paragraph().layout();
+		Layout::LaTeXArgMap args = lay.args();
+		Layout::LaTeXArgMap::const_iterator const lait = args.find(ia->name());
+		if (lait != args.end())
+			cotextinsert = (*lait).second.insertcotext;
+		else {
+			InsetLayout const & il = cur.inset().getLayout();
+			args = il.args();
+			Layout::LaTeXArgMap::const_iterator const ilait = args.find(ia->name());
+			if (ilait != args.end())
+				cotextinsert = (*ilait).second.insertcotext;
+		}
+		// The argument requests to insert a copy of the co-text to the inset
+		if (cotextinsert) {
+			docstring ds;
+			// If we have a selection within a paragraph, use this
+			if (cur.selection() && cur.selBegin().pit() == cur.selEnd().pit())
+				ds = cur.selectionAsString(false);
+			// else use the whole paragraph
+			else
+				ds = cur.paragraph().asString();
+			text->insertInset(cur, inset);
+			if (edit)
+				inset->edit(cur, true);
+			// Now put co-text into inset
+			Font const f(inherit_font, cur.current_font.language());
+			if (!ds.empty()) {
+				cur.text()->insertStringAsLines(cur, ds, f);
+				cur.leaveInset(*inset);
+			}
+			return true;
+		}
+	}
 
 	bool gotsel = false;
 	if (cur.selection()) {
@@ -344,8 +375,6 @@ static void outline(OutlineOp mode, Cursor & cur)
 	ParagraphList::iterator finish = start;
 	ParagraphList::iterator const end = pars.end();
 
-	DocumentClass const & tc = buf.params().documentClass();
-
 	int const thistoclevel = buf.text().getTocLevel(distance(bgn, start));
 	int toclevel;
 
@@ -382,7 +411,7 @@ static void outline(OutlineOp mode, Cursor & cur)
 			pit_type const newpit = distance(bgn, dest);
 			pit_type const len = distance(start, finish);
 			pit_type const deletepit = pit + len;
-			buf.undo().recordUndo(cur, ATOMIC_UNDO, newpit, deletepit - 1);
+			buf.undo().recordUndo(cur, newpit, deletepit - 1);
 			pars.splice(dest, start, finish);
 			cur.pit() = newpit;
 			break;
@@ -392,7 +421,7 @@ static void outline(OutlineOp mode, Cursor & cur)
 				// Nothing to move.
 				return;
 			// Go one down from *this* header:
-			ParagraphList::iterator dest = lyx::next(finish, 1);
+			ParagraphList::iterator dest = next(finish, 1);
 			// Go further down to find header to insert in front of:
 			for (; dest != end; ++dest) {
 				toclevel = buf.text().getTocLevel(distance(bgn, dest));
@@ -402,45 +431,33 @@ static void outline(OutlineOp mode, Cursor & cur)
 			}
 			// One such was found:
 			pit_type newpit = distance(bgn, dest);
-			buf.undo().recordUndo(cur, ATOMIC_UNDO, pit, newpit - 1);
+			buf.undo().recordUndo(cur, pit, newpit - 1);
 			pit_type const len = distance(start, finish);
 			pars.splice(dest, start, finish);
 			cur.pit() = newpit - len;
 			break;
 		}
-		case OutlineIn: {
-			pit_type const len = distance(start, finish);
-			buf.undo().recordUndo(cur, ATOMIC_UNDO, pit, pit + len - 1);
-			for (; start != finish; ++start) {
-				toclevel = buf.text().getTocLevel(distance(bgn, start));
-				if (toclevel == Layout::NOT_IN_TOC)
-					continue;
-				DocumentClass::const_iterator lit = tc.begin();
-				DocumentClass::const_iterator len = tc.end();
-				for (; lit != len; ++lit) {
-					if (lit->toclevel == toclevel + 1 &&
-					    start->layout().labeltype == lit->labeltype) {
-						start->setLayout(*lit);
-						break;
-					}
-				}
-			}
-			break;
-		}
+		case OutlineIn:
 		case OutlineOut: {
 			pit_type const len = distance(start, finish);
-			buf.undo().recordUndo(cur, ATOMIC_UNDO, pit, pit + len - 1);
+			buf.undo().recordUndo(cur, pit, pit + len - 1);
 			for (; start != finish; ++start) {
 				toclevel = buf.text().getTocLevel(distance(bgn, start));
 				if (toclevel == Layout::NOT_IN_TOC)
 					continue;
+
+				DocumentClass const & tc = buf.params().documentClass();
 				DocumentClass::const_iterator lit = tc.begin();
 				DocumentClass::const_iterator len = tc.end();
+				int const newtoclevel = 
+					(mode == OutlineIn ? toclevel + 1 : toclevel - 1);
+				LabelType const oldlabeltype = start->layout().labeltype;
+
 				for (; lit != len; ++lit) {
-					if (lit->toclevel == toclevel - 1 &&
-						start->layout().labeltype == lit->labeltype) {
-							start->setLayout(*lit);
-							break;
+					if (lit->toclevel ==  newtoclevel &&
+					     lit->labeltype == oldlabeltype) {
+						start->setLayout(*lit);
+						break;
 					}
 				}
 			}
@@ -464,6 +481,15 @@ bool Text::isRTL(Paragraph const & par) const
 	return par.isRTL(buffer.params());
 }
 
+	
+namespace {
+		
+	Language const * getLanguage(Cursor const & cur, string const & lang) {
+		return lang.empty() ? cur.getFont().language() : languages.getLanguage(lang);
+	}
+
+}
+
 
 void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 {
@@ -489,6 +515,8 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	cur.noScreenUpdate();
 
 	LBUFERR(this == cur.text());
+	
+	// NOTE: This should NOT be a reference. See commit 94a5481a.
 	CursorSlice const oldTopSlice = cur.top();
 	bool const oldBoundary = cur.boundary();
 	bool const oldSelection = cur.selection();
@@ -507,7 +535,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 
 	case LFUN_PARAGRAPH_MOVE_DOWN: {
 		pit_type const pit = cur.pit();
-		recUndo(cur, pit, pit + 1);
+		cur.recordUndo(pit, pit + 1);
 		cur.finishUndo();
 		pars_.swap(pit, pit + 1);
 		needsUpdate = true;
@@ -518,7 +546,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 
 	case LFUN_PARAGRAPH_MOVE_UP: {
 		pit_type const pit = cur.pit();
-		recUndo(cur, pit - 1, pit);
+		cur.recordUndo(pit - 1, pit);
 		cur.finishUndo();
 		pars_.swap(pit, pit - 1);
 		--cur.pit();
@@ -537,7 +565,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		// FIXME: this don't work for multipart document!
 		for (pit_type tmp = 0, end = pars_.size(); tmp != end; ++tmp) {
 			if (pars_[tmp].params().startOfAppendix()) {
-				recUndo(cur, tmp);
+				cur.recordUndo(tmp, tmp);
 				pars_[tmp].params().startOfAppendix(false);
 				break;
 			}
@@ -685,7 +713,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				cmd = FuncRequest(LFUN_FINISHED_LEFT);
 			}
 		} else {
-			if (reverseDirectionNeeded(cur)) {
+			if (cur.reverseDirectionNeeded()) {
 				cmd.setAction(cmd.action() == LFUN_CHAR_LEFT_SELECT ?
 					LFUN_CHAR_FORWARD_SELECT : LFUN_CHAR_FORWARD);
 			} else {
@@ -709,7 +737,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				cmd = FuncRequest(LFUN_FINISHED_RIGHT);
 			}
 		} else {
-			if (reverseDirectionNeeded(cur)) {
+			if (cur.reverseDirectionNeeded()) {
 				cmd.setAction(cmd.action() == LFUN_CHAR_RIGHT_SELECT ?
 					LFUN_CHAR_BACKWARD_SELECT : LFUN_CHAR_BACKWARD);
 			} else {
@@ -821,7 +849,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				cmd = FuncRequest(LFUN_FINISHED_RIGHT);
 			}
 		} else {
-			if (reverseDirectionNeeded(cur)) {
+			if (cur.reverseDirectionNeeded()) {
 				cmd.setAction(cmd.action() == LFUN_WORD_RIGHT_SELECT ?
 						LFUN_WORD_BACKWARD_SELECT : LFUN_WORD_BACKWARD);
 			} else {
@@ -873,7 +901,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				cmd = FuncRequest(LFUN_FINISHED_LEFT);
 			}
 		} else {
-			if (reverseDirectionNeeded(cur)) {
+			if (cur.reverseDirectionNeeded()) {
 				cmd.setAction(cmd.action() == LFUN_WORD_LEFT_SELECT ?
 						LFUN_WORD_FORWARD_SELECT : LFUN_WORD_FORWARD);
 			} else {
@@ -945,7 +973,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			pit_type const pit_end = cur.selEnd().pit();
 			for (pit_type pit = cur.selBegin().pit(); pit <= pit_end; pit++) {
 				pars_[pit].insertChar(0, '\t',
-						      bv->buffer().params().trackChanges);
+						      bv->buffer().params().track_changes);
 				// Update the selection pos to make sure the selection does not
 				// change as the inserted tab will increase the logical pos.
 				if (cur.realAnchor().pit() == pit)
@@ -964,7 +992,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	}
 
 	case LFUN_TAB_DELETE: {
-		bool const tc = bv->buffer().params().trackChanges;
+		bool const tc = bv->buffer().params().track_changes;
 		if (cur.selection()) {
 			// If there is a selection, a tab (if present) is removed from
 			// the beginning of each paragraph.
@@ -1025,8 +1053,6 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				singleParUpdate = false;
 			needsUpdate |= erase(cur);
 			cur.resetAnchor();
-			// It is possible to make it a lot faster still
-			// just comment out the line below...
 		} else {
 			cutSelection(cur, true, false);
 			singleParUpdate = false;
@@ -1037,13 +1063,18 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_CHAR_DELETE_BACKWARD:
 		if (!cur.selection()) {
 			if (bv->getIntl().getTransManager().backspace()) {
+				bool par_boundary = cur.pos() == 0;
+				bool first_par = cur.pit() == 0;
 				// Par boundary, full-screen update
-				if (cur.pos() == 0)
+				if (par_boundary)
 					singleParUpdate = false;
 				needsUpdate |= backspace(cur);
 				cur.resetAnchor();
-				// It is possible to make it a lot faster still
-				// just comment out the line below...
+				if (par_boundary && !first_par && cur.pos() > 0
+				    && cur.paragraph().isEnvSeparator(cur.pos() - 1)) {
+					needsUpdate |= backspace(cur);
+					cur.resetAnchor();
+				}
 			}
 		} else {
 			cutSelection(cur, true, false);
@@ -1051,11 +1082,59 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		}
 		break;
 
-	case LFUN_PARAGRAPH_BREAK:
+	case LFUN_PARAGRAPH_BREAK: {
 		cap::replaceSelection(cur);
-		breakParagraph(cur, cmd.argument() == "inverse");
+		pit_type pit = cur.pit();
+		Paragraph const & par = pars_[pit];
+		bool lastpar = (pit == pit_type(pars_.size() - 1));
+		Paragraph const & nextpar = lastpar ? par : pars_[pit + 1];
+		pit_type prev = pit;
+		if (pit > 0) {
+			if (!pars_[pit - 1].layout().isEnvironment())
+				prev = depthHook(pit, par.getDepth());
+			else if (pars_[pit - 1].getDepth() >= par.getDepth())
+				prev = pit - 1;
+		}
+		if (prev < pit && cur.pos() == par.beginOfBody()
+		    && !par.size() && !par.isEnvSeparator(cur.pos())
+		    && !par.layout().isCommand()
+		    && pars_[prev].layout() != par.layout()
+		    && pars_[prev].layout().isEnvironment()
+		    && !nextpar.isEnvSeparator(nextpar.beginOfBody())) {
+			if (par.layout().isEnvironment()
+			    && pars_[prev].getDepth() == par.getDepth()) {
+				docstring const layout = par.layout().name();
+				DocumentClass const & tc = bv->buffer().params().documentClass();
+				lyx::dispatch(FuncRequest(LFUN_LAYOUT, tc.plainLayout().name()));
+				lyx::dispatch(FuncRequest(LFUN_SEPARATOR_INSERT, "plain"));
+				lyx::dispatch(FuncRequest(LFUN_PARAGRAPH_BREAK, "inverse"));
+				lyx::dispatch(FuncRequest(LFUN_LAYOUT, layout));
+			} else {
+				lyx::dispatch(FuncRequest(LFUN_SEPARATOR_INSERT, "plain"));
+				breakParagraph(cur);
+			}
+			Font const f(inherit_font, cur.current_font.language());
+			pars_[cur.pit() - 1].resetFonts(f);
+		} else {
+			if (par.isEnvSeparator(cur.pos()))
+				cur.posForward();
+			breakParagraph(cur, cmd.argument() == "inverse");
+		}
 		cur.resetAnchor();
+		// If we have a list and autoinsert item insets,
+		// insert them now.
+		Layout::LaTeXArgMap args = par.layout().args();
+		Layout::LaTeXArgMap::const_iterator lait = args.begin();
+		Layout::LaTeXArgMap::const_iterator const laend = args.end();
+		for (; lait != laend; ++lait) {
+			Layout::latexarg arg = (*lait).second;
+			if (arg.autoinsert && prefixIs((*lait).first, "item:")) {
+				FuncRequest cmd(LFUN_ARGUMENT_INSERT, (*lait).first);
+				lyx::dispatch(cmd);
+			}
+		}
 		break;
+	}
 
 	case LFUN_INSET_INSERT: {
 		cur.recordUndo();
@@ -1080,13 +1159,13 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			/*
 			Paragraph & par = pars_[cur.pit()];
 			if (inset->lyxCode() == LABEL_CODE
-				&& !par.layout().counter.empty() {
+				&& !par.layout().counter.empty()) {
 				// Go to the end of the paragraph
 				// Warning: Because of Change-Tracking, the last
 				// position is 'size()' and not 'size()-1':
 				cur.pos() = par.size();
 				// Insert a new paragraph
-				FuncRequest fr(LFUN_BREAK_PARAGRAPH);
+				FuncRequest fr(LFUN_PARAGRAPH_BREAK);
 				dispatch(cur, fr);
 			}
 			*/
@@ -1165,6 +1244,14 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			specialChar(cur, InsetSpecialChar::END_OF_SENTENCE);
 		else if (name == "menu-separator")
 			specialChar(cur, InsetSpecialChar::MENU_SEPARATOR);
+		else if (name == "lyx")
+			specialChar(cur, InsetSpecialChar::PHRASE_LYX);
+		else if (name == "tex")
+			specialChar(cur, InsetSpecialChar::PHRASE_TEX);
+		else if (name == "latex")
+			specialChar(cur, InsetSpecialChar::PHRASE_LATEX);
+		else if (name == "latex2e")
+			specialChar(cur, InsetSpecialChar::PHRASE_LATEX2E);
 		else if (name.empty())
 			lyxerr << "LyX function 'specialchar-insert' needs an argument." << endl;
 		else
@@ -1202,15 +1289,15 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	}
 
 	case LFUN_WORD_UPCASE:
-		changeCase(cur, text_uppercase);
+		changeCase(cur, text_uppercase, cmd.getArg(0) == "partial");
 		break;
 
 	case LFUN_WORD_LOWCASE:
-		changeCase(cur, text_lowercase);
+		changeCase(cur, text_lowercase, cmd.getArg(0) == "partial");
 		break;
 
 	case LFUN_WORD_CAPITALIZE:
-		changeCase(cur, text_capitalization);
+		changeCase(cur, text_capitalization, cmd.getArg(0) == "partial");
 		break;
 
 	case LFUN_CHARS_TRANSPOSE:
@@ -1258,8 +1345,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			else if (arg == "wmf")
 				type = Clipboard::WmfGraphicsType;
 			else
-				// We used to assert, but couldn't the argument come from, say, the
-				// minibuffer and just be mistyped?
+				// we also check in getStatus()
 				LYXERR0("Unrecognized graphics type: " << arg);
 
 			pasteClipboardGraphics(cur, bv->buffer().errorList("Paste"), type);
@@ -1403,15 +1489,10 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			while (cur.paragraph().params().depth() > split_depth)
 				lyx::dispatch(FuncRequest(LFUN_DEPTH_DECREMENT));
 		}
-		bool const morecont = cur.lastpos() > cur.pos();
-		// FIXME This hardcoding is bad
-		docstring const sep =
-				cur.buffer()->params().documentClass().hasLayout(from_ascii("Separator"))
-					? from_ascii("Separator") : from_ascii("--Separator--");
-		lyx::dispatch(FuncRequest(LFUN_LAYOUT, sep));
+		DocumentClass const & tc = bv->buffer().params().documentClass();
+		lyx::dispatch(FuncRequest(LFUN_LAYOUT, tc.plainLayout().name()));
+		lyx::dispatch(FuncRequest(LFUN_SEPARATOR_INSERT, "plain"));
 		lyx::dispatch(FuncRequest(LFUN_PARAGRAPH_BREAK, "inverse"));
-		if (morecont)
-			lyx::dispatch(FuncRequest(LFUN_DOWN));
 		lyx::dispatch(FuncRequest(LFUN_LAYOUT, layout));
 
 		break;
@@ -1527,17 +1608,19 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		case mouse_button::button1:
 			// Set the cursor
 			if (!bv->mouseSetCursor(cur, cmd.argument() == "region-select"))
-				cur.screenUpdateFlags(Update::SinglePar | Update::FitCursor);
+				cur.screenUpdateFlags(Update::FitCursor);
 			if (bvcur.wordSelection())
 				selectWord(bvcur, WHOLE_WORD);
 			break;
 
 		case mouse_button::button2:
-			// Middle mouse pasting.
-			bv->mouseSetCursor(cur);
-			lyx::dispatch(
-				FuncRequest(LFUN_COMMAND_ALTERNATIVES,
-					    "selection-paste ; primary-selection-paste paragraph"));
+			if (lyxrc.mouse_middlebutton_paste) {
+				// Middle mouse pasting.
+				bv->mouseSetCursor(cur);
+				lyx::dispatch(
+					FuncRequest(LFUN_COMMAND_ALTERNATIVES,
+						    "selection-paste ; primary-selection-paste paragraph"));
+			}
 			cur.noScreenUpdate();
 			break;
 
@@ -1550,7 +1633,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				return;
 			}
 			if (!bv->mouseSetCursor(cur, false))
-				cur.screenUpdateFlags(Update::SinglePar | Update::FitCursor);
+				cur.screenUpdateFlags(Update::FitCursor);
 			break;
 		}
 
@@ -1902,6 +1985,16 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		cur.posForward();
 		break;
 
+	case LFUN_SEPARATOR_INSERT: {
+		doInsertInset(cur, this, cmd, false, false);
+		cur.posForward();
+		// remove a following space
+		Paragraph & par = cur.paragraph();
+		if (cur.pos() != cur.lastpos() && par.isLineSeparator(cur.pos()))
+		    par.eraseChar(cur.pos(), cur.buffer()->params().track_changes);
+		break;
+	}
+
 	case LFUN_DEPTH_DECREMENT:
 		changeDepth(cur, DEC_DEPTH);
 		break;
@@ -1910,22 +2003,38 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		changeDepth(cur, INC_DEPTH);
 		break;
 
-	case LFUN_MATH_DISPLAY:
-		mathDispatch(cur, cmd, true);
-		break;
-
 	case LFUN_REGEXP_MODE:
 		regexpDispatch(cur, cmd);
 		break;
 
-	case LFUN_MATH_MODE:
-		if (cmd.argument() == "on")
+	case LFUN_MATH_MODE: {
+		if (cmd.argument() == "on" || cmd.argument() == "") {
 			// don't pass "on" as argument
 			// (it would appear literally in the first cell)
-			mathDispatch(cur, FuncRequest(LFUN_MATH_MODE), false);
-		else
-			mathDispatch(cur, cmd, false);
+			docstring sel = cur.selectionAsString(false);
+			MathMacroTemplate * macro = new MathMacroTemplate(cur.buffer());
+			// create a macro template if we see "\\newcommand" somewhere, and
+			// an ordinary formula otherwise
+			if (!sel.empty()
+				&& (sel.find(from_ascii("\\newcommand")) != string::npos
+					|| sel.find(from_ascii("\\newlyxcommand")) != string::npos
+					|| sel.find(from_ascii("\\def")) != string::npos)
+				&& macro->fromString(sel)) {
+				cur.recordUndo();
+				replaceSelection(cur);
+				cur.insert(macro);
+			} else {
+				// no meaningful macro template was found
+				delete macro;
+				mathDispatch(cur,FuncRequest(LFUN_MATH_MODE));
+			}
+		} else
+			// The argument is meaningful
+			// We replace cmd with LFUN_MATH_INSERT because LFUN_MATH_MODE
+			// has a different meaning in math mode
+			mathDispatch(cur, FuncRequest(LFUN_MATH_INSERT,cmd.argument()));
 		break;
+	}
 
 	case LFUN_MATH_MACRO:
 		if (cmd.argument().empty())
@@ -1953,27 +2062,16 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		}
 		break;
 
-	// passthrough hat and underscore outside mathed:
+	case LFUN_MATH_DISPLAY:
 	case LFUN_MATH_SUBSCRIPT:
-		mathDispatch(cur, FuncRequest(LFUN_SELF_INSERT, "_"), false);
-		break;
 	case LFUN_MATH_SUPERSCRIPT:
-		mathDispatch(cur, FuncRequest(LFUN_SELF_INSERT, "^"), false);
-		break;
-
 	case LFUN_MATH_INSERT:
 	case LFUN_MATH_AMS_MATRIX:
 	case LFUN_MATH_MATRIX:
 	case LFUN_MATH_DELIM:
-	case LFUN_MATH_BIGDELIM: {
-		cur.recordUndo();
-		cap::replaceSelection(cur);
-		cur.insert(new InsetMathHull(cur.buffer(), hullSimple));
-		checkAndActivateInset(cur, true);
-		LASSERT(cur.inMathed(), break);
-		cur.dispatch(cmd);
+	case LFUN_MATH_BIGDELIM:
+		mathDispatch(cur, cmd);
 		break;
-	}
 
 	case LFUN_FONT_EMPH: {
 		Font font(ignore_font, ignore_language);
@@ -2163,6 +2261,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_ACCENT_GRAVE:
 	case LFUN_ACCENT_ACUTE:
 	case LFUN_ACCENT_TILDE:
+	case LFUN_ACCENT_PERISPOMENI:
 	case LFUN_ACCENT_CEDILLA:
 	case LFUN_ACCENT_MACRON:
 	case LFUN_ACCENT_DOT:
@@ -2226,6 +2325,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	}
 
 	case LFUN_THESAURUS_ENTRY: {
+		Language const * language = cur.getFont().language();
 		docstring arg = cmd.argument();
 		if (arg.empty()) {
 			arg = cur.selectionAsString(false);
@@ -2234,16 +2334,33 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				// Get word or selection
 				selectWordWhenUnderCursor(cur, WHOLE_WORD);
 				arg = cur.selectionAsString(false);
-				arg += " lang=" + from_ascii(cur.getFont().language()->lang());
+				arg += " lang=" + from_ascii(language->lang());
 			}
+		} else {
+			string lang = cmd.getArg(1);
+			// This duplicates the code in GuiThesaurus::initialiseParams
+			if (prefixIs(lang, "lang=")) {
+				language = languages.getLanguage(lang.substr(5));
+				if (!language)
+					language = cur.getFont().language();
+			}
+		}
+		string lang = language->code();
+		if (lyxrc.thesaurusdir_path.empty() && !thesaurus.thesaurusInstalled(from_ascii(lang))) {
+			LYXERR(Debug::ACTION, "Command " << cmd << ". Thesaurus not found for language " << lang);
+			frontend::Alert::warning(_("Path to thesaurus directory not set!"),
+					_("The path to the thesaurus directory has not been specified.\n"
+					  "The thesaurus is not functional.\n"
+					  "Please refer to sec. 6.15.1 of the User's Guide for setup\n"
+					  "instructions."));
 		}
 		bv->showDialog("thesaurus", to_utf8(arg));
 		break;
 	}
 
 	case LFUN_SPELLING_ADD: {
+		Language const * language = getLanguage(cur, cmd.getArg(1));
 		docstring word = from_utf8(cmd.getArg(0));
-		Language * lang;
 		if (word.empty()) {
 			word = cur.selectionAsString(false);
 			// FIXME
@@ -2252,22 +2369,15 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				selectWordWhenUnderCursor(cur, WHOLE_WORD);
 				word = cur.selectionAsString(false);
 			}
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else if (cmd.getArg(1).empty()) {
-			// optional language argument is missing
-			// use the language at cursor position
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else {
-			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
 		}
-		WordLangTuple wl(word, lang);
+		WordLangTuple wl(word, language);
 		theSpellChecker()->insert(wl);
 		break;
 	}
 
 	case LFUN_SPELLING_IGNORE: {
+		Language const * language = getLanguage(cur, cmd.getArg(1));
 		docstring word = from_utf8(cmd.getArg(0));
-		Language * lang;
 		if (word.empty()) {
 			word = cur.selectionAsString(false);
 			// FIXME
@@ -2276,20 +2386,15 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				selectWordWhenUnderCursor(cur, WHOLE_WORD);
 				word = cur.selectionAsString(false);
 			}
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else if (cmd.getArg(1).empty()) {
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else {
-			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
 		}
-		WordLangTuple wl(word, lang);
+		WordLangTuple wl(word, language);
 		theSpellChecker()->accept(wl);
 		break;
 	}
 
 	case LFUN_SPELLING_REMOVE: {
+		Language const * language = getLanguage(cur, cmd.getArg(1));
 		docstring word = from_utf8(cmd.getArg(0));
-		Language * lang;
 		if (word.empty()) {
 			word = cur.selectionAsString(false);
 			// FIXME
@@ -2298,13 +2403,8 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				selectWordWhenUnderCursor(cur, WHOLE_WORD);
 				word = cur.selectionAsString(false);
 			}
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else if (cmd.getArg(1).empty()) {
-			lang = const_cast<Language *>(cur.getFont().language());
-		} else {
-			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
 		}
-		WordLangTuple wl(word, lang);
+		WordLangTuple wl(word, language);
 		theSpellChecker()->remove(wl);
 		break;
 	}
@@ -2368,6 +2468,33 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		needsUpdate = true;
 		break;
 
+	case LFUN_SERVER_GET_STATISTICS:
+		{
+			DocIterator from, to;
+			if (cur.selection()) {
+				from = cur.selectionBegin();
+				to = cur.selectionEnd();
+			} else {
+				from = doc_iterator_begin(cur.buffer());
+				to = doc_iterator_end(cur.buffer());
+			}
+
+			cur.buffer()->updateStatistics(from, to);
+			string const arg0 = cmd.getArg(0);
+			if (arg0 == "words") {
+				cur.message(convert<docstring>(cur.buffer()->wordCount()));
+			} else if (arg0 == "chars") {
+				cur.message(convert<docstring>(cur.buffer()->charCount(false)));
+			} else if (arg0 == "chars-space") {
+				cur.message(convert<docstring>(cur.buffer()->charCount(true)));
+			} else {
+				cur.message(convert<docstring>(cur.buffer()->wordCount()) + " "
+				+ convert<docstring>(cur.buffer()->charCount(false)) + " "
+				+ convert<docstring>(cur.buffer()->charCount(true)));
+			}
+		}
+		break;
+
 	default:
 		LYXERR(Debug::ACTION, "Command " << cmd << " not DISPATCHED by Text");
 		cur.undispatched();
@@ -2591,11 +2718,8 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_CAPTION_INSERT: {
 		code = CAPTION_CODE;
 		string arg = cmd.getArg(0);
-		bool varia = arg != "LongTableNoNumber";
-		if (cur.depth() > 0) {
-			if (&cur[cur.depth() - 1].inset())
-				varia = cur[cur.depth() - 1].inset().allowsCaptionVariation(arg);
-		}
+		bool varia = arg != "Unnumbered"
+			&& cur.inset().allowsCaptionVariation(arg);
 		// not allowed in description items,
 		// and in specific insets
 		enable = !inDescriptionItem(cur)
@@ -2849,22 +2973,30 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 			break;
 		}
 
-		// explicit graphics type?
 		Clipboard::GraphicsType type = Clipboard::AnyGraphicsType;
-		if ((arg == "pdf" && (type = Clipboard::PdfGraphicsType))
-			  || (arg == "png" && (type = Clipboard::PngGraphicsType))
-			  || (arg == "jpeg" && (type = Clipboard::JpegGraphicsType))
-			  || (arg == "linkback" &&  (type = Clipboard::LinkBackGraphicsType))
-			  || (arg == "emf" &&  (type = Clipboard::EmfGraphicsType))
-			  || (arg == "wmf" &&  (type = Clipboard::WmfGraphicsType))) {
-			enable = theClipboard().hasGraphicsContents(type);
+		if (arg == "pdf")
+			type = Clipboard::PdfGraphicsType;
+		else if (arg == "png")
+			type = Clipboard::PngGraphicsType;
+		else if (arg == "jpeg")
+			type = Clipboard::JpegGraphicsType;
+		else if (arg == "linkback")
+			type = Clipboard::LinkBackGraphicsType;
+		else if (arg == "emf")
+			type = Clipboard::EmfGraphicsType;
+		else if (arg == "wmf")
+			type = Clipboard::WmfGraphicsType;
+		else {
+			// unknown argument
+			LYXERR0("Unrecognized graphics type: " << arg);
+			// we don't want to assert if the user just mistyped the LFUN
+			LATTEST(cmd.origin() != FuncRequest::INTERNAL);
+			enable = false;
 			break;
 		}
-
-		// unknown argument
-		enable = false;
+		enable = theClipboard().hasGraphicsContents(type);
 		break;
-	 }
+	}
 
 	case LFUN_CLIPBOARD_PASTE:
 	case LFUN_CLIPBOARD_PASTE_SIMPLE:
@@ -2917,6 +3049,11 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 			&& cur.pos() > cur.paragraph().beginOfBody();
 		break;
 
+	case LFUN_SEPARATOR_INSERT:
+		// Always enabled for now
+		enable = true;
+		break;
+
 	case LFUN_TAB_INSERT:
 	case LFUN_TAB_DELETE:
 		enable = cur.paragraph().isPassThru();
@@ -2950,7 +3087,7 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 		break;
 
 	case LFUN_PARAGRAPH_BREAK:
-		enable = cur.inset().getLayout().isMultiPar();
+		enable = inset().allowMultiPar();
 		break;
 
 	case LFUN_SPELLING_ADD:
@@ -2976,12 +3113,6 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	}
 
 	case LFUN_ENVIRONMENT_SPLIT: {
-		// FIXME This hardcoding is bad
-		if (!cur.buffer()->params().documentClass().hasLayout(from_ascii("Separator"))
-		    && !cur.buffer()->params().documentClass().hasLayout(from_ascii("--Separator--"))) {
-			enable = false;
-			break;
-		}
 		if (cmd.argument() == "outer") {
 			// check if we have an environment in our nesting hierarchy
 			bool res = false;
@@ -3015,6 +3146,9 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 		break;
 
 	// FIXME: why are accent lfuns forbidden with pass_thru layouts?
+	//  Because they insert COMBINING DIACRITICAL Unicode characters,
+	//  that cannot be handled by LaTeX but must be converted according
+	//  to the definition in lib/unicodesymbols?
 	case LFUN_ACCENT_ACUTE:
 	case LFUN_ACCENT_BREVE:
 	case LFUN_ACCENT_CARON:
@@ -3028,6 +3162,7 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_ACCENT_OGONEK:
 	case LFUN_ACCENT_TIE:
 	case LFUN_ACCENT_TILDE:
+	case LFUN_ACCENT_PERISPOMENI:
 	case LFUN_ACCENT_UMLAUT:
 	case LFUN_ACCENT_UNDERBAR:
 	case LFUN_ACCENT_UNDERDOT:
@@ -3098,6 +3233,7 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_UNICODE_INSERT:
 	case LFUN_THESAURUS_ENTRY:
 	case LFUN_ESCAPE:
+	case LFUN_SERVER_GET_STATISTICS:
 		// these are handled in our dispatch()
 		enable = true;
 		break;
