@@ -30,6 +30,10 @@
 #include <string>
 #include <vector>
 
+#ifdef ERROR
+#undef ERROR
+#endif
+
 namespace lyx {
 
 namespace support { class FileName; }
@@ -199,6 +203,8 @@ public:
 	OutputType outputType() const { return outputType_; }
 	/// Can be latex, docbook ... (the name of a format)
 	std::string outputFormat() const { return outputFormat_; }
+	///
+	docstring outlinerName(std::string const & type) const;
 protected:
 	/// Protect construction
 	TextClass();
@@ -327,6 +333,8 @@ protected:
 	bool cite_full_author_list_;
 	/// The possible citation styles
 	std::map<CiteEngineType, std::vector<CitationStyle> > cite_styles_;
+	///
+	std::map<std::string, docstring> outliner_names_;
 private:
 	///////////////////////////////////////////////////////////////////
 	// helper routines for reading layout files
@@ -359,6 +367,8 @@ private:
 	int readCiteEngineType(Lexer &) const;
 	///
 	bool readCiteFormat(Lexer &);
+	///
+	bool readOutlinerName(Lexer &);
 };
 
 
@@ -369,12 +379,8 @@ private:
 /// In that regard, DocumentClass objects are "dynamic". But this is really
 /// an illusion, since DocumentClass objects are not (currently) changed
 /// when, say, a new Module is loaded. Rather, the old DocumentClass is
-/// discarded---actually, it's kept around in case something on the cut
+/// discarded---actually, it will be kept around if something on the cut
 /// stack needs it---and a new one is created from scratch.
-///
-/// In the main LyX code, DocumentClass objects are created only by
-/// DocumentClassBundle, for which see below.
-///
 class DocumentClass : public TextClass {
 public:
 	///
@@ -488,8 +494,7 @@ protected:
 	/// Needed in tex2lyx
 	DocumentClass() {}
 private:
-	/// The only class that can create a DocumentClass is
-	/// DocumentClassBundle, which calls the protected constructor.
+	/// The only way to make a DocumentClass is to call this function.
 	friend DocumentClassPtr
 		getDocumentClass(LayoutFile const &, LayoutModuleList const &,
 				 bool const clone);
