@@ -16,6 +16,7 @@
 
 #include "Font.h"
 #include "InsetList.h"
+#include "LayoutEnums.h"
 #include "ParagraphMetrics.h"
 
 #include "support/types.h"
@@ -34,7 +35,7 @@ class TextMetrics
 {
 public:
 	/// Default constructor (only here for STL containers).
-	TextMetrics() : text_(0) {}
+	TextMetrics() : bv_(0), text_(0), main_text_(false), max_width_(0) {}
 	/// The only useful constructor.
 	TextMetrics(BufferView *, Text *);
 
@@ -105,30 +106,21 @@ public:
 
 	///
 	int maxWidth() const { return max_width_; }
-	///
-	int singleWidth(pit_type const pit,	pos_type pos) const;
 
 	///
 	int rightMargin(ParagraphMetrics const & pm) const;
 	int rightMargin(pit_type const pit) const;
 
-	/** this calculates the specified parameters. needed when setting
-	 * the cursor and when creating a visible row */
-	void computeRowMetrics(pit_type pit, Row & row, int width) const;
-
 	///
 	void draw(PainterInfo & pi, int x, int y) const;
-	
+
 	void drawParagraph(PainterInfo & pi, pit_type pit, int x, int y) const;
 
 	/// Returns the height of the row (width member is set to 0).
 	/// If \c topBottomSpace is true, extra space is added for the
 	/// top and bottom row.
-	Dimension rowHeight(
-		pit_type const pit,
-		pos_type const first,
-		pos_type const end,
-		bool topBottomSpace = true) const;
+	void setRowHeight(Row & row, pit_type const pit,
+			  bool topBottomSpace = true) const;
 
 private:
 	///
@@ -142,19 +134,13 @@ private:
 
 	/// sets row.end to the pos value *after* which a row should break.
 	/// for example, the pos after which isNewLine(pos) == true
-	pos_type rowBreakPoint(
-		int width,
-		pit_type const pit,
-		pos_type first
-		) const;
+	void breakRow(Row & row, int right_margin, pit_type const pit) const;
 
-	/// returns the minimum space a row needs on the screen in pixel
-	int rowWidth(
-		int right_margin,
-		pit_type const pit,
-		pos_type const first,
-		pos_type const end
-		) const;
+	// Expand the alignment of row \param row in paragraph \param par
+	LyXAlignment getAlign(Paragraph const & par, Row const & row) const;
+	/** this calculates the specified parameters. needed when setting
+	 * the cursor and when creating a visible row */
+	void computeRowMetrics(pit_type pit, Row & row, int width) const;
 
 	// Helper function for the other checkInsetHit method.
 	InsetList::InsetTable * checkInsetHit(pit_type pit, int x, int y);
@@ -162,11 +148,10 @@ private:
 
 // Temporary public:
 public:
-	/// returns the column near the specified x-coordinate of the row.
+	/// returns the position near the specified x-coordinate of the row.
 	/// x is an absolute screen coord, it is set to the real beginning
 	/// of this column.
-	pos_type getColumnNearX(pit_type pit, Row const & row, int & x,
-		bool & boundary) const;
+	pos_type getPosNearX(Row const & row, int & x, bool & boundary) const;
 
 	/// returns pos in given par at given x coord.
 	pos_type x2pos(pit_type pit, int row, int x) const;
@@ -193,7 +178,7 @@ public:
 	\param x,y are absolute screen coordinates.
 	\param assert_in_view if true the cursor will be set on a row
            that is completely visible
-    \param up whether we are going up or down (only used when
+	\param up whether we are going up or down (only used when
            assert_in_view is true
 	\retval inset is non-null if the cursor is positionned inside
 	*/
@@ -206,12 +191,16 @@ public:
 	/// x,y are screen coordinates
 	void setCursorFromCoordinates(Cursor & cur, int x, int y);
 
+	/// Helper function: find row element that contains pos, and
+	/// compute x offset.
+	Row::const_iterator const
+	findRowElement(Row const & row, pos_type const pos,
+                   bool const boundary, double & x) const;
+
 	///
-	int cursorX(CursorSlice const & cursor,
-		bool boundary) const;
+	int cursorX(CursorSlice const & cursor, bool boundary) const;
 	///
-	int cursorY(CursorSlice const & cursor,
-		bool boundary) const;
+	int cursorY(CursorSlice const & cursor, bool boundary) const;
 
 	///
 	bool cursorHome(Cursor & cur);
@@ -241,7 +230,7 @@ public:
 	int leftMargin(int max_width, pit_type pit) const;
 
 	/// calculates the position of a completion popup
-	void completionPosAndDim(Cursor const & cur, int & x, int & y, 
+	void completionPosAndDim(Cursor const & cur, int & x, int & y,
 		Dimension & dim) const;
 
 private:
@@ -268,7 +257,7 @@ public:
 	/// our 'outermost' font.
 	/// This is handed down from the surrounding
 	/// inset through the pi/mi parameter (pi.base.font)
-	/// It is used in applyOuterFont() and setCharFont() for reasons 
+	/// It is used in applyOuterFont() and setCharFont() for reasons
 	/// that are not clear... to hand hand the outermost language and
 	/// also for char style apparently.
 	Font font_;
