@@ -14,9 +14,11 @@
 #define PAINTER_H
 
 #include "support/strfwd.h"
+#include "support/types.h"
 
 namespace lyx {
 
+class Font;
 class FontInfo;
 
 namespace graphics { class Image; }
@@ -52,10 +54,9 @@ namespace frontend {
  */
 class Painter {
 public:
-	Painter() : drawing_enabled_(true) {}
+	Painter(double pixel_ratio) : drawing_enabled_(true), pixel_ratio_(pixel_ratio) {}
 
-	float line_width;
-	static const float thin_line;
+	static const int thin_line;
 
 	/// possible line styles
 	enum line_style {
@@ -63,6 +64,13 @@ public:
 		line_onoffdash //< dashes with spaces
 	};
 
+	/// possible fill styles
+	enum fill_style {
+		fill_none,
+		fill_oddeven,
+		fill_winding
+	};
+
 	/// possible character styles of preedit string.
 	/// This is used for CJK input method support.
 	enum preedit_style {
@@ -75,7 +83,7 @@ public:
 
 	/// draw a line from point to point
 	virtual void line(int x1, int y1, int x2, int y2, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		line_style = line_solid, int line_width = thin_line) = 0;
 
 	/**
 	 * lines -  draw a set of lines
@@ -84,11 +92,29 @@ public:
 	 * @param np size of the points array
 	 */
 	virtual void lines(int const * xp, int const * yp, int np, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		fill_style = fill_none, line_style = line_solid,
+		int line_width = thin_line) = 0;
+
+	/**
+	 * path -  draw a path with bezier curves
+	 * @param xp array of points' x co-ords
+	 * @param yp array of points' y co-ords
+	 * @param c1x array of first control points' x co-ords
+	 * @param c1y array of first control points' y co-ords
+	 * @param c2x array of second control points' x co-ords
+	 * @param c2y array of second control points' y co-ords
+	 * @param np size of the points array
+	 */
+	virtual void path(int const * xp, int const * yp,
+		int const * c1x, int const * c1y,
+		int const * c2x, int const * c2y,
+		int np, Color,
+		fill_style = fill_none, line_style = line_solid,
+		int line_width = thin_line) = 0;
 
 	/// draw a rectangle
 	virtual void rectangle(int x, int y, int w, int h, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		line_style = line_solid, int line_width = thin_line) = 0;
 
 	/// draw a filled rectangle
 	virtual void fillRectangle(int x, int y, int w, int h, Color) = 0;
@@ -107,11 +133,28 @@ public:
 	virtual void image(int x, int y, int w, int h,
 		graphics::Image const & image) = 0;
 
-	/// draw a string at position x, y (y is the baseline)
-	/**
-	* \return the width of the drawn text.
-	*/
-	virtual int text(int x, int y, docstring const & str, FontInfo const & f) = 0;
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is given by \c rtl.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, FontInfo const & f,
+                     bool rtl = false, double wordspacing = 0.0) = 0;
+
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is enforced by the \c Font.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, Font const & f,
+                     double wordspacing = 0.0) = 0;
+
+	/** draw a string at position x, y (y is the baseline), but
+	 * make sure that the part between \c from and \c to is in
+	 * \c other color. The text direction is enforced by the \c Font.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, Font const & f,
+                     Color other, size_type from, size_type to,
+                     double const wordspacing) = 0;
 
 	void setDrawingEnabled(bool drawing_enabled)
 	{ drawing_enabled_ = drawing_enabled; }
@@ -119,6 +162,8 @@ public:
 	/// Indicate wether real screen drawing shall be done or not.
 	bool isDrawingEnabled() const { return drawing_enabled_; }
 
+	double pixelRatio() const { return pixel_ratio_; }
+
 	/// draw a char at position x, y (y is the baseline)
 	/**
 	* \return the width of the drawn text.
@@ -155,6 +200,8 @@ public:
 private:
 	///
 	bool drawing_enabled_;
+	/// Ratio between physical pixels and device-independent pixels
+	double pixel_ratio_;
 };
 
 } // namespace frontend
