@@ -15,7 +15,7 @@
 
 #include "InsetParamsWidget.h"
 #include "ui_BoxUi.h"
-
+#include "Font.h"
 
 namespace lyx {
 namespace frontend {
@@ -30,6 +30,7 @@ public:
 private Q_SLOTS:
 	void on_innerBoxCO_activated(int);
 	void on_typeCO_activated(int);
+	void on_frameColorCO_currentIndexChanged(int);
 	void initDialog();
 	void on_widthCB_stateChanged(int state);
 	void on_heightCB_stateChanged(int state);
@@ -42,8 +43,11 @@ private:
 	FuncCode creationCode() const { return LFUN_BOX_INSERT; }
 	void paramsToDialog(Inset const *);
 	docstring dialogToParams() const;
+	bool checkWidgets(bool readonly) const;
 	//@}
 
+	/// Fill the color combos
+	void fillComboColor(QComboBox * combo, bool const is_none);
 	/// add and remove special lengths
 	void setSpecial(bool ibox);
 	/// only show valid inner box items
@@ -56,6 +60,8 @@ private:
 	QStringList ids_spec_;
 	///
 	QStringList gui_names_spec_;
+	///
+	QList<ColorCode> color_codes_;
 };
 
 } // namespace frontend
