@@ -54,6 +54,8 @@ using namespace lyx::support;
 namespace lyx {
 namespace frontend {
 
+// FIXME THREAD
+// I am guessing that it would not hurt to make these private members.
 static vector<string> citeCmds_;
 static vector<CitationStyle> citeStyles_;
 
@@ -143,7 +145,7 @@ void GuiCitation::applyView()
 	QString const before = textBeforeED->text();
 	QString const after = textAfterED->text();
 
-	apply(choice, full, force, before, after);
+	applyParams(choice, full, force, before, after);
 }
 
 
@@ -151,10 +153,40 @@ void GuiCitation::showEvent(QShowEvent * e)
 {
 	findLE->clear();
 	availableLV->setFocus();
+
+	// Set the minimal size of the QToolbox. Without this, the size of the
+	// QToolbox is only determined by values in the ui file (e.g. computed by
+	// qtcreator) and therefore causes portability and localisation issues. Note
+	// that the page widgets must have a layout with layoutSizeContraint =
+	// SetMinimumSize or similar.  KNOWN ISSUE: the calculations are incorrect
+	// the first time the dialog is shown. This problem is mitigated by the fact
+	// that LyX remembers the dialog sizes between sessions.
+	QSize minimum_size = QSize(0,0);
+	// Compute the max of the minimal sizes of the pages
+	QWidget * page;
+	for (int i = 0; (page = citationTB->widget(i)); ++i)
+		minimum_size = minimum_size.expandedTo(page->minimumSizeHint());
+	// Add the height of the tabs
+	if (citationTB->currentWidget())
+		minimum_size.rheight() += citationTB->height() -
+			citationTB->currentWidget()->height();
+	citationTB->setMinimumSize(minimum_size);
+
 	DialogView::showEvent(e);
 }
 
 
+void GuiCitation::on_citationTB_currentChanged(int i)
+{
+	if (i == 0)
+		findLE->setFocus();
+	else if (citationStyleCO->isEnabled())
+		citationStyleCO->setFocus();
+	else
+		textAfterED->setFocus();
+}
+
+
 void GuiCitation::on_okPB_clicked()
 {
 	applyView();
@@ -247,9 +279,6 @@ void GuiCitation::updateStyles(BiblioInfo const & bi)
 		return;
 	}
 
-	if (!selectedLV->selectionModel()->selectedIndexes().empty())
-		curr = selectedLV->selectionModel()->selectedIndexes()[0].row();
-
 	static const size_t max_length = 80;
 	QStringList sty = citationStyles(bi, max_length);
 
@@ -441,7 +470,7 @@ void GuiCitation::changed()
 }
 
 
-void GuiCitation::apply(int const choice, bool full, bool force,
+void GuiCitation::applyParams(int const choice, bool full, bool force,
 	QString before, QString after)
 {
 	if (cited_keys_.isEmpty())
@@ -540,6 +569,7 @@ void GuiCitation::findKey(BiblioInfo const & bi,
 	docstring field, docstring entry_type,
 	bool case_sensitive, bool reg_exp, bool reset)
 {
+	// FIXME THREAD
 	// Used for optimisation: store last searched string.
 	static QString last_searched_string;
 	// Used to disable the above optimisation.
@@ -654,18 +684,24 @@ void GuiCitation::filterByEntryType(BiblioInfo const & bi,
 static docstring escape_special_chars(docstring const & expr)
 {
 	// Search for all chars '.|*?+(){}[^$]\'
-	// Note that '[' and '\' must be escaped.
-	// This is a limitation of lyx::regex, but all other chars in BREs
-	// are assumed literal.
-	static const lyx::regex reg("[].|*?+(){}^$\\[\\\\]");
+	// Note that '[', ']', and '\' must be escaped.
+	static const lyx::regex reg("[.|*?+(){}^$\\[\\]\\\\]");
 
-	// $& is a perl-like expression that expands to all
+	// $& is an ECMAScript format expression that expands to all
 	// of the current match
-	// The '$' must be prefixed with the escape character '\' for
-	// boost to treat it as a literal.
-	// Thus, to prefix a matched expression with '\', we use:
+#if defined(LYX_USE_CXX11) && defined(LYX_USE_STD_REGEX)
+	// To prefix a matched expression with a single literal backslash, we
+	// need to escape it for the C++ compiler and use:
+	// FIXME: UNICODE
+	return from_utf8(lyx::regex_replace(to_utf8(expr), reg, string("\\$&")));
+#else
+	// A backslash in the format string starts an escape sequence in boost.
+	// Thus, to prefix a matched expression with a single literal backslash,
+	// we need to give two backslashes to the regex engine, and escape both
+	// for the C++ compiler and use:
 	// FIXME: UNICODE
 	return from_utf8(lyx::regex_replace(to_utf8(expr), reg, string("\\\\$&")));
+#endif
 }
 
 
