@@ -29,7 +29,7 @@ private Q_SLOTS:
 	///
 	void changedSlot();
 	///
-	void enableWidgets();
+	void enableWidgets() const;
 
 private:
 	/// \name InsetParamsWidget inherited methods
@@ -38,7 +38,7 @@ private:
 	FuncCode creationCode() const { return LFUN_INSET_INSERT; }
 	void paramsToDialog(Inset const *);
 	docstring dialogToParams() const;
-	bool checkWidgets() const;
+	bool checkWidgets(bool readonly) const;
 	//@}
 	///
 	bool const math_mode_;
