@@ -20,7 +20,7 @@
 #include "GuiImage.h"
 #include "qt_helpers.h"
 
-#include "FontInfo.h"
+#include "Font.h"
 #include "Language.h"
 #include "LyXRC.h"
 
@@ -34,21 +34,22 @@
 
 // Set USE_PIXMAP_CACHE to 1 for enabling the use of a Pixmap cache when
 // drawing text. This is especially useful for older PPC/Mac systems.
-#if defined(Q_WS_X11)
+#if defined(Q_WS_X11) || defined(QPA_XCB)
 #define USE_PIXMAP_CACHE 0
 #else
 #define USE_PIXMAP_CACHE 1
 #endif
 
 using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 namespace frontend {
-  
-const float Painter::thin_line = 0.0;
 
-GuiPainter::GuiPainter(QPaintDevice * device)
-	: QPainter(device), Painter(),
+const int Painter::thin_line = 1;
+
+GuiPainter::GuiPainter(QPaintDevice * device, double pixel_ratio)
+	: QPainter(device), Painter(pixel_ratio),
 	  use_pixmap_cache_(lyxrc.use_pixmap_cache && USE_PIXMAP_CACHE)
 {
 	// new QPainter has default QPen:
@@ -66,7 +67,7 @@ GuiPainter::~GuiPainter()
 
 
 void GuiPainter::setQPainterPen(QColor const & col,
-	Painter::line_style ls, float lw)
+	Painter::line_style ls, int lw)
 {
 	if (col == current_color_ && ls == current_ls_ && lw == current_lw_)
 		return;
@@ -83,13 +84,15 @@ void GuiPainter::setQPainterPen(QColor const & col,
 		case line_onoffdash: pen.setStyle(Qt::DotLine); break;
 	}
 
-	pen.setWidthF(lw);
+	pen.setWidth(lw);
 
 	setPen(pen);
 }
 
 
-QString GuiPainter::generateStringSignature(QString const & str, FontInfo const & f)
+QString GuiPainter::generateStringSignature(QString const & str,
+                                            FontInfo const & f,
+                                            double wordspacing)
 {
 	QString sig = str;
 	sig.append(QChar(static_cast<short>(f.family())));
@@ -99,6 +102,7 @@ QString GuiPainter::generateStringSignature(QString const & str, FontInfo const
 	Color const & color = f.realColor();
 	sig.append(QChar(static_cast<short>(color.baseColor)));
 	sig.append(QChar(static_cast<short>(color.mergeColor)));
+	sig.append(QString::number(wordspacing));
 	if (!monochrome_min_.empty()) {
 		QColor const & min = monochrome_min_.top();
 		QColor const & max = monochrome_max_.top();
@@ -175,7 +179,7 @@ void GuiPainter::point(int x, int y, Color col)
 void GuiPainter::line(int x1, int y1, int x2, int y2,
 	Color col,
 	line_style ls,
-	float lw)
+	int lw)
 {
 	if (!isDrawingEnabled())
 		return;
@@ -191,13 +195,15 @@ void GuiPainter::line(int x1, int y1, int x2, int y2,
 
 void GuiPainter::lines(int const * xp, int const * yp, int np,
 	Color col,
+	fill_style fs,
 	line_style ls,
-	float lw)
+	int lw)
 {
 	if (!isDrawingEnabled())
 		return;
 
 	// double the size if needed
+	// FIXME THREAD
 	static QVector<QPoint> points(32);
 	if (np > points.size())
 		points.resize(2 * np);
@@ -209,10 +215,54 @@ void GuiPainter::lines(int const * xp, int const * yp, int np,
 		if (i != 0)
 			antialias |= xp[i-1] != xp[i] && yp[i-1] != yp[i];
 	}
-	setQPainterPen(computeColor(col), ls, lw);
+	QColor const color = computeColor(col);
+	setQPainterPen(color, ls, lw);
 	bool const text_is_antialiased = renderHints() & TextAntialiasing;
 	setRenderHint(Antialiasing, antialias && text_is_antialiased);
-	drawPolyline(points.data(), np);
+	if (fs == fill_none) {
+		drawPolyline(points.data(), np);
+	} else {
+		QBrush const oldbrush = brush();
+		setBrush(QBrush(color));
+		drawPolygon(points.data(), np, fs == fill_oddeven ?
+		            Qt::OddEvenFill : Qt::WindingFill);
+		setBrush(oldbrush);
+	}
+	setRenderHint(Antialiasing, false);
+}
+
+
+void GuiPainter::path(int const * xp, int const * yp,
+	int const * c1x, int const * c1y,
+	int const * c2x, int const * c2y,
+	int np,
+	Color col,
+	fill_style fs,
+	line_style ls,
+	int lw)
+{
+	if (!isDrawingEnabled())
+		return;
+
+	QPainterPath bpath;
+	// This is the starting point, so its control points are meaningless
+	bpath.moveTo(xp[0], yp[0]);
+
+	for (int i = 1; i < np; ++i) {
+		bool line = c1x[i] == xp[i - 1] && c1y[i] == yp[i - 1] &&
+			    c2x[i] == xp[i] && c2y[i] == yp[i];
+		if (line)
+			bpath.lineTo(xp[i], yp[i]);
+		else
+			bpath.cubicTo(c1x[i], c1y[i],  c2x[i], c2y[i], xp[i], yp[i]);
+	}
+	QColor const color = computeColor(col);
+	setQPainterPen(color, ls, lw);
+	bool const text_is_antialiased = renderHints() & TextAntialiasing;
+	setRenderHint(Antialiasing, text_is_antialiased);
+	drawPath(bpath);
+	if (fs != fill_none)
+		fillPath(bpath, QBrush(color));
 	setRenderHint(Antialiasing, false);
 }
 
@@ -220,7 +270,7 @@ void GuiPainter::lines(int const * xp, int const * yp, int np,
 void GuiPainter::rectangle(int x, int y, int w, int h,
 	Color col,
 	line_style ls,
-	float lw)
+	int lw)
 {
 	if (!isDrawingEnabled())
 		return;
@@ -264,47 +314,65 @@ void GuiPainter::image(int x, int y, int w, int h, graphics::Image const & i)
 	if (!isDrawingEnabled())
 		return;
 
-	drawImage(x, y, qlimage.image(), 0, 0, w, h);
+	QImage const image = qlimage.image();
+	QRectF const drect = QRectF(x, y, w, h);
+	QRectF const srect = QRectF(0, 0, image.width(), image.height());
+	// Bilinear filtering is needed on a rare occasion for instant previews when
+	// the user's configuration mixes low-dpi and high-dpi monitors (#10114).
+	// This filter is optimised by qt on pixel-aligned images, so this does not
+	// affect performances in other cases.
+	setRenderHint(SmoothPixmapTransform);
+	drawImage(drect, image, srect);
+	setRenderHint(SmoothPixmapTransform, false);
 }
 
 
 int GuiPainter::text(int x, int y, char_type c, FontInfo const & f)
 {
-	docstring s(1, c);
-	return text(x, y, s, f);
+	return text(x, y, docstring(1, c), f);
 }
 
 
-int GuiPainter::smallCapsText(int x, int y,
-	QString const & s, FontInfo const & f)
+void GuiPainter::do_drawText(int x, int y, QString str, bool rtl, FontInfo const & f, QFont ff)
 {
-	FontInfo smallfont(f);
-	smallfont.decSize().decSize().setShape(UP_SHAPE);
-
-	QFont const & qfont = getFont(f);
-	QFont const & qsmallfont = getFont(smallfont);
-
 	setQPainterPen(computeColor(f.realColor()));
-	int textwidth = 0;
-	size_t const ls = s.length();
-	for (unsigned int i = 0; i < ls; ++i) {
-		QChar const c = s[i].toUpper();
-		if (c != s.at(i)) {
-			setFont(qsmallfont);
-		} else {
-			setFont(qfont);
-		}
-		if (isDrawingEnabled())
-			drawText(x + textwidth, y, c);
-		textwidth += fontMetrics().width(c);
-	}
-	return textwidth;
+	if (font() != ff)
+		setFont(ff);
+
+	 /* In LyX, the character direction is forced by the language.
+	  * Therefore, we have to signal that fact to Qt.
+	  */
+#if 1
+	/* Use unicode override characters to enforce drawing direction
+	 * Source: http://www.iamcal.com/understanding-bidirectional-text/
+	 */
+	if (rtl)
+		// Right-to-left override: forces to draw text right-to-left
+		str = QChar(0x202E) + str;
+	else
+		// Left-to-right override: forces to draw text left-to-right
+		str =  QChar(0x202D) + str;
+	drawText(x, y, str);
+#else
+	/* This looks like a cleaner solution, but it has drawbacks
+	 * - does not work reliably (Mac OS X, ...)
+	 * - it is not really documented
+	 * Keep it here for now, in case it can be helpful
+	 */
+	//This is much stronger than setLayoutDirection.
+	int flag = rtl ? Qt::TextForceRightToLeft : Qt::TextForceLeftToRight;
+	drawText(x + (rtl ? textwidth : 0), y - fm.maxAscent(), 0, 0,
+		 flag | Qt::TextDontClip,
+		 str);
+#endif
 }
 
 
 int GuiPainter::text(int x, int y, docstring const & s,
-		FontInfo const & f)
+                     FontInfo const & f, bool const rtl,
+                     double const wordspacing)
 {
+	//LYXERR0("text: x=" << x << ", s=" << s);
 	if (s.empty())
 		return 0;
 
@@ -328,52 +396,24 @@ int GuiPainter::text(int x, int y, docstring const & s,
 		str = ' ' + str;
 #endif
 
-	QFont const & ff = getFont(f); 
-	GuiFontMetrics const & fm = getFontMetrics(f); 
-
-	int textwidth;
-
-	if (f.realShape() == SMALLCAPS_SHAPE) {
-		textwidth = smallCapsText(x, y, str, f);
-		textDecoration(f, x, y, textwidth);
-		return textwidth;
-	}
+	QFont ff = getFont(f);
+	ff.setWordSpacing(wordspacing);
+	GuiFontMetrics const & fm = getFontMetrics(f);
 
 	// Here we use the font width cache instead of
 	//   textwidth = fontMetrics().width(str);
 	// because the above is awfully expensive on MacOSX
-	textwidth = fm.width(s);
-	textDecoration(f, x, y, textwidth);
+	// Note that we have to take in account space stretching (word spacing)
+	int const textwidth = fm.width(s) + count(s.begin(), s.end(), ' ') * wordspacing;
 
 	if (!isDrawingEnabled())
 		return textwidth;
 
-	// Qt4 does not display a glyph whose codepoint is the
-	// same as that of a soft-hyphen (0x00ad), unless it
-	// occurs at a line-break. As a kludge, we force Qt to
-	// render this glyph using a one-column line.
-	// This is needed for some math glyphs.
-	// FIXME In texted, this behaves differently depending
-	// on lyxrc.force_paint_single_char status.
-	// Should the soft hyphen char be displayed at all?
-	// I don't think so (i.e., Qt is correct as far as
-	// texted is concerned). /spitz
-	if (s.size() == 1 && str[0].unicode() == 0x00ad) {
-		setQPainterPen(computeColor(f.realColor()));
-		QTextLayout adsymbol(str);
-		adsymbol.setFont(ff);
-		adsymbol.beginLayout();
-		QTextLine line = adsymbol.createLine();
-		line.setNumColumns(1);
-		line.setPosition(QPointF(0, -line.ascent()));
-		adsymbol.endLayout();
-		line.draw(this, QPointF(x, y));
-		return textwidth;
-	}
+	textDecoration(f, x, y, textwidth);
 
 	if (use_pixmap_cache_) {
 		QPixmap pm;
-		QString key = generateStringSignature(str, f);
+		QString key = generateStringSignature(str, f, wordspacing);
 
 		// Warning: Left bearing is in general negative! Only the case
 		// where left bearing is negative is of interest WRT the
@@ -381,6 +421,8 @@ int GuiPainter::text(int x, int y, docstring const & s,
 		// Only the left bearing of the first character is important
 		// as we always write from left to right, even for
 		// right-to-left languages.
+		// FIXME: this is probably broken for RTL now that we draw full strings.
+		// Morover the first/last element is possibly not the right one since the glyph may have changed.
 		int const lb = min(fm.lbearing(s[0]), 0);
 		int const mA = fm.maxAscent();
 		if (QPixmapCache::find(key, pm)) {
@@ -397,15 +439,14 @@ int GuiPainter::text(int x, int y, docstring const & s,
 		int const mD = fm.maxDescent();
 		int const h = mA + mD;
 		if (w > 0 && h > 0) {
-			pm = QPixmap(w, h);
+			pm = QPixmap(static_cast<int>(pixelRatio() * w),
+						 static_cast<int>(pixelRatio() * h));
+#if QT_VERSION >= 0x050000
+			pm.setDevicePixelRatio(pixelRatio());
+#endif
 			pm.fill(Qt::transparent);
-			GuiPainter p(&pm);
-			p.setQPainterPen(computeColor(f.realColor()));
-			if (p.font() != ff)
-				p.setFont(ff);
-			// We need to draw the text as LTR as we use our own bidi code.
-			p.setLayoutDirection(Qt::LeftToRight);
-			p.drawText(-lb, mA, str);
+			GuiPainter p(&pm, pixelRatio());
+			p.do_drawText(-lb, mA, str, rtl, f, ff);
 			QPixmapCache::insert(key, pm);
 			//LYXERR(Debug::PAINTING, "h=" << h << "  mA=" << mA << "  mD=" << mD
 			//	<< "  w=" << w << "  lb=" << lb << "  tw=" << textwidth
@@ -413,25 +454,62 @@ int GuiPainter::text(int x, int y, docstring const & s,
 
 			// Draw the new cached pixmap.
 			drawPixmap(x + lb, y - mA, pm);
-
-			return textwidth;
+			//rectangle(x-lb, y-mA, w, h, Color_green);
 		}
+		return textwidth;
 	}
 
 	// don't use the pixmap cache,
-	// draw directly onto the painting device
-	setQPainterPen(computeColor(f.realColor()));
-	if (font() != ff)
-		setFont(ff);
-	// We need to draw the text as LTR as we use our own bidi code.
-	QPainter::setLayoutDirection(Qt::LeftToRight);
-	drawText(x, y, str);
+	do_drawText(x, y, str, rtl, f, ff);
 	//LYXERR(Debug::PAINTING, "draw " << string(str.toUtf8())
 	//	<< " at " << x << "," << y);
 	return textwidth;
 }
 
 
+int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
+                     double const wordspacing)
+{
+	return text(x, y, str, f.fontInfo(), f.isVisibleRightToLeft(), wordspacing);
+}
+
+
+int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
+                     Color other, size_type const from, size_type const to,
+                     double const wordspacing)
+{
+	GuiFontMetrics const & fm = getFontMetrics(f.fontInfo());
+	FontInfo fi = f.fontInfo();
+	bool const rtl = f.isVisibleRightToLeft();
+
+	// dimensions
+	int const ascent = fm.maxAscent();
+	int const height = fm.maxAscent() + fm.maxDescent();
+	int xmin = fm.pos2x(str, from, rtl, wordspacing);
+	int xmax = fm.pos2x(str, to, rtl, wordspacing);
+	if (xmin > xmax)
+		swap(xmin, xmax);
+
+	// First the part in other color
+	Color const orig = fi.realColor();
+	fi.setPaintColor(other);
+	QRegion const clip(x + xmin, y - ascent, xmax - xmin, height);
+	setClipRegion(clip);
+	int const textwidth = text(x, y, str, fi, rtl, wordspacing);
+
+	// Then the part in normal color
+	// Note that in Qt5, it is not possible to use Qt::UniteClip,
+	// therefore QRegion is used.
+	fi.setPaintColor(orig);
+	QRegion region(viewport());
+	setClipRegion(region - clip);
+	text(x, y, str, fi, rtl, wordspacing);
+	setClipping(false);
+
+	return textwidth;
+}
+
+
 void GuiPainter::textDecoration(FontInfo const & f, int x, int y, int width)
 {
 	if (f.underbar() == FONT_ON)
@@ -539,42 +617,37 @@ int GuiPainter::preeditText(int x, int y, char_type c,
 }
 
 
-void GuiPainter::doubleUnderline(FontInfo const & f, int x, int y, int width)
+void GuiPainter::underline(FontInfo const & f, int x, int y, int width,
+                           line_style ls)
 {
 	FontMetrics const & fm = theFontMetrics(f);
+	int const pos = fm.underlinePos();
 
-	int const below = max(fm.maxDescent() / 2, 2);
-
-	line(x, y + below, x + width, y + below, f.realColor());
-	line(x, y + below - 2, x + width, y + below - 2, f.realColor());
-}
-
-
-void GuiPainter::underline(FontInfo const & f, int x, int y, int width)
-{
-	FontMetrics const & fm = theFontMetrics(f);
-
-	int const below = max(fm.maxDescent() / 2, 2);
-	int const height = max((fm.maxDescent() / 4) - 1, 1);
-
-	if (height < 2)
-		line(x, y + below, x + width, y + below, f.realColor());
-	else
-		fillRectangle(x, y + below, width, below + height, f.realColor());
+	line(x, y + pos, x + width, y + pos,
+	     f.realColor(), ls, fm.lineWidth());
 }
 
 
 void GuiPainter::strikeoutLine(FontInfo const & f, int x, int y, int width)
 {
 	FontMetrics const & fm = theFontMetrics(f);
+	int const pos = fm.strikeoutPos();
 
-	int const middle = max((fm.maxHeight() / 4), 1);
-	int const height =  middle/3;
+	line(x, y - pos, x + width, y - pos,
+	     f.realColor(), line_solid, fm.lineWidth());
+}
 
-	if (height < 2)
-		line(x, y - middle, x + width, y - middle, f.realColor());
-	else
-		fillRectangle(x, y - middle, width, height, f.realColor());
+
+void GuiPainter::doubleUnderline(FontInfo const & f, int x, int y, int width)
+{
+	FontMetrics const & fm = theFontMetrics(f);
+	int const pos1 = fm.underlinePos() + fm.lineWidth();
+	int const pos2 = fm.underlinePos() - fm.lineWidth() + 1;
+
+	line(x, y + pos1, x + width, y + pos1,
+		 f.realColor(), line_solid, fm.lineWidth());
+	line(x, y + pos2, x + width, y + pos2,
+		 f.realColor(), line_solid, fm.lineWidth());
 }
 
 
