@@ -18,6 +18,7 @@
 #include "frontends/Painter.h"
 
 #include <QPainter>
+#include <QPainterPath>
 #include <stack>
 
 class QString;
@@ -29,11 +30,11 @@ class FontInfo;
 namespace frontend {
 
 /**
- * GuiPainter - a painter implementation for Qt4
+ * GuiPainter - a painter implementation for Qt
  */
 class GuiPainter : public QPainter, public Painter {
 public:
-	GuiPainter(QPaintDevice *);
+	GuiPainter(QPaintDevice *, double pixel_ratio);
 	virtual ~GuiPainter();
 
 	/// draw a line from point to point
@@ -41,8 +42,8 @@ public:
 		int x1, int y1,
 		int x2, int y2,
 		Color,
-		line_style = line_solid,
-		float line_width = thin_line);
+		line_style ls = line_solid,
+		int lw = thin_line);
 
 	/**
 	 * lines -  draw a set of lines
@@ -55,8 +56,26 @@ public:
 		int const * yp,
 		int np,
 		Color,
-		line_style = line_solid,
-		float line_width = thin_line);
+		fill_style fs = fill_none,
+		line_style ls = line_solid,
+		int lw = thin_line);
+
+	/**
+	 * path -  draw a path with bezier curves
+	 * @param xp array of points' x co-ords
+	 * @param yp array of points' y co-ords
+	 * @param c1x array of first control points' x co-ords
+	 * @param c1y array of first control points' y co-ords
+	 * @param c2x array of second control points' x co-ords
+	 * @param c2y array of second control points' y co-ords
+	 * @param np size of the points array
+	 */
+	virtual void path(int const * xp, int const * yp,
+		int const * c1x, int const * c1y,
+		int const * c2x, int const * c2y,
+		int np, Color,
+		fill_style = fill_none, line_style = line_solid,
+		int line_width = thin_line);
 
 	/// draw a rectangle
 	virtual void rectangle(
@@ -64,7 +83,7 @@ public:
 		int w, int h,
 		Color,
 		line_style = line_solid,
-		float line_width = thin_line);
+		int lw = thin_line);
 
 	/// draw a filled rectangle
 	virtual void fillRectangle(
@@ -86,9 +105,28 @@ public:
 	virtual void image(int x, int y, int w, int h,
 		lyx::graphics::Image const & image);
 
-	/// draw a string at position x, y (y is the baseline)
-	virtual int text(int x, int y,
-		docstring const & str, FontInfo const & f);
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is given by \c rtl.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, FontInfo const & f,
+                     bool rtl = false, double wordspacing = 0.0);
+
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is enforced by the \c Font.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, Font const & f,
+                     double wordspacing = 0.0);
+
+	/** draw a string at position x, y (y is the baseline), but
+	 * make sure that the part between \c from and \c to is in
+	 * \c other color. The text direction is enforced by the \c Font.
+	 * \return the width of the drawn text.
+	 */
+	virtual int text(int x, int y, docstring const & str, Font const & f,
+                     Color other, size_type from, size_type to,
+                     double const wordspacing);
 
 	/// draw a char at position x, y (y is the baseline)
 	virtual int text(int x, int y, char_type c, FontInfo const & f);
@@ -127,7 +165,7 @@ public:
 private:
 	/// check the font, and if set, draw an underline
 	void underline(FontInfo const & f,
-		int x, int y, int width);
+	               int x, int y, int width, line_style ls = line_solid);
 
 	/// check the font, and if set, draw an dashed underline
 	void dashedUnderline(FontInfo const & f,
@@ -144,20 +182,16 @@ private:
 	/// draw a bevelled button border
 	void buttonFrame(int x, int y, int w, int h);
 
-	/// draw small caps text
-	/**
-	\return width of the drawn text.
-	*/
-	int smallCapsText(int x, int y,
-		QString const & str, FontInfo const & f);
-
 	/// set pen parameters
 	void setQPainterPen(QColor const & col,
-		line_style ls = line_solid, float lw = thin_line);
+		line_style ls = line_solid, int lw = thin_line);
+
+	// Helper for text() method
+	void do_drawText(int x, int y, QString str, bool rtl, FontInfo const & f, QFont ff);
 
 	QColor current_color_;
 	Painter::line_style current_ls_;
-	float current_lw_;
+	int current_lw_;
 	///
 	bool const use_pixmap_cache_;
 	///
@@ -169,7 +203,8 @@ private:
 	/// possibly apply monochrome mode
 	QColor filterColor(QColor const & col);
 	///
-	QString generateStringSignature(QString const & str, FontInfo const & f);	
+	QString generateStringSignature(QString const & str, FontInfo const & f,
+	                                double wordspacing);
 };
 
 } // namespace frontend
