@@ -75,7 +75,7 @@ void GuiToc::enableView(bool /*enable*/)
 void GuiToc::closeEvent(QCloseEvent * /*event*/)
 {
 	is_closing_ = true;
-	((GuiView *)parent())->updateToolbars();
+	static_cast<GuiView *>(parent())->updateToolbars();
 	is_closing_ = false;
 }
 
@@ -95,15 +95,7 @@ bool GuiToc::getStatus(Cursor & cur, FuncRequest const & cmd,
 
 Dialog * createGuiToc(GuiView & lv)
 {
-	GuiToc * toc;
-#ifdef Q_WS_MACX
-	// On Mac show at the right and floating
-	toc = new GuiToc(lv, Qt::RightDockWidgetArea);
-	toc->setFloating(true);
-#else
-	toc = new GuiToc(lv);
-#endif
-	return toc;
+	return new GuiToc(lv);
 }
 
 
