@@ -49,6 +49,7 @@
 #include "support/gettext.h"
 #include "support/FileName.h"
 #include "support/lassert.h"
+#include "support/TempFile.h"
 
 #include "frontends/Application.h"
 #include "frontends/FontMetrics.h"
@@ -60,8 +61,8 @@
 #endif
 #include <QDrag>
 #include <QHelpEvent>
-#ifdef Q_WS_MACX
-#include <QMacStyle>
+#ifdef Q_OS_MAC
+#include <QProxyStyle>
 #endif
 #include <QMainWindow>
 #include <QMimeData>
@@ -131,7 +132,9 @@ namespace frontend {
 
 class CursorWidget {
 public:
-	CursorWidget() {
+	CursorWidget() : rtl_(false), l_shape_(false), completable_(false),
+		show_(false), x_(0), cursor_width_(0)
+	{
 		recomputeWidth();
 	}
 
@@ -243,9 +246,11 @@ SyntheticMouseEvent::SyntheticMouseEvent()
 
 
 GuiWorkArea::Private::Private(GuiWorkArea * parent)
-: p(parent), screen_(0), buffer_view_(0), lyx_view_(0), cursor_visible_(false),
+: p(parent), screen_(0), buffer_view_(0), read_only_(false), lyx_view_(0),
+cursor_visible_(false), cursor_(0),
 need_resize_(false), schedule_redraw_(false), preedit_lines_(1),
-completer_(new GuiCompleter(p, p))
+pixel_ratio_(1.0),
+completer_(new GuiCompleter(p, p)), dialog_mode_(false)
 {
 }
 
@@ -260,11 +265,22 @@ GuiWorkArea::GuiWorkArea(Buffer & buffer, GuiView & gv)
 : d(new Private(this))
 {
 	setGuiView(gv);
+	buffer.params().display_pixel_ratio = theGuiApp()->pixelRatio();
 	setBuffer(buffer);
 	init();
 }
 
 
+double GuiWorkArea::pixelRatio() const
+{
+#if QT_VERSION >= 0x050000
+	return devicePixelRatio();
+#else
+	return 1.0;
+#endif
+}
+
+
 void GuiWorkArea::init()
 {
 	// Setup the signals
@@ -627,6 +643,12 @@ void GuiWorkArea::Private::showCursor()
 		&& !completer_->inlineVisible();
 	cursor_visible_ = true;
 	cursor_->recomputeWidth();
+
+	//int cur_x = buffer_view_->getPos(cur).x_;
+	// We may have decided to slide the cursor row so that cursor
+	// is visible.
+	p.x_ -= buffer_view_->horizScrollOffset();
+
 	showCursor(p.x_, p.y_, h, l_shape, isrtl, completable);
 }
 
@@ -676,6 +698,7 @@ void GuiWorkArea::scrollTo(int value)
 	}
 	// Show the cursor immediately after any operation.
 	startBlinkingCursor();
+	// FIXME QT5
 #ifdef Q_WS_X11
 	QApplication::syncX();
 #endif
@@ -798,7 +821,7 @@ void GuiWorkArea::mousePressEvent(QMouseEvent * e)
 		return;
 	}
 
-#if (QT_VERSION < 0x050000)
+#if (QT_VERSION < 0x050000) && !defined(__HAIKU__)
 	inputContext()->reset();
 #endif
 
@@ -997,11 +1020,11 @@ void GuiWorkArea::generateSyntheticMouseEvent()
 			break;
 		yy += h;
 	}
-	
+
 	// Find the position of the cursor
 	bool bound;
 	int x = d->synthetic_mouse_event_.cmd.x();
-	pos_type const pos = rit->pos() + tm.getColumnNearX(pit, *rit, x, bound);
+	pos_type const pos = tm.getPosNearX(*rit, x, bound);
 
 	// Set the cursor
 	cur.pit() = pit;
@@ -1041,7 +1064,7 @@ void GuiWorkArea::keyPressEvent(QKeyEvent * ev)
 	// do nothing if there are other events
 	// (the auto repeated events come too fast)
 	// it looks like this is only needed on X11
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) || defined(QPA_XCB)
 	if (qApp->hasPendingEvents() && ev->isAutoRepeat()) {
 		switch (ev->key()) {
 		case Qt::Key_PageDown:
@@ -1119,11 +1142,11 @@ void GuiWorkArea::Private::update(int x, int y, int w, int h)
 
 void GuiWorkArea::paintEvent(QPaintEvent * ev)
 {
-	QRect const rc = ev->rect();
+	QRectF const rc = ev->rect();
 	// LYXERR(Debug::PAINTING, "paintEvent begin: x: " << rc.x()
 	//	<< " y: " << rc.y() << " w: " << rc.width() << " h: " << rc.height());
 
-	if (d->need_resize_) {
+	if (d->needResize()) {
 		d->resetScreen();
 		d->resizeBufferView();
 		if (d->cursor_visible_) {
@@ -1133,10 +1156,15 @@ void GuiWorkArea::paintEvent(QPaintEvent * ev)
 	}
 
 	QPainter pain(viewport());
+	double const pr = pixelRatio();
+	QRectF const rcs = QRectF(rc.x() * pr, rc.y() * pr, rc.width() * pr, rc.height() * pr);
+
 	if (lyxrc.use_qimage) {
-		pain.drawImage(rc, static_cast<QImage const &>(*d->screen_), rc);
+		QImage const & image = static_cast<QImage const &>(*d->screen_);
+		pain.drawImage(rc, image, rcs);
 	} else {
-		pain.drawPixmap(rc, static_cast<QPixmap const &>(*d->screen_), rc);
+		QPixmap const & pixmap = static_cast<QPixmap const &>(*d->screen_);
+		pain.drawPixmap(rc, pixmap, rcs);
 	}
 	d->cursor_->draw(pain);
 	ev->accept();
@@ -1145,7 +1173,7 @@ void GuiWorkArea::paintEvent(QPaintEvent * ev)
 
 void GuiWorkArea::Private::updateScreen()
 {
-	GuiPainter pain(screen_);
+	GuiPainter pain(screen_, p->pixelRatio());
 	buffer_view_->draw(pain);
 }
 
@@ -1210,6 +1238,8 @@ void GuiWorkArea::inputMethodEvent(QInputMethodEvent * e)
 		stopBlinkingCursor();
 
 	// last_width : for checking if last preedit string was/wasn't empty.
+	// FIXME THREAD
+	// We could have more than one work area, right?
 	static bool last_width = false;
 	if (!last_width && preedit_string.empty()) {
 		// if last_width is last length of preedit string.
@@ -1217,7 +1247,7 @@ void GuiWorkArea::inputMethodEvent(QInputMethodEvent * e)
 		return;
 	}
 
-	GuiPainter pain(d->screen_);
+	GuiPainter pain(d->screen_, pixelRatio());
 	d->buffer_view_->updateMetrics();
 	d->buffer_view_->draw(pain);
 	FontInfo font = d->buffer_view_->cursor().getFont().fontInfo();
@@ -1424,8 +1454,9 @@ GuiView & GuiWorkArea::view()
 
 EmbeddedWorkArea::EmbeddedWorkArea(QWidget * w): GuiWorkArea(w)
 {
-	buffer_ = theBufferList().newInternalBuffer(
-		support::FileName::tempName("embedded.internal").absFileName());
+	support::TempFile tempfile("embedded.internal");
+	tempfile.setAutoRemove(false);
+	buffer_ = theBufferList().newInternalBuffer(tempfile.name().absFileName());
 	buffer_->setUnnamed(true);
 	buffer_->setFullyLoaded(true);
 	setBuffer(*buffer_);
@@ -1483,14 +1514,14 @@ void EmbeddedWorkArea::disable()
 //
 ////////////////////////////////////////////////////////////////////
 
-#ifdef Q_WS_MACX
-class NoTabFrameMacStyle : public QMacStyle {
+#ifdef Q_OS_MAC
+class NoTabFrameMacStyle : public QProxyStyle {
 public:
 	///
 	QRect subElementRect(SubElement element, const QStyleOption * option,
 			     const QWidget * widget = 0) const
 	{
-		QRect rect = QMacStyle::subElementRect(element, option, widget);
+		QRect rect = QProxyStyle::subElementRect(element, option, widget);
 		bool noBar = static_cast<QTabWidget const *>(widget)->count() <= 1;
 
 		// The Qt Mac style puts the contents into a 3 pixel wide box
@@ -1514,7 +1545,7 @@ NoTabFrameMacStyle noTabFrameMacStyle;
 TabWorkArea::TabWorkArea(QWidget * parent)
 	: QTabWidget(parent), clicked_tab_(-1)
 {
-#ifdef Q_WS_MACX
+#ifdef Q_OS_MAC
 	setStyle(&noTabFrameMacStyle);
 #endif
 
@@ -1532,7 +1563,7 @@ TabWorkArea::TabWorkArea(QWidget * parent)
 	closeBufferButton = new QToolButton(this);
 	closeBufferButton->setPalette(pal);
 	// FIXME: rename the icon to closebuffer.png
-	closeBufferButton->setIcon(QIcon(getPixmap("images/", "closetab", "png")));
+	closeBufferButton->setIcon(QIcon(getPixmap("images/", "closetab", "svgz,png")));
 	closeBufferButton->setText("Close File");
 	closeBufferButton->setAutoRaise(true);
 	closeBufferButton->setCursor(Qt::ArrowCursor);
@@ -1824,7 +1855,7 @@ public:
 
 		if (!dotted) {
 			if (dottedPrefix_ && !prefix_.isEmpty())
-				prefix_ += ".../";
+				prefix_ += ellipsisSlash_;
 			prefix_ += postfix_.front() + "/";
 		}
 		dottedPrefix_ = dotted && !prefix_.isEmpty();
@@ -1837,7 +1868,7 @@ public:
 			return filename_;
 
 		bool dots = dottedPrefix_ || !postfix_.isEmpty();
-		return prefix_ + (dots ? ".../" : "") + filename_;
+		return prefix_ + (dots ? ellipsisSlash_ : "") + filename_;
 	}
 	///
 	QString forecastPathString() const
@@ -1846,7 +1877,7 @@ public:
 			return displayString();
 
 		return prefix_
-			+ (dottedPrefix_ ? ".../" : "")
+			+ (dottedPrefix_ ? ellipsisSlash_ : "")
 			+ postfix_.front() + "/";
 	}
 	///
@@ -1855,6 +1886,8 @@ public:
 	int tab() const { return tab_; }
 
 private:
+	/// ".../"
+	static QString const ellipsisSlash_;
 	///
 	QString prefix_;
 	///
@@ -1870,6 +1903,9 @@ private:
 };
 
 
+QString const DisplayPath::ellipsisSlash_ = QString(QChar(0x2026)) + "/";
+
+
 ///
 bool operator<(DisplayPath const & a, DisplayPath const & b)
 {
@@ -1983,8 +2019,14 @@ void TabWorkArea::updateTabTexts()
 		QString tab_text = it->displayString().replace("&", "&&");
 		if (!buf.fileName().empty() && !buf.isClean())
 			tab_text += "*";
+		QString tab_tooltip = it->abs();
+		if (buf.isReadonly()) {
+			setTabIcon(tab_index, QIcon(getPixmap("images/", "emblem-readonly", "svgz,png")));
+			tab_tooltip = qt_("%1 (read only)").arg(it->abs());
+		} else
+			setTabIcon(tab_index, QIcon());
 		setTabText(tab_index, tab_text);
-		setTabToolTip(tab_index, it->abs());
+		setTabToolTip(tab_index, tab_tooltip);
 	}
 }
 
@@ -1998,9 +2040,9 @@ void TabWorkArea::showContextMenu(const QPoint & pos)
 
 	// show tab popup
 	QMenu popup;
-	popup.addAction(QIcon(getPixmap("images/", "hidetab", "png")),
+	popup.addAction(QIcon(getPixmap("images/", "hidetab", "svgz,png")),
 		qt_("Hide tab"), this, SLOT(hideCurrentTab()));
-	popup.addAction(QIcon(getPixmap("images/", "closetab", "png")),
+	popup.addAction(QIcon(getPixmap("images/", "closetab", "svgz,png")),
 		qt_("Close tab"), this, SLOT(closeCurrentBuffer()));
 	popup.exec(tabBar()->mapToGlobal(pos));
 
