@@ -354,7 +354,7 @@ public:
 	void expandFloatListInsert(Buffer const * buf);
 	void expandFloatInsert(Buffer const * buf);
 	void expandFlexInsert(Buffer const * buf, InsetLayout::InsetLyXType type);
-	void expandToc2(Toc const & toc_list, size_t from, size_t to, int depth);
+	void expandToc2(Toc const & toc_list, size_t from, size_t to, int depth, string toc_type);
 	void expandToc(Buffer const * buf);
 	void expandPasteRecent(Buffer const * buf);
 	void expandToolbars();
@@ -417,8 +417,8 @@ void MenuDefinition::addWithStatusCheck(MenuItem const & i)
 				  cit != i.submenu().end(); ++cit) {
 				// Only these kind of items affect the status of the submenu
 				if ((cit->kind() == MenuItem::Command
-					|| cit->kind() == MenuItem::Submenu
-					|| cit->kind() == MenuItem::Help)
+				     || cit->kind() == MenuItem::Submenu
+				     || cit->kind() == MenuItem::Help)
 				    && cit->status().enabled()) {
 					enabled = true;
 					break;
@@ -769,11 +769,9 @@ bool MenuDefinition::searchMenu(FuncRequest const & func, docstring_list & names
 QString limitStringLength(docstring const & str)
 {
 	size_t const max_item_length = 45;
-
-	if (str.size() > max_item_length)
-		return toqstr(str.substr(0, max_item_length - 3) + "...");
-
-	return toqstr(str);
+	docstring ret = str.substr(0, max_item_length + 1);
+	support::truncateWithEllipsis(ret, max_item_length);
+	return toqstr(ret);
 }
 
 
@@ -1123,12 +1121,10 @@ void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf
 			label += '|' + shortcut;
 
 		if (view_update) {
-			if (buf)
-				item.submenu().addWithStatusCheck(MenuItem(MenuItem::Command,
-					toqstr(label), FuncRequest(action, (*fit)->name())));
-			else
-				item.submenu().add(MenuItem(MenuItem::Command, toqstr(label),
-					FuncRequest(action, (*fit)->name())));
+			// note that at this point, we know that buf is not null
+			LATTEST(buf);
+			item.submenu().addWithStatusCheck(MenuItem(MenuItem::Command,
+				toqstr(label), FuncRequest(action, (*fit)->name())));
 		} else {
 			if (buf)
 				addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
@@ -1205,9 +1201,11 @@ void MenuDefinition::expandFlexInsert(
 	TextClass::InsetLayouts::const_iterator end = insetLayouts.end();
 	for (; cit != end; ++cit) {
 		if (cit->second.lyxtype() == type) {
+			if (!cit->second.obsoleted_by().empty())
+				continue;
 			docstring label = cit->first;
 			// we remove the "Flex:" prefix, if it is present
-			if (prefixIs(label, from_utf8("Flex:")))
+			if (prefixIs(label, from_ascii("Flex:")))
 				label = label.substr(5);
 			addWithStatusCheck(MenuItem(MenuItem::Command,
 				toqstr(translateIfPossible(label)),
@@ -1215,15 +1213,23 @@ void MenuDefinition::expandFlexInsert(
 		}
 	}
 	// FIXME This is a little clunky.
-	if (items_.empty() && type == InsetLayout::CUSTOM)
+	if (items_.empty() && type == InsetLayout::CUSTOM && !buf->isReadonly())
 		add(MenuItem(MenuItem::Help, qt_("No Custom Insets Defined!")));
 }
 
 
-size_t const max_number_of_items = 25;
+// Threshold before we stop displaying sub-items alongside items
+// (for display purposes). Ideally this should fit on a screen.
+size_t const max_number_of_items = 30;
+// Size limit for the menu. This is for performance purposes,
+// because qt already displays a scrollable menu when necessary.
+// Ideally this should be the menu size from which scrollable
+// menus become unpractical.
+size_t const menu_size_limit = 80;
 
 void MenuDefinition::expandToc2(Toc const & toc_list,
-		size_t from, size_t to, int depth)
+                                size_t from, size_t to, int depth,
+                                string toc_type)
 {
 	int shortcut_count = 0;
 
@@ -1237,7 +1243,7 @@ void MenuDefinition::expandToc2(Toc const & toc_list,
 	if (to - from <= max_number_of_items) {
 		for (size_t i = from; i < to; ++i) {
 			QString label(4 * max(0, toc_list[i].depth() - depth), ' ');
-			label += limitStringLength(toc_list[i].str());
+			label += limitStringLength(toc_list[i].asString());
 			if (toc_list[i].depth() == depth) {
 				label += '|';
 			    if (shortcut_count < 9) {
@@ -1247,16 +1253,20 @@ void MenuDefinition::expandToc2(Toc const & toc_list,
 			}
 			add(MenuItem(MenuItem::Command, label,
 					    FuncRequest(toc_list[i].action())));
+			// separator after the menu heading
+			if (toc_list[i].depth() < depth)
+				add(MenuItem(MenuItem::Separator));
 		}
 	} else {
 		size_t pos = from;
+		size_t size = 1;
 		while (pos < to) {
 			size_t new_pos = pos + 1;
 			while (new_pos < to && toc_list[new_pos].depth() > depth)
 				++new_pos;
 
 			QString label(4 * max(0, toc_list[pos].depth() - depth), ' ');
-			label += limitStringLength(toc_list[pos].str());
+			label += limitStringLength(toc_list[pos].asString());
 			if (toc_list[pos].depth() == depth) {
 				label += '|';
 			    if (shortcut_count < 9) {
@@ -1264,16 +1274,22 @@ void MenuDefinition::expandToc2(Toc const & toc_list,
 						label += QString::number(++shortcut_count);
 				}
 			}
+			if (size >= menu_size_limit) {
+				FuncRequest f(LFUN_DIALOG_SHOW, "toc " + toc_type);
+				add(MenuItem(MenuItem::Command, "...", f));
+				break;
+			}
 			if (new_pos == pos + 1) {
 				add(MenuItem(MenuItem::Command,
 						    label, FuncRequest(toc_list[pos].action())));
 			} else {
 				MenuDefinition sub;
-				sub.expandToc2(toc_list, pos, new_pos, depth + 1);
+				sub.expandToc2(toc_list, pos, new_pos, depth + 1, toc_type);
 				MenuItem item(MenuItem::Submenu, label);
 				item.setSubmenu(sub);
 				add(item);
 			}
+			++size;
 			pos = new_pos;
 		}
 	}
@@ -1286,12 +1302,10 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	// all MenuItem constructors and to expandToc2. However, we
 	// know that all the entries in a TOC will be have status_ ==
 	// OK, so we avoid this unnecessary overhead (JMarc)
-
 	if (!buf) {
-		add(MenuItem(MenuItem::Info, qt_("<No Document Open>")));
+		add(MenuItem(MenuItem::Info, qt_("(No Document Open)")));
 		return;
 	}
-
 	// Add an entry for the master doc if this is a child doc
 	Buffer const * const master = buf->masterBuffer();
 	if (buf != master) {
@@ -1302,37 +1316,27 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	}
 
 	MenuDefinition other_lists;
-
 	FloatList const & floatlist = buf->params().documentClass().floats();
 	TocList const & toc_list = buf->tocBackend().tocs();
 	TocList::const_iterator cit = toc_list.begin();
 	TocList::const_iterator end = toc_list.end();
 	for (; cit != end; ++cit) {
-		// Handle this later
-		if (cit->first == "tableofcontents")
+		// Handle table of contents later
+		if (cit->first == "tableofcontents" || cit->second->empty())
 			continue;
-
 		MenuDefinition submenu;
-		if (cit->second.size() >= 30) {
-			FuncRequest f(LFUN_DIALOG_SHOW, "toc " + cit->first);
-			submenu.add(MenuItem(MenuItem::Command, qt_("Open Navigator..."), f));
-		} else {
-			TocIterator ccit = cit->second.begin();
-			TocIterator eend = cit->second.end();
-			for (; ccit != eend; ++ccit) {
-				submenu.add(MenuItem(MenuItem::Command,
-					limitStringLength(ccit->str()) + '|',
-					FuncRequest(ccit->action())));
-			}
-		}
-
+		// "Open outliner..." entry
+		FuncRequest f(LFUN_DIALOG_SHOW, "toc " + cit->first);
+		submenu.add(MenuItem(MenuItem::Command, qt_("Open Outliner..."), f));
+		submenu.add(MenuItem(MenuItem::Separator));
+		// add entries
+		submenu.expandToc2(*cit->second, 0, cit->second->size(), 0, cit->first);
 		MenuItem item(MenuItem::Submenu, guiName(cit->first, buf->params()));
 		item.setSubmenu(submenu);
-		if (floatlist.typeExist(cit->first) || cit->first == "child") {
-			// Those two types deserve to be in the main menu.
-			item.setSubmenu(submenu);
+		// deserves to be in the main menu?
+		if (floatlist.typeExist(cit->first) || cit->first == "child")
 			add(item);
-		} else
+		else
 			other_lists.add(item);
 	}
 	if (!other_lists.empty()) {
@@ -1340,16 +1344,17 @@ void MenuDefinition::expandToc(Buffer const * buf)
 		item.setSubmenu(other_lists);
 		add(item);
 	}
-
 	// Handle normal TOC
+	add(MenuItem(MenuItem::Separator));
 	cit = toc_list.find("tableofcontents");
 	if (cit == end)
 		LYXERR(Debug::GUI, "No table of contents.");
 	else {
-		if (!cit->second.empty())
-			expandToc2(cit->second, 0, cit->second.size(), 0);
+		if (!cit->second->empty())
+			expandToc2(*cit->second, 0, cit->second->size(), 0,
+			           "tableofcontents");
 		else
-			add(MenuItem(MenuItem::Info, qt_("<Empty Table of Contents>")));
+			add(MenuItem(MenuItem::Info, qt_("(Empty Table of Contents)")));
 	}
 }
 
@@ -1393,7 +1398,7 @@ void MenuDefinition::expandToolbars()
 
 void MenuDefinition::expandBranches(Buffer const * buf)
 {
-	if (!buf)
+	if (!buf || buf->isReadonly())
 		return;
 
 	BufferParams const & master_params = buf->masterBuffer()->params();
@@ -1766,7 +1771,7 @@ void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 		else if (m->kind() == MenuItem::Submenu) {
 			QMenu * subMenu = qMenu.addMenu(label(*m));
 			populate(*subMenu, m->submenu());
-			subMenu->setEnabled(m->status().enabled());
+			subMenu->setEnabled(!subMenu->isEmpty());
 		} else {
 			// we have a MenuItem::Command
 			qMenu.addAction(new Action(QIcon(), label(*m),
@@ -1775,7 +1780,7 @@ void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 	}
 }
 
-#if defined(Q_WS_WIN) && (QT_VERSION >= 0x040600)
+#if (defined(Q_OS_WIN) || defined(Q_CYGWIN_WIN)) && (QT_VERSION >= 0x040600)
 class AlwaysMnemonicStyle : public QProxyStyle {
 public:
 	int styleHint(StyleHint hint, const QStyleOption *opt = 0, const QWidget *widget = 0,
@@ -1795,7 +1800,7 @@ public:
 Menu::Menu(GuiView * gv, QString const & name, bool top_level, bool keyboard)
 : QMenu(gv), d(new Menu::Impl)
 {
-#if defined(Q_WS_WIN) && (QT_VERSION >= 0x040600)
+#if (defined(Q_OS_WIN) || defined(Q_CYGWIN_WIN)) && (QT_VERSION >= 0x040600)
 	if (keyboard)
 		setStyle(new AlwaysMnemonicStyle);
 #else
@@ -1945,6 +1950,7 @@ void Menus::Impl::macxMenuBarInit(QMenuBar * qmb)
 	const size_t num_entries = sizeof(entries) / sizeof(entries[0]);
 	const bool first_call = mac_special_menu_.empty();
 
+	LYXERR(Debug::GUI, "Creating Mac OS X special menu bar");
 	// the special menu for Menus. Fill it up only once.
 	if (first_call) {
 		for (size_t i = 0 ; i < num_entries ; ++i) {
@@ -2157,7 +2163,7 @@ Menus::Menus() : d(new Impl) {}
 
 Menus::~Menus()
 {
-  delete d;
+	delete d;
 }
 
 
@@ -2232,17 +2238,19 @@ bool Menus::searchMenu(FuncRequest const & func,
 void Menus::fillMenuBar(QMenuBar * qmb, GuiView * view, bool initial)
 {
 	if (initial) {
-#ifdef Q_WS_MACX
+#ifdef Q_OS_MAC
 		// setup special mac specific menu items, but only do this
 		// the first time a QMenuBar is created. Otherwise Qt will
 		// create duplicate items in the application menu. It seems
 		// that Qt does not remove them when the QMenubar is cleared.
-		LYXERR(Debug::GUI, "Creating Mac OS X special menu bar");
 		d->macxMenuBarInit(qmb);
 #endif
 	} else {
 		// Clear all menubar contents before filling it.
 		qmb->clear();
+#if (QT_VERSION >= 0x050000 && defined(Q_OS_MAC))
+		d->macxMenuBarInit(qmb);
+#endif
 	}
 
 	LYXERR(Debug::GUI, "populating menu bar" << d->menubar_.name());
@@ -2283,7 +2291,7 @@ void Menus::fillMenuBar(QMenuBar * qmb, GuiView * view, bool initial)
 		Menu * menu = new Menu(view, m->submenuname(), true);
 		menu->setTitle(label(*m));
 
-#if defined(Q_WS_MACX) && (defined(QT_MAC_USE_COCOA) || (QT_VERSION >= 0x050000))
+#if defined(Q_OS_MAC) && (defined(QT_MAC_USE_COCOA) || (QT_VERSION >= 0x050000))
 		// On Mac OS with QT/cocoa, the menu is not displayed if there is no action
 		// so we create a temporary one here
 		QAction * action = new QAction(menu);
