@@ -194,7 +194,7 @@ Buffer & Inset::buffer()
 		string const iname = insetName(lyxCode());
 		LYXERR0("Inset: " << this << " LyX Code: " << lyxCode()
 					<< " name: " << iname);
-		s << "LyX Code: " << lyxCode() << " name: " << iname;
+		s << "LyX Code: " << lyxCode() << " name: " << from_ascii(iname);
 		LATTEST(false);
 		throw ExceptionMessage(BufferException, 
 			from_ascii("Inset::buffer_ member not initialized!"), s.str());
@@ -258,7 +258,7 @@ docstring Inset::toolTip(BufferView const &, int, int) const
 }
 
 
-void Inset::forToc(docstring &, size_t) const
+void Inset::forOutliner(docstring &, size_t const, bool const) const
 {
 }
 
@@ -388,11 +388,6 @@ bool Inset::getStatus(Cursor &, FuncRequest const & cmd,
 		// Allow modification of our data.
 		// This needs to be handled in the doDispatch method of our
 		// instantiatable children.
-		// FIXME: Why don't we let the insets determine whether this
-		// should be enabled or not ? Now we need this check for 
-		// the tabular features. (vfr)
-		if (cmd.getArg(0) == "tabular")
-			return false;
 		flag.setEnabled(true);
 		return true;
 
