@@ -74,7 +74,7 @@ std::string insetName(InsetCode);
 /// Eg, insetDisplayName(BRANCH_CODE) == _("Branch")
 docstring insetDisplayName(InsetCode);
 ///
-static int const TOC_ENTRY_LENGTH = 128;
+static int const TOC_ENTRY_LENGTH = 120;
 
 /// Common base class to all insets
 
@@ -108,8 +108,6 @@ public:
 
 	/// change associated Buffer
 	virtual void setBuffer(Buffer & buffer);
-	/// remove the buffer reference
-	void resetBuffer() { setBuffer( *static_cast<Buffer *>(0)); }
 	/// retrieve associated Buffer
 	virtual Buffer & buffer();
 	virtual Buffer const & buffer() const;
@@ -329,8 +327,9 @@ public:
 	virtual void toString(odocstream &) const {}
 	/// Appends a potentially abbreviated version of the inset to
 	/// \param str. Intended for use by the TOC.
-	virtual void forToc(docstring & str,
-	                    size_t maxlen = TOC_ENTRY_LENGTH) const;
+	virtual void forOutliner(docstring & str,
+							 size_t const maxlen = TOC_ENTRY_LENGTH,
+							 bool const shorten = true) const;
 
 	/// can the contents of the inset be edited on screen ?
 	// true for InsetCollapsables (not ButtonOnly) (not InsetInfo), InsetText
@@ -436,8 +435,8 @@ public:
 	/// is this equivalent to a space (which is BTW different from
 	/// a line separator)?
 	virtual bool isSpace() const { return false; }
-	/// is this an expandible space (rubber length)?
-	virtual bool isStretchableSpace() const { return false; }
+	/// does this inset try to use all available space (like \\hfill does)?
+	virtual bool isHfill() const { return false; }
 
 	enum DisplayType {
 		Inline = 0,
@@ -495,7 +494,15 @@ public:
 	/// Add an entry to the TocList
 	/// Pass a DocIterator that points at the paragraph containing
 	/// the inset
-	virtual void addToToc(DocIterator const & /* di */, bool /* output_active */) const {}
+	///
+	/// \param output_active : is the inset active or is it in an inactive
+	/// branch or a note?
+	///
+	/// \param utype : is the toc being generated for use by the output
+	/// routines?
+	virtual void addToToc(DocIterator const & /* di */,
+						  bool /* output_active */,
+						  UpdateType /* utype*/) const {}
 	/// Collect BibTeX information
 	virtual void collectBibKeys(InsetIterator const &) const {}
 	/// Update the counters of this inset and of its contents.
