@@ -6,6 +6,7 @@
  * \author Angus Leeming
  * \author Martin Vermeer
  * \author Jürgen Spitzmüller
+ * \author Uwe Stöhr
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -78,14 +79,14 @@ BoxTranslatorLoc initBoxtranslatorLoc()
 
 BoxTranslator const & boxtranslator()
 {
-	static BoxTranslator translator = initBoxtranslator();
+	static BoxTranslator const translator = initBoxtranslator();
 	return translator;
 }
 
 
 BoxTranslatorLoc const & boxtranslator_loc()
 {
-	static BoxTranslatorLoc translator = initBoxtranslatorLoc();
+	static BoxTranslatorLoc const translator = initBoxtranslatorLoc();
 	return translator;
 }
 
@@ -157,12 +158,25 @@ void InsetBox::setButtonLabel()
 		label = bformat(_("%1$s (%2$s, %3$s)"),
 			type, inner, frame);
 	setLabel(label);
+
+	// set the frame color for the inset if the type is Boxed
+	if (btype == Boxed)
+		setFrameColor(lcolor.getFromLaTeXName(params_.framecolor));
+	else
+		setFrameColor(Color_collapsableframe);
 }
 
 
 bool InsetBox::hasFixedWidth() const
 {
-	return !params_.width.empty() && params_.special == "none";
+	return !params_.width.empty();
+}
+
+
+bool InsetBox::allowMultiPar() const
+{
+	return (params_.inner_box && !params_.use_makebox)
+		|| params_.type == "Shaded" || params_.type == "Framed";
 }
 
 
@@ -171,7 +185,7 @@ void InsetBox::metrics(MetricsInfo & m, Dimension & dim) const
 	// back up textwidth.
 	int textwidth_backup = m.base.textwidth;
 	if (hasFixedWidth())
-		m.base.textwidth = params_.width.inPixels(m.base.textwidth);
+		m.base.textwidth = params_.width.inPixels(m.base);
 	InsetCollapsable::metrics(m, dim);
 	// retore textwidth.
 	m.base.textwidth = textwidth_backup;
@@ -187,15 +201,50 @@ bool InsetBox::forcePlainLayout(idx_type) const
 
 ColorCode InsetBox::backgroundColor(PainterInfo const &) const
 {
-	if (params_.type != "Shaded")
+	// we only support background color for 3 types
+	if (params_.type != "Shaded" && params_.type != "Frameless" && params_.type != "Boxed")
 		return getLayout().bgcolor();
-	// FIXME: This hardcoded color is a hack!
-	if (buffer().params().boxbgcolor == lyx::rgbFromHexName("#ff0000"))
-		return getLayout().bgcolor();
-	ColorCode c = lcolor.getFromLyXName("boxbgcolor");
-	if (c == Color_none)
-		return getLayout().bgcolor();
-	return c;
+
+	if (params_.type == "Shaded") {
+		// FIXME: This hardcoded color is a hack!
+		if (buffer().params().boxbgcolor == lyx::rgbFromHexName("#ff0000"))
+			return getLayout().bgcolor();
+
+		ColorCode c = lcolor.getFromLyXName("boxbgcolor");
+		if (c == Color_none)
+			return getLayout().bgcolor();
+		return c;
+	}
+
+	if (params_.backgroundcolor != "none")
+		return lcolor.getFromLaTeXName(params_.backgroundcolor);
+
+	return getLayout().bgcolor();
+}
+
+
+LyXAlignment InsetBox::contentAlignment() const
+{
+	if (!params_.use_makebox)
+		return LYX_ALIGN_NONE;
+
+	// The default value below is actually irrelevant
+	LyXAlignment align = LYX_ALIGN_NONE;
+	switch (params_.hor_pos) {
+	case 'l':
+		align = LYX_ALIGN_LEFT;
+		break;
+	case 'c':
+		align = LYX_ALIGN_CENTER;
+		break;
+	case 'r':
+		align = LYX_ALIGN_RIGHT;
+		break;
+	case 's':
+		align = LYX_ALIGN_BLOCK;
+		break;
+	}
+	return align;
 }
 
 
@@ -214,7 +263,7 @@ void InsetBox::doDispatch(Cursor & cur, FuncRequest & cmd)
 			cur.undispatched();
 			return;
 		}
-		cur.recordUndoInset(ATOMIC_UNDO, this);
+		cur.recordUndoInset(this);
 		if (change_type) {
 			params_.type = cmd.getArg(1);
 			// set a makebox if there is no inner box but Frameless was exectued
@@ -266,38 +315,48 @@ bool InsetBox::getStatus(Cursor & cur, FuncRequest const & cmd,
 		flag.setEnabled(true);
 		return true;
 
-	case LFUN_PARAGRAPH_BREAK:
-		if ((params_.inner_box && !params_.use_makebox)
-		     || params_.type == "Shaded" || params_.type == "Framed")
-			return InsetCollapsable::getStatus(cur, cmd, flag);
-		flag.setEnabled(false);
-		return true;
-
 	default:
 		return InsetCollapsable::getStatus(cur, cmd, flag);
 	}
 }
 
 
+const string defaultThick = "0.4pt";
+const string defaultSep = "3pt";
+const string defaultShadow = "4pt";
+
 void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 {
 	BoxType btype = boxtranslator().find(params_.type);
 
 	string width_string = params_.width.asLatexString();
+	string thickness_string = params_.thickness.asLatexString();
+	string separation_string = params_.separation.asLatexString();
+	string shadowsize_string = params_.shadowsize.asLatexString();
 	bool stdwidth = false;
-	// FIXME: do not test explicitely values of width_string
-	if (params_.inner_box &&
-			(width_string.find("1.0\\columnwidth") != string::npos
-			|| width_string.find("1.0\\textwidth") != string::npos)) {
+	// in general the overall width of some decorated boxes is wider thean the inner box
+	// we could therefore calculate the real width for all sizes so that if the user wants
+	// e.g. 0.1\columnwidth or 2cm he gets exactly this size
+	// however this makes problems when importing TeX code
+	// therefore only recalculate for the most common case that the box should not protrude
+	// the page margins
+	if (params_.inner_box
+		&& ((width_string.find("1\\columnwidth") != string::npos
+			|| width_string.find("1\\textwidth") != string::npos)
+			|| width_string.find("1\\paperwidth") != string::npos
+			|| width_string.find("1\\linewidth") != string::npos)) {
 		stdwidth = true;
 		switch (btype) {
 		case Frameless:
+			break;
 		case Framed:
+			width_string += " - 2\\FrameSep - 2\\FrameRule";
 			break;
 		case Boxed:
-		case Shaded:
 			width_string += " - 2\\fboxsep - 2\\fboxrule";
 			break;
+		case Shaded:
+			break;
 		case ovalbox:
 			width_string += " - 2\\fboxsep - 0.8pt";
 			break;
@@ -305,8 +364,7 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 			width_string += " - 2\\fboxsep - 1.6pt";
 			break;
 		case Shadowbox:
-			// Shadow falls outside right margin... opinions?
-			width_string += " - 2\\fboxsep - 2\\fboxrule"/* "-\\shadowsize"*/;
+			width_string += " - 2\\fboxsep - 2\\fboxrule - \\shadowsize";
 			break;
 		case Doublebox:
 			width_string += " - 2\\fboxsep - 7.5\\fboxrule - 1pt";
@@ -318,44 +376,88 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 	if (runparams.lastid != -1)
 		os.texrow().start(runparams.lastid, runparams.lastpos);
 
-	// Adapt to column/text width correctly also if paragraphs indented:
-	if (stdwidth)
+	// adapt column/text width correctly also if paragraphs indented
+	if (stdwidth && !(buffer().params().paragraph_separation))
 		os << "\\noindent";
 
 	switch (btype) {
 	case Frameless:
 		break;
 	case Framed:
+		if (thickness_string != defaultThick) {
+			os << "{\\FrameRule " << from_ascii(thickness_string);
+			if (separation_string != defaultSep)
+				os << "\\FrameSep " << from_ascii(separation_string);
+		}
+		if (separation_string != defaultSep && thickness_string == defaultThick)
+			os << "{\\FrameSep " << from_ascii(separation_string);
+
 		os << "\\begin{framed}%\n";
 		break;
 	case Boxed:
-		if (!width_string.empty()) {
-			if (!params_.inner_box) {
-				os << "\\framebox";
-				// Special widths, see usrguide sec. 3.5
-				// FIXME UNICODE
-				if (params_.special != "none") {
-					os << "[" << params_.width.value()
-					   << '\\' << from_utf8(params_.special)
-					   << ']';
-				} else
-					os << '[' << from_ascii(width_string)
-					   << ']';
-				if (params_.hor_pos != 'c')
-					os << "[" << params_.hor_pos << "]";
+		if (thickness_string != defaultThick) {
+			os << "{\\fboxrule " << from_ascii(thickness_string);
+			if (separation_string != defaultSep)
+				os << "\\fboxsep " << from_ascii(separation_string);
+		}
+		if (separation_string != defaultSep && thickness_string == defaultThick)
+			os << "{\\fboxsep " << from_ascii(separation_string);
+		if (!params_.inner_box && !width_string.empty()) {
+			if (params_.framecolor != "black" || params_.backgroundcolor != "none") {
+				os << "\\fcolorbox{" << params_.framecolor << "}{" << params_.backgroundcolor << "}{";
+				os << "\\makebox";
 			} else
+				os << "\\framebox";
+			// Special widths, see usrguide sec. 3.5
+			// FIXME UNICODE
+			if (params_.special != "none") {
+				os << "[" << params_.width.value()
+				   << '\\' << from_utf8(params_.special)
+				   << ']';
+			} else
+				os << '[' << from_ascii(width_string)
+				   << ']';
+			// default horizontal alignment is 'c'
+			if (params_.hor_pos != 'c')
+				os << "[" << params_.hor_pos << "]";
+		} else {
+			if (params_.framecolor != "black" || params_.backgroundcolor != "none")
+				os << "\\fcolorbox{" << params_.framecolor << "}{" << params_.backgroundcolor << "}";
+			else
 				os << "\\fbox";
-		} else
-			os << "\\fbox";
+		}
 		os << "{";
 		break;
 	case ovalbox:
+		if (!separation_string.empty() && separation_string != defaultSep)
+			os << "{\\fboxsep " << from_ascii(separation_string);
 		os << "\\ovalbox{";
 		break;
 	case Ovalbox:
+		if (!separation_string.empty() && separation_string != defaultSep)
+			os << "{\\fboxsep " << from_ascii(separation_string);
 		os << "\\Ovalbox{";
 		break;
 	case Shadowbox:
+		if (thickness_string != defaultThick) {
+			os << "{\\fboxrule " << from_ascii(thickness_string);
+			if (separation_string != defaultSep) {
+				os << "\\fboxsep " << from_ascii(separation_string);
+				if (shadowsize_string != defaultShadow)
+					os << "\\shadowsize " << from_ascii(shadowsize_string);
+			}
+			if (shadowsize_string != defaultShadow	&& separation_string == defaultSep)
+				os << "\\shadowsize " << from_ascii(shadowsize_string);
+		}
+		if (separation_string != defaultSep && thickness_string == defaultThick) {
+				os << "{\\fboxsep " << from_ascii(separation_string);
+				if (shadowsize_string != defaultShadow)
+					os << "\\shadowsize " << from_ascii(shadowsize_string);
+		}
+		if (shadowsize_string != defaultShadow
+				&& separation_string == defaultSep
+				&& thickness_string == defaultThick)
+				os << "{\\shadowsize " << from_ascii(shadowsize_string);
 		os << "\\shadowbox{";
 		break;
 	case Shaded:
@@ -363,15 +465,26 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 		// it is inside a minipage or parbox
 		break;
 	case Doublebox:
+		if (thickness_string != defaultThick) {
+			os << "{\\fboxrule " << from_ascii(thickness_string);
+			if (separation_string != defaultSep)
+				os << "\\fboxsep " << from_ascii(separation_string);
+		}
+		if (separation_string != defaultSep && thickness_string == defaultThick)
+			os << "{\\fboxsep " << from_ascii(separation_string);
 		os << "\\doublebox{";
 		break;
 	}
 
 	if (params_.inner_box) {
-		if (params_.use_parbox)
+		if (params_.use_parbox) {
+			if (params_.backgroundcolor != "none" && btype == Frameless)
+				os << "\\colorbox{" << params_.backgroundcolor << "}{";
 			os << "\\parbox";
-		else if (params_.use_makebox) {
+		} else if (params_.use_makebox) {
 			if (!width_string.empty()) {
+				if (params_.backgroundcolor != "none")
+					os << "\\colorbox{" << params_.backgroundcolor << "}{";
 				os << "\\makebox";
 				// FIXME UNICODE
 				// output the width and horizontal position
@@ -384,12 +497,19 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 					   << ']';
 				if (params_.hor_pos != 'c')
 					os << "[" << params_.hor_pos << "]";
-			} else
-				os << "\\mbox";
+			} else {
+				if (params_.backgroundcolor != "none")
+					os << "\\colorbox{" << params_.backgroundcolor << "}";
+				else
+					os << "\\mbox";
+			}
 			os << "{";
 		}
-		else
+		else {
+			if (params_.backgroundcolor != "none" && btype == Frameless)
+				os << "\\colorbox{" << params_.backgroundcolor << "}{";
 			os << "\\begin{minipage}";
+		}
 
 		// output parameters for parbox and minipage
 		if (!params_.use_makebox) {
@@ -436,6 +556,9 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 			os << "%\n}";
 		else
 			os << "%\n\\end{minipage}";
+		if (params_.backgroundcolor != "none" && btype == Frameless
+			&& !(params_.use_makebox && width_string.empty()))
+			os << "}";
 	}
 
 	switch (btype) {
@@ -443,15 +566,38 @@ void InsetBox::latex(otexstream & os, OutputParams const & runparams) const
 		break;
 	case Framed:
 		os << "\\end{framed}";
+		if (separation_string != defaultSep	|| thickness_string != defaultThick)
+			os << "}";
 		break;
 	case Boxed:
 		os << "}";
+		if (!params_.inner_box && !width_string.empty()
+			&& (params_.framecolor != "black" || params_.backgroundcolor != "none"))
+			os << "}";
+		if (separation_string != defaultSep	|| thickness_string != defaultThick)
+			os << "}";
 		break;
 	case ovalbox:
+		os << "}";
+		if (separation_string != defaultSep)
+			os << "}";
+		break;
 	case Ovalbox:
+		os << "}";
+		if (separation_string != defaultSep)
+			os << "}";
+		break;
 	case Doublebox:
+		os << "}";
+		if (separation_string != defaultSep || thickness_string != defaultThick)
+			os << "}";
+		break;
 	case Shadowbox:
 		os << "}";
+		if (separation_string != defaultSep
+			|| thickness_string != defaultThick
+			|| shadowsize_string != defaultShadow)
+			os << "}";
 		break;
 	case Shaded:
 		// already done
@@ -558,12 +704,17 @@ void InsetBox::validate(LaTeXFeatures & features) const
 	BoxType btype = boxtranslator().find(params_.type);
 	switch (btype) {
 	case Frameless:
+		if (params_.backgroundcolor != "none")
+			features.require("xcolor");
 		break;
 	case Framed:
+		features.require("calc");
 		features.require("framed");
 		break;
 	case Boxed:
 		features.require("calc");
+		if (params_.framecolor != "black" || params_.backgroundcolor != "none")
+			features.require("xcolor");
 		break;
 	case ovalbox:
 	case Ovalbox:
@@ -644,7 +795,12 @@ InsetBoxParams::InsetBoxParams(string const & label)
 	  hor_pos('c'),
 	  inner_pos('t'),
 	  height(Length("1in")),
-	  height_special("totalheight") // default is 1\\totalheight
+	  height_special("totalheight"), // default is 1\\totalheight
+	  thickness(Length(defaultThick)),
+	  separation(Length(defaultSep)),
+	  shadowsize(Length(defaultShadow)),
+	  framecolor("black"),
+	  backgroundcolor("none")
 {}
 
 
@@ -661,6 +817,11 @@ void InsetBoxParams::write(ostream & os) const
 	os << "special \"" << special << "\"\n";
 	os << "height \"" << height.asString() << "\"\n";
 	os << "height_special \"" << height_special << "\"\n";
+	os << "thickness \"" << thickness.asString() << "\"\n";
+	os << "separation \"" << separation.asString() << "\"\n";
+	os << "shadowsize \"" << shadowsize.asString() << "\"\n";
+	os << "framecolor \"" << framecolor << "\"\n";
+	os << "backgroundcolor \"" << backgroundcolor << "\"\n";
 }
 
 
@@ -680,6 +841,11 @@ void InsetBoxParams::read(Lexer & lex)
 	lex >> "special" >> special;
 	lex >> "height" >> height;
 	lex >> "height_special" >> height_special;
+	lex >> "thickness" >> thickness;
+	lex >> "separation" >> separation;
+	lex >> "shadowsize" >> shadowsize;
+	lex >> "framecolor" >> framecolor;
+	lex >> "backgroundcolor" >> backgroundcolor;
 }
 
 
