@@ -76,13 +76,14 @@ private:
 	///
 	void toString(odocstream &) const;
 	///
-	void forToc(docstring &, size_t) const;
+	void forOutliner(docstring &, size_t const, bool const) const;
 	///
 	void validate(LaTeXFeatures &) const;
 	///
 	std::string contextMenuName() const;
 	///
-	void addToToc(DocIterator const & di, bool output_active) const;
+	void addToToc(DocIterator const & di, bool output_active,
+				  UpdateType utype) const;
 	///
 	InsetBranchParams const & params() const { return params_; }
 	///
