@@ -39,6 +39,8 @@ public:
 	///
 	InsetCommand(InsetCommand const & rhs);
 	///
+	InsetCommand & operator=(InsetCommand const & rhs);
+	///
 	virtual ~InsetCommand();
 	///
 	InsetCommand * asInsetCommand() { return this; }
@@ -66,7 +68,7 @@ public:
 	///
 	void write(std::ostream & os) const { p_.write(os); }
 	///
-	void read(Lexer & lex) { p_.read(lex); }
+	void read(Lexer & lex) { p_.Read(lex, &buffer()); }
 	///
 	void doDispatch(Cursor & cur, FuncRequest & cmd);
 	///
