@@ -22,6 +22,10 @@
 #include <boost/signals/trackable.hpp>
 
 
+namespace lyx {
+
+namespace external {
+
 /** No two InsetExternalParams variables can have the same temporary file.
  *  This class has copy-semantics but the copy constructor
  *  and assignment operator simply call the default constructor.
@@ -29,17 +33,13 @@
  *  copy constructor and assignment operator for the
  *  InsetExternalParams class.
  */
-namespace lyx {
-
-namespace external {
-
 class TempName {
 public:
 	TempName();
 	TempName(TempName const &);
 	~TempName();
 	TempName & operator=(TempName const &);
-	support::FileName const & operator()() const { return tempname_; }
+	support::FileName operator()() const;
 private:
 	support::FileName tempname_;
 };
@@ -56,7 +56,7 @@ public:
 	bool read(Buffer const &, Lexer &);
 
 	/// The name of the tempfile used for manipulations.
-	support::FileName const & tempname() const { return tempname_(); }
+	support::FileName tempname() const { return tempname_(); }
 
 	/// The template currently in use.
 	void settemplate(std::string const &);
@@ -92,6 +92,9 @@ class RenderBase;
 ///
 class InsetExternal : public Inset, public boost::signals::trackable
 {
+	// Disable assignment operator, since it is not used, and it is too
+	// complicated to implement it consistently with the copy constructor
+	InsetExternal & operator=(InsetExternal const &);
 public:
 	InsetExternal(Buffer *);
 	///
@@ -111,11 +114,14 @@ public:
 	/// \returns the number of rows (\n's) of generated code.
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	docstring contextMenuName() const;
+	std::string contextMenuName() const;
 	///
-	bool setMouseHover(BufferView const * bv, bool mouse_hover);
+	bool setMouseHover(BufferView const * bv, bool mouse_hover) const;
 	///
 	bool clickable(int, int) const { return true; }
+	///
+	void addToToc(DocIterator const & di, bool output_active,
+				  UpdateType utype) const;
 private:
 	///
 	InsetExternal(InsetExternal const &);
@@ -132,7 +138,8 @@ private:
 	///
 	void read(Lexer & lex);
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	/// For now, this does nothing. Someone who knows about this
