@@ -36,6 +36,8 @@ public:
 	~InsetIPA();
 	///
 	InsetIPA(InsetIPA const & other);
+	///
+	InsetIPA & operator=(InsetIPA const & other);
 
 	/// \name Methods inherited from Inset class
 	//@{
@@ -72,6 +74,8 @@ public:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
+	docstring xhtml(XHTMLStream & xs, OutputParams const &) const;
+	///
 	void validate(LaTeXFeatures & features) const;
 	///
 	bool allowSpellCheck() const { return false; }
