@@ -29,31 +29,34 @@ using std::string;
 using std::set;
 using std::vector;
 
+using namespace lyx::support;
+
 namespace lyx {
 
 InsetLayout::InsetLayout() :
 	name_(from_ascii("undefined")), lyxtype_(STANDARD),
 	labelstring_(from_ascii("UNDEFINED")), contentaslabel_(false),
-	decoration_(DEFAULT), latextype_(NOLATEXTYPE), font_(inherit_font), 
-	labelfont_(sane_font), bgcolor_(Color_error), 
-	htmlforcecss_ (false), htmlisblock_(true),
-	multipar_(true), custompars_(true), forceplain_(false), 
-	passthru_(false), parbreakisnewline_(false), freespacing_(false), 
-	keepempty_(false), forceltr_(false), 
-	needprotect_(false), intoc_(false), spellcheck_(true), 
-	resetsfont_(false), display_(true), forcelocalfontswitch_(false)
-{ 
+	decoration_(DEFAULT), latextype_(NOLATEXTYPE), font_(inherit_font),
+	labelfont_(sane_font), bgcolor_(Color_error),
+	fixedwidthpreambleencoding_(false), htmlforcecss_ (false),
+	htmlisblock_(true), multipar_(true), custompars_(true),
+	forceplain_(false), passthru_(false), parbreakisnewline_(false),
+	freespacing_(false), keepempty_(false), forceltr_(false),
+	forceownlines_(false), needprotect_(false), intoc_(false),
+	spellcheck_(true), resetsfont_(false), display_(true),
+	forcelocalfontswitch_(false), add_to_toc_(false), is_toc_caption_(false)
+{
 	labelfont_.setColor(Color_error);
 }
 
 
-InsetLayout::InsetDecoration translateDecoration(std::string const & str) 
+InsetLayout::InsetDecoration translateDecoration(std::string const & str)
 {
-	if (support::compare_ascii_no_case(str, "classic") == 0)
+	if (compare_ascii_no_case(str, "classic") == 0)
 		return InsetLayout::CLASSIC;
-	if (support::compare_ascii_no_case(str, "minimalistic") == 0)
+	if (compare_ascii_no_case(str, "minimalistic") == 0)
 		return InsetLayout::MINIMALISTIC;
-	if (support::compare_ascii_no_case(str, "conglomerate") == 0)
+	if (compare_ascii_no_case(str, "conglomerate") == 0)
 		return InsetLayout::CONGLOMERATE;
 	return InsetLayout::DEFAULT;
 }
@@ -62,11 +65,11 @@ namespace {
 
 InsetLayout::InsetLaTeXType translateLaTeXType(std::string const & str)
 {
-	if (support::compare_ascii_no_case(str, "command") == 0)
+	if (compare_ascii_no_case(str, "command") == 0)
 		return InsetLayout::COMMAND;
-	if (support::compare_ascii_no_case(str, "environment") == 0)
+	if (compare_ascii_no_case(str, "environment") == 0)
 		return InsetLayout::ENVIRONMENT;
-	if (support::compare_ascii_no_case(str, "none") == 0)
+	if (compare_ascii_no_case(str, "none") == 0)
 		return InsetLayout::NOLATEXTYPE;
 	return InsetLayout::ILT_ERROR;
 }
@@ -77,6 +80,7 @@ InsetLayout::InsetLaTeXType translateLaTeXType(std::string const & str)
 bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 {
 	enum {
+		IL_ADDTOTOC,
 		IL_ARGUMENT,
 		IL_BABELPREAMBLE,
 		IL_BGCOLOR,
@@ -86,9 +90,11 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		IL_CUSTOMPARS,
 		IL_DECORATION,
 		IL_DISPLAY,
+		IL_FIXEDWIDTH_PREAMBLE_ENCODING,
 		IL_FONT,
 		IL_FORCE_LOCAL_FONT_SWITCH,
 		IL_FORCELTR,
+		IL_FORCEOWNLINES,
 		IL_FORCEPLAIN,
 		IL_FREESPACING,
 		IL_HTMLTAG,
@@ -101,6 +107,7 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		IL_HTMLSTYLE,
 		IL_HTMLPREAMBLE,
 		IL_INTOC,
+		IL_ISTOCCAPTION,
 		IL_LABELFONT,
 		IL_LABELSTRING,
 		IL_LANGPREAMBLE,
@@ -109,10 +116,12 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		IL_LATEXTYPE,
 		IL_LEFTDELIM,
 		IL_LYXTYPE,
+		IL_OBSOLETEDBY,
 		IL_KEEPEMPTY,
 		IL_MULTIPAR,
 		IL_NEEDPROTECT,
 		IL_PASSTHRU,
+		IL_PASSTHRU_CHARS,
 		IL_PARBREAKISNEWLINE,
 		IL_PREAMBLE,
 		IL_REQUIRES,
@@ -126,19 +135,22 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 
 
 	LexerKeyword elementTags[] = {
+		{ "addtotoc", IL_ADDTOTOC },
 		{ "argument", IL_ARGUMENT },
 		{ "babelpreamble", IL_BABELPREAMBLE },
 		{ "bgcolor", IL_BGCOLOR },
 		{ "contentaslabel", IL_CONTENTASLABEL },
-		{ "copystyle", IL_COPYSTYLE }, 
+		{ "copystyle", IL_COPYSTYLE },
 		{ "counter", IL_COUNTER},
 		{ "custompars", IL_CUSTOMPARS },
 		{ "decoration", IL_DECORATION },
 		{ "display", IL_DISPLAY },
 		{ "end", IL_END },
+		{ "fixedwidthpreambleencoding", IL_FIXEDWIDTH_PREAMBLE_ENCODING },
 		{ "font", IL_FONT },
 		{ "forcelocalfontswitch", IL_FORCE_LOCAL_FONT_SWITCH },
 		{ "forceltr", IL_FORCELTR },
+		{ "forceownlines", IL_FORCEOWNLINES },
 		{ "forceplain", IL_FORCEPLAIN },
 		{ "freespacing", IL_FREESPACING },
 		{ "htmlattr", IL_HTMLATTR },
@@ -151,6 +163,7 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		{ "htmlstyle", IL_HTMLSTYLE },
 		{ "htmltag", IL_HTMLTAG },
 		{ "intoc", IL_INTOC },
+		{ "istoccaption", IL_ISTOCCAPTION },
 		{ "keepempty", IL_KEEPEMPTY },
 		{ "labelfont", IL_LABELFONT },
 		{ "labelstring", IL_LABELSTRING },
@@ -162,8 +175,10 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		{ "lyxtype", IL_LYXTYPE },
 		{ "multipar", IL_MULTIPAR },
 		{ "needprotect", IL_NEEDPROTECT },
+		{ "obsoletedby", IL_OBSOLETEDBY },
 		{ "parbreakisnewline", IL_PARBREAKISNEWLINE },
 		{ "passthru", IL_PASSTHRU },
+		{ "passthruchars", IL_PASSTHRU_CHARS },
 		{ "preamble", IL_PREAMBLE },
 		{ "refprefix", IL_REFPREFIX },
 		{ "requires", IL_REQUIRES },
@@ -182,7 +197,7 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 	// for issuing a warning in case MultiPars comes later
 	bool readCustomOrPlain = false;
 
-	string tmp;	
+	string tmp;
 	while (!getout && lex.isOK()) {
 		int le = lex.lex();
 		switch (le) {
@@ -235,19 +250,22 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 			break;
 		case IL_LATEXPARAM:
 			lex >> tmp;
-			latexparam_ = support::subst(tmp, "&quot;", "\"");
+			latexparam_ = subst(tmp, "&quot;", "\"");
 			break;
 		case IL_LEFTDELIM:
 			lex >> leftdelim_;
-			leftdelim_ = support::subst(leftdelim_, from_ascii("<br/>"),
+			leftdelim_ = subst(leftdelim_, from_ascii("<br/>"),
 						    from_ascii("\n"));
 			break;
+		case IL_FIXEDWIDTH_PREAMBLE_ENCODING:
+			lex >> fixedwidthpreambleencoding_;
+			break;
 		case IL_FORCE_LOCAL_FONT_SWITCH:
 			lex >> forcelocalfontswitch_;
 			break;
 		case IL_RIGHTDELIM:
 			lex >> rightdelim_;
-			rightdelim_ = support::subst(rightdelim_, from_ascii("<br/>"),
+			rightdelim_ = subst(rightdelim_, from_ascii("<br/>"),
 						     from_ascii("\n"));
 			break;
 		case IL_LABELFONT:
@@ -256,6 +274,9 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		case IL_FORCELTR:
 			lex >> forceltr_;
 			break;
+		case IL_FORCEOWNLINES:
+			lex >> forceownlines_;
+			break;
 		case IL_INTOC:
 			lex >> intoc_;
 			break;
@@ -283,6 +304,9 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		case IL_PASSTHRU:
 			lex >> passthru_;
 			break;
+		case IL_PASSTHRU_CHARS:
+			lex >> passthru_chars_;
+			break;
 		case IL_PARBREAKISNEWLINE:
 			lex >> parbreakisnewline_;
 			break;
@@ -302,11 +326,11 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 			// initialize with a known style
 			docstring style;
 			lex >> style;
-			style = support::subst(style, '_', ' ');
+			style = subst(style, '_', ' ');
 
 			// We don't want to apply the algorithm in DocumentClass::insetLayout()
 			// here. So we do it the long way.
-			TextClass::InsetLayouts::const_iterator it = 
+			TextClass::InsetLayouts::const_iterator it =
 					tclass.insetLayouts().find(style);
 			if (it != tclass.insetLayouts().end()) {
 				docstring const tmpname = name_;
@@ -314,11 +338,42 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 				name_ = tmpname;
 			} else {
 				LYXERR0("Cannot copy unknown InsetLayout `"
+					<< style << "' to InsetLayout `"
+					<< name() << "'\n"
+					<< "All InsetLayouts so far:");
+				TextClass::InsetLayouts::const_iterator lit =
+						tclass.insetLayouts().begin();
+				TextClass::InsetLayouts::const_iterator len =
+						tclass.insetLayouts().end();
+				for (; lit != len; ++lit)
+					lyxerr << lit->second.name() << "\n";
+			}
+			break;
+		}
+		case IL_OBSOLETEDBY: {
+			docstring style;
+			lex >> style;
+			style = subst(style, '_', ' ');
+
+			// We don't want to apply the algorithm in DocumentClass::insetLayout()
+			// here. So we do it the long way.
+			TextClass::InsetLayouts::const_iterator it =
+					tclass.insetLayouts().find(style);
+			if (it != tclass.insetLayouts().end()) {
+				docstring const tmpname = name_;
+				this->operator=(it->second);
+				name_ = tmpname;
+				if (obsoleted_by().empty())
+					obsoleted_by_ = style;
+			} else {
+				LYXERR0("Cannot replace InsetLayout `"
+					<< name()
+					<< "' with unknown InsetLayout `"
 					<< style << "'\n"
 					<< "All InsetLayouts so far:");
-				TextClass::InsetLayouts::const_iterator lit = 
+				TextClass::InsetLayouts::const_iterator lit =
 						tclass.insetLayouts().begin();
-				TextClass::InsetLayouts::const_iterator len = 
+				TextClass::InsetLayouts::const_iterator len =
 						tclass.insetLayouts().end();
 				for (; lit != len; ++lit)
 					lyxerr << lit->second.name() << "\n";
@@ -390,7 +445,7 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		case IL_REQUIRES: {
 			lex.eatLine();
 			vector<string> const req
-				= support::getVectorFromString(lex.getString(true));
+				= getVectorFromString(lex.getString(true));
 			requires_.insert(req.begin(), req.end());
 			break;
 		}
@@ -403,6 +458,13 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 		case IL_DISPLAY:
 			lex >> display_;
 			break;
+		case IL_ADDTOTOC:
+			lex >> toc_type_;
+			add_to_toc_ = !toc_type_.empty();
+			break;
+		case IL_ISTOCCAPTION:
+			lex >> is_toc_caption_;
+			break;
 		case IL_END:
 			getout = true;
 			break;
@@ -412,7 +474,7 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 	// Here add element to list if getout == true
 	if (!getout)
 		return false;
-	
+
 	// The label font is generally used as-is without
 	// any realization against a given context.
 	labelfont_.realize(sane_font);
@@ -422,18 +484,17 @@ bool InsetLayout::read(Lexer & lex, TextClass const & tclass)
 }
 
 
-InsetLayout::InsetLyXType translateLyXType(std::string const & str) 
+InsetLayout::InsetLyXType translateLyXType(std::string const & str)
 {
-	
-	if (support::compare_ascii_no_case(str, "charstyle") == 0)
+	if (compare_ascii_no_case(str, "charstyle") == 0)
 		return InsetLayout::CHARSTYLE;
-	if (support::compare_ascii_no_case(str, "custom") == 0)
+	if (compare_ascii_no_case(str, "custom") == 0)
 		return InsetLayout::CUSTOM;
-	if (support::compare_ascii_no_case(str, "element") == 0)
+	if (compare_ascii_no_case(str, "element") == 0)
 		return InsetLayout::ELEMENT;
-	if (support::compare_ascii_no_case(str, "end") == 0)
+	if (compare_ascii_no_case(str, "end") == 0)
 		return InsetLayout::END;
-	if (support::compare_ascii_no_case(str, "standard") == 0)
+	if (compare_ascii_no_case(str, "standard") == 0)
 		return InsetLayout::STANDARD;
 	return InsetLayout::NOLYXTYPE;
 }
@@ -443,7 +504,7 @@ string const & InsetLayout::htmltag() const
 {
 	if (htmltag_.empty())
 		htmltag_ = multipar_ ? "div" : "span";
-	return htmltag_; 
+	return htmltag_;
 }
 
 
@@ -451,7 +512,7 @@ string const & InsetLayout::htmlattr() const
 {
 	if (htmlattr_.empty())
 		htmlattr_ = "class=\"" + defaultCSSClass() + "\"";
-	return htmlattr_; 
+	return htmlattr_;
 }
 
 
@@ -459,12 +520,12 @@ string const & InsetLayout::htmlinnerattr() const
 {
 	if (htmlinnerattr_.empty())
 		htmlinnerattr_ = "class=\"" + defaultCSSClass() + "_inner\"";
-	return htmlinnerattr_; 
+	return htmlinnerattr_;
 }
 
 
 string InsetLayout::defaultCSSClass() const
-{ 
+{
 	if (!defaultcssclass_.empty())
 		return defaultcssclass_;
 	string d;
@@ -477,7 +538,7 @@ string InsetLayout::defaultCSSClass() const
 		else if (isLower(*it))
 			d += *it;
 		else
-			d += support::lowercase(*it);
+			d += lowercase(*it);
 	}
 	// are there other characters we need to remove?
 	defaultcssclass_ = d;
@@ -487,18 +548,18 @@ string InsetLayout::defaultCSSClass() const
 
 void InsetLayout::makeDefaultCSS() const
 {
-	if (!htmldefaultstyle_.empty()) 
+	if (!htmldefaultstyle_.empty())
 		return;
 	docstring const mainfontCSS = font_.asCSS();
 	if (!mainfontCSS.empty())
-		htmldefaultstyle_ = 
+		htmldefaultstyle_ =
 				from_ascii(htmltag() + "." + defaultCSSClass() + " {\n") +
 				mainfontCSS + from_ascii("\n}\n");
 }
 
 
-docstring InsetLayout::htmlstyle() const 
-{ 
+docstring InsetLayout::htmlstyle() const
+{
 	if (!htmlstyle_.empty() && !htmlforcecss_)
 		return htmlstyle_;
 	if (htmldefaultstyle_.empty())
@@ -514,16 +575,18 @@ void InsetLayout::readArgument(Lexer & lex)
 	Layout::latexarg arg;
 	arg.mandatory = false;
 	arg.autoinsert = false;
+	arg.insertcotext = false;
 	bool error = false;
 	bool finished = false;
 	arg.font = inherit_font;
 	arg.labelfont = inherit_font;
+	arg.is_toc_caption = false;
 	string nr;
 	lex >> nr;
-	bool const postcmd = support::prefixIs(nr, "post:");
+	bool const postcmd = prefixIs(nr, "post:");
 	while (!finished && lex.isOK() && !error) {
 		lex.next();
-		string const tok = support::ascii_lowercase(lex.getString());
+		string const tok = ascii_lowercase(lex.getString());
 
 		if (tok.empty()) {
 			continue;
@@ -541,15 +604,21 @@ void InsetLayout::readArgument(Lexer & lex)
 		} else if (tok == "autoinsert") {
 			lex.next();
 			arg.autoinsert = lex.getBool();
+		} else if (tok == "insertcotext") {
+			lex.next();
+			arg.insertcotext = lex.getBool();
+		} else if (tok == "insertcotext") {
+			lex.next();
+			arg.insertcotext = lex.getBool();
 		} else if (tok == "leftdelim") {
 			lex.next();
 			arg.ldelim = lex.getDocString();
-			arg.ldelim = support::subst(arg.ldelim,
+			arg.ldelim = subst(arg.ldelim,
 						    from_ascii("<br/>"), from_ascii("\n"));
 		} else if (tok == "rightdelim") {
 			lex.next();
 			arg.rdelim = lex.getDocString();
-			arg.rdelim = support::subst(arg.rdelim,
+			arg.rdelim = subst(arg.rdelim,
 						    from_ascii("<br/>"), from_ascii("\n"));
 		} else if (tok == "defaultarg") {
 			lex.next();
@@ -570,6 +639,12 @@ void InsetLayout::readArgument(Lexer & lex)
 			arg.font = lyxRead(lex, arg.font);
 		} else if (tok == "labelfont") {
 			arg.labelfont = lyxRead(lex, arg.labelfont);
+		} else if (tok == "passthruchars") {
+			lex.next();
+			arg.pass_thru_chars = lex.getDocString();
+		} else if (tok == "istoccaption") {
+			lex.next();
+			arg.is_toc_caption = lex.getBool();
 		} else {
 			lex.printError("Unknown tag");
 			error = true;
