@@ -41,11 +41,19 @@ public:
 		/// breakable slash
 		SLASH,
 		/// protected dash
-		NOBREAKDASH
+		NOBREAKDASH,
+		/// LyX logo
+		PHRASE_LYX,
+		/// TeX logo
+		PHRASE_TEX,
+		/// LaTeX2e logo
+		PHRASE_LATEX2E,
+		/// LaTeX logo
+		PHRASE_LATEX
 	};
 
 	///
-	InsetSpecialChar() : Inset(0) {}
+	InsetSpecialChar() : Inset(0), kind_(HYPHENATION) {}
 	///
 	explicit InsetSpecialChar(Kind k);
 	///
@@ -70,7 +78,7 @@ public:
 	///
 	void toString(odocstream &) const;
 	///
-	void forToc(docstring &, size_t) const;
+	void forOutliner(docstring &, size_t const, bool const) const;
 	///
 	InsetCode lyxCode() const { return SPECIALCHAR_CODE; }
 	/// We don't need \begin_inset and \end_inset
