@@ -433,6 +433,8 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 	// environment. Standard collapsable insets should not
 	// redefine this, non-standard ones may call this.
 	InsetLayout const & il = getLayout();
+	if (il.forceOwnlines())
+		os << breakln;
 	if (!il.latexname().empty()) {
 		if (il.latextype() == InsetLayout::COMMAND) {
 			// FIXME UNICODE
@@ -476,6 +478,8 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 		rp.pass_thru = true;
 	if (il.isNeedProtect())
 		rp.moving_arg = true;
+	if (!il.passThruChars().empty())
+		rp.pass_thru_chars += il.passThruChars();
 	rp.par_begin = 0;
 	rp.par_end = paragraphs().size();
 
@@ -497,11 +501,13 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 				os << breakln;
 			else
 				os << safebreakln;
-			os << "\\end{" << from_utf8(il.latexname()) << "}\n";
+			os << "\\end{" << from_utf8(il.latexname()) << "}" << breakln;
 			if (!il.isDisplay())
 				os.protectSpace(true);
 		}
 	}
+	if (il.forceOwnlines())
+		os << breakln;
 }
 
 
@@ -609,7 +615,7 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 		xs << html::StartTag(il.htmlinnertag(), il.htmlinnerattr());
 
 	// we will eventually lose information about the containing inset
-	if (!il.isMultiPar() || opts == JustText)
+	if (!allowMultiPar() || opts == JustText)
 		runparams.html_make_pars = false;
 	if (il.isPassThru())
 		runparams.pass_thru = true;
@@ -657,27 +663,6 @@ void InsetText::setText(docstring const & data, Font const & font, bool trackCha
 }
 
 
-void InsetText::setAutoBreakRows(bool flag)
-{
-	if (flag == text_.autoBreakRows_)
-		return;
-
-	text_.autoBreakRows_ = flag;
-	if (flag)
-		return;
-
-	// remove previously existing newlines
-	ParagraphList::iterator it = paragraphs().begin();
-	ParagraphList::iterator end = paragraphs().end();
-	for (; it != end; ++it)
-		for (int i = 0; i < it->size(); ++i)
-			if (it->isNewline(i))
-				// do not track the change, because the user
-				// is not allowed to revert/reject it
-				it->eraseChar(i, false);
-}
-
-
 void InsetText::setDrawFrame(bool flag)
 {
 	drawFrame_ = flag;
@@ -789,6 +774,9 @@ void InsetText::updateBuffer(ParIterator const & it, UpdateType utype)
 		// since we are saving and restoring the existing counters, etc.
 		Counters const savecnt = tclass.counters();
 		tclass.counters().reset();
+		// we need float information even in note insets (#9760)
+		tclass.counters().current_float(savecnt.current_float());
+		tclass.counters().isSubfloat(savecnt.isSubfloat());
 		buffer().updateBuffer(it2, utype);
 		tclass.counters() = savecnt;
 	}
@@ -801,26 +789,30 @@ void InsetText::toString(odocstream & os) const
 }
 
 
-void InsetText::forToc(docstring & os, size_t maxlen) const
+void InsetText::forOutliner(docstring & os, size_t const maxlen,
+							bool const shorten) const
 {
 	if (!getLayout().isInToc())
 		return;
-	text().forToc(os, maxlen, false);
+	text().forOutliner(os, maxlen, shorten);
 }
 
 
-void InsetText::addToToc(DocIterator const & cdit, bool output_active) const
+void InsetText::addToToc(DocIterator const & cdit, bool output_active,
+						 UpdateType utype) const
 {
 	DocIterator dit = cdit;
 	dit.push_back(CursorSlice(const_cast<InsetText &>(*this)));
-	iterateForToc(dit, output_active);
+	iterateForToc(dit, output_active, utype);
 }
 
 
-void InsetText::iterateForToc(DocIterator const & cdit, bool output_active) const
+void InsetText::iterateForToc(DocIterator const & cdit, bool output_active,
+							  UpdateType utype) const
 {
 	DocIterator dit = cdit;
-	Toc & toc = buffer().tocBackend().toc("tableofcontents");
+	// This also ensures that any document has a table of contents
+	shared_ptr<Toc> toc = buffer().tocBackend().toc("tableofcontents");
 
 	BufferParams const & bufparams = buffer_->params();
 	int const min_toclevel = bufparams.documentClass().min_toclevel();
@@ -843,7 +835,7 @@ void InsetText::iterateForToc(DocIterator const & cdit, bool output_active) cons
 			Inset & inset = *it->inset;
 			dit.pos() = it->pos;
 			//lyxerr << (void*)&inset << " code: " << inset.lyxCode() << std::endl;
-			inset.addToToc(dit, doing_output);
+			inset.addToToc(dit, doing_output, utype);
 			if (inset.lyxCode() == ARG_CODE)
 				arginset = inset.asInsetText();
 		}
@@ -852,17 +844,18 @@ void InsetText::iterateForToc(DocIterator const & cdit, bool output_active) cons
 		if (toclevel != Layout::NOT_IN_TOC && toclevel >= min_toclevel) {
 			// insert this into the table of contents
 			docstring tocstring;
-			int const length = doing_output ? INT_MAX : TOC_ENTRY_LENGTH;
+			int const length = (doing_output && utype == OutputUpdate) ?
+				INT_MAX : TOC_ENTRY_LENGTH;
 			if (arginset) {
 				tocstring = par.labelString();
 				if (!tocstring.empty())
 					tocstring += ' ';
-				arginset->text().forToc(tocstring, length);
+				arginset->text().forOutliner(tocstring, length);
 			} else
-				par.forToc(tocstring, length);
+				par.forOutliner(tocstring, length);
 			dit.pos() = 0;
-			toc.push_back(TocItem(dit, toclevel - min_toclevel,
-				tocstring, doing_output, tocstring));
+			toc->push_back(TocItem(dit, toclevel - min_toclevel,
+								  tocstring, doing_output, tocstring));
 		}
 		
 		// And now the list of changes.
