@@ -12,6 +12,7 @@
 #include "insets/InsetBox.cpp" 
 #include "insets/InsetBranch.cpp" 
 #include "insets/InsetCaption.cpp" 
+#include "insets/InsetCaptionable.cpp" 
 #include "insets/InsetCitation.cpp" 
 #include "insets/InsetCollapsable.cpp" 
 #include "insets/InsetCommand.cpp" 
@@ -46,6 +47,7 @@
 #include "insets/InsetQuotes.cpp" 
 #include "insets/InsetRef.cpp" 
 #include "insets/InsetScript.cpp" 
+#include "insets/InsetSeparator.cpp" 
 #include "insets/InsetSpace.cpp" 
 #include "insets/InsetSpecialChar.cpp" 
 #include "insets/InsetTabular.cpp" 
