@@ -127,7 +127,7 @@ HullType hullType(docstring const & s)
 	if (s == "gather")    return hullGather;
 	if (s == "flalign")   return hullFlAlign;
 	if (s == "regexp")    return hullRegexp;
-  	lyxerr << "unknown hull type '" << to_utf8(s) << "'" << endl;
+	lyxerr << "unknown hull type '" << to_utf8(s) << "'" << endl;
 	return HullType(-1);
 }
 
@@ -147,7 +147,7 @@ docstring hullName(HullType type)
 		case hullGather:     return from_ascii("gather");
 		case hullFlAlign:    return from_ascii("flalign");
 		case hullRegexp:     return from_ascii("regexp");
- 		default:
+		default:
 			lyxerr << "unknown hull type '" << type << "'" << endl;
 			return from_ascii("none");
 	}
@@ -156,9 +156,9 @@ docstring hullName(HullType type)
 static InsetLabel * dummy_pointer = 0;
 
 InsetMathHull::InsetMathHull(Buffer * buf)
-	: InsetMathGrid(buf, 1, 1), type_(hullNone), numbered_(1, true),
-    numbers_(1, empty_docstring()), label_(1, dummy_pointer),
-    preview_(new RenderPreview(this))
+	: InsetMathGrid(buf, 1, 1), type_(hullNone), numbered_(1, NUMBER),
+	  numbers_(1, empty_docstring()), label_(1, dummy_pointer),
+	  preview_(new RenderPreview(this))
 {
 	//lyxerr << "sizeof InsetMath: " << sizeof(InsetMath) << endl;
 	//lyxerr << "sizeof MetricsInfo: " << sizeof(MetricsInfo) << endl;
@@ -171,9 +171,9 @@ InsetMathHull::InsetMathHull(Buffer * buf)
 
 
 InsetMathHull::InsetMathHull(Buffer * buf, HullType type)
-	: InsetMathGrid(buf, getCols(type), 1), type_(type), numbered_(1, true),
-    numbers_(1, empty_docstring()), label_(1, dummy_pointer),
-    preview_(new RenderPreview(this))
+	: InsetMathGrid(buf, getCols(type), 1), type_(type), numbered_(1, NUMBER),
+	  numbers_(1, empty_docstring()), label_(1, dummy_pointer),
+	  preview_(new RenderPreview(this))
 {
 	buffer_ = buf;
 	initMath();
@@ -292,7 +292,8 @@ void InsetMathHull::updateBuffer(ParIterator const & it, UpdateType utype)
 }
 
 
-void InsetMathHull::addToToc(DocIterator const & pit, bool output_active) const
+void InsetMathHull::addToToc(DocIterator const & pit, bool output_active,
+							 UpdateType utype) const
 {
 	if (!buffer_) {
 		//FIXME: buffer_ should be set at creation for this inset! Problem is
@@ -301,14 +302,14 @@ void InsetMathHull::addToToc(DocIterator const & pit, bool output_active) const
 		return;
 	}
 
-	Toc & toc = buffer().tocBackend().toc("equation");
+	shared_ptr<Toc> toc = buffer().tocBackend().toc("equation");
 
 	for (row_type row = 0; row != nrows(); ++row) {
-		if (!numbered_[row])
+		if (!numbered(row))
 			continue;
 		if (label_[row])
-			label_[row]->addToToc(pit, output_active);
-		toc.push_back(TocItem(pit, 0, nicelabel(row), output_active));
+			label_[row]->addToToc(pit, output_active, utype);
+		toc->push_back(TocItem(pit, 0, nicelabel(row), output_active));
 	}
 }
 
@@ -362,15 +363,16 @@ char InsetMathHull::defaultColAlign(col_type col)
 }
 
 
-char InsetMathHull::displayColAlign(col_type col, row_type row) const
+char InsetMathHull::displayColAlign(idx_type idx) const
 {
 	if (type_ == hullMultline) {
-		if (row == 0)
+		row_type const r = row(idx);
+		if (r == 0)
 			return 'l';
-		if (row == nrows() - 1)
+		if (r == nrows() - 1)
 			return 'r';
 	}
-	return InsetMathGrid::displayColAlign(col, row);
+	return InsetMathGrid::displayColAlign(idx);
 }
 
 
@@ -420,8 +422,8 @@ ColorCode InsetMathHull::standardColor() const
 
 bool InsetMathHull::previewState(const BufferView *const bv) const
 {
-	if (!editing(bv) && RenderPreview::status() == LyXRC::PREVIEW_ON
-		&& type_ != hullRegexp)
+	if (!editing(bv) && RenderPreview::previewMath()
+	    && type_ != hullRegexp)
 	{
 		graphics::PreviewImage const * pimage =
 			preview_->getPreviewImage(bv->buffer());
@@ -512,10 +514,10 @@ void InsetMathHull::drawBackground(PainterInfo & pi, int x, int y) const
 {
 	Dimension const dim = dimension(*pi.base.bv);
 	if (previewTooSmall(dim)) {
-		pi.pain.fillRectangle(x, y - 2 * ERROR_FRAME_WIDTH, 
+		pi.pain.fillRectangle(x, y - 2 * ERROR_FRAME_WIDTH,
 		    dim.wid, dim.asc + dim.des, backgroundColor(pi));
 		return;
-	} 
+	}
 	pi.pain.fillRectangle(x + 1, y - dim.asc + 1, dim.wid - 2,
 			dim.asc + dim.des - 1, pi.backgroundColor(this));
 }
@@ -572,7 +574,9 @@ void InsetMathHull::metricsT(TextMetricsInfo const & mi, Dimension & dim) const
 		InsetMathGrid::metricsT(mi, dim);
 	} else {
 		odocstringstream os;
-		WriteStream wi(os, false, true, WriteStream::wsDefault);
+		TexRow texrow(false);
+		otexrowstream ots(os,texrow);
+		WriteStream wi(ots, false, true, WriteStream::wsDefault);
 		write(wi);
 		dim.wid = os.str().size();
 		dim.asc = 1;
@@ -587,7 +591,9 @@ void InsetMathHull::drawT(TextPainter & pain, int x, int y) const
 		InsetMathGrid::drawT(pain, x, y);
 	} else {
 		odocstringstream os;
-		WriteStream wi(os, false, true, WriteStream::wsDefault);
+		TexRow texrow(false);
+		otexrowstream ots(os,texrow);
+		WriteStream wi(ots, false, true, WriteStream::wsDefault);
 		write(wi);
 		pain.draw(x, y, os.str().c_str());
 	}
@@ -605,7 +611,9 @@ static docstring latexString(InsetMathHull const & inset)
 	static Encoding const * encoding = 0;
 	if (inset.isBufferValid())
 		encoding = &(inset.buffer().params().encoding());
-	WriteStream wi(ls, false, true, WriteStream::wsPreview, encoding);
+	TexRow texrow(false);
+	otexrowstream ots(ls,texrow);
+	WriteStream wi(ots, false, true, WriteStream::wsPreview, encoding);
 	inset.write(wi);
 	return ls.str();
 }
@@ -621,7 +629,7 @@ void InsetMathHull::initUnicodeMath() const
 void InsetMathHull::addPreview(DocIterator const & inset_pos,
 	graphics::PreviewLoader & /*ploader*/) const
 {
-	if (RenderPreview::status() == LyXRC::PREVIEW_ON) {
+	if (RenderPreview::previewMath()) {
 		preparePreview(inset_pos);
 	}
 }
@@ -690,7 +698,7 @@ void InsetMathHull::preparePreview(DocIterator const & pos,
 {
 	// there is no need to do all the macro stuff if we're not
 	// actually going to generate the preview.
-	if (RenderPreview::status() != LyXRC::PREVIEW_ON && !forexport)
+	if (!RenderPreview::previewMath() && !forexport)
 		return;
 
 	Buffer const * buffer = pos.buffer();
@@ -710,6 +718,23 @@ void InsetMathHull::preparePreview(DocIterator const & pos,
 	for (; it != end; ++it)
 		macro_preamble.append(*it);
 
+	// set the font series and size for this snippet
+	DocIterator dit = pos;
+	while (dit.inMathed())
+		dit.pop_back();
+	Paragraph const & par = dit.paragraph();
+	Font font = par.getFontSettings(buffer->params(), dit.pos());
+	font.fontInfo().realize(par.layout().font);
+	string const lsize = font.latexSize();
+	docstring setfont;
+	docstring endfont;
+	if (font.fontInfo().series() == BOLD_SERIES) {
+		setfont += from_ascii("\\textbf{");
+		endfont += '}';
+	}
+	if (lsize != "normalsize" && !prefixIs(lsize, "error"))
+		setfont += from_ascii("\\" + lsize + '\n'); 
+
 	docstring setcnt;
 	if (forexport && haveNumbers()) {
 		docstring eqstr = from_ascii("equation");
@@ -731,7 +756,8 @@ void InsetMathHull::preparePreview(DocIterator const & pos,
 				          '{' + convert<docstring>(num) + '}';
 		}
 	}
-	docstring const snippet = macro_preamble + setcnt + latexString(*this);
+	docstring const snippet = macro_preamble + setfont + setcnt
+	                          + latexString(*this) + endfont;
 	LYXERR(Debug::MACROS, "Preview snippet: " << snippet);
 	preview_->addPreview(snippet, *buffer, forexport);
 }
@@ -754,7 +780,7 @@ void InsetMathHull::loadPreview(DocIterator const & pos) const
 
 bool InsetMathHull::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 {
-	if (RenderPreview::status() == LyXRC::PREVIEW_ON) {
+	if (RenderPreview::previewMath()) {
 		reloadPreview(old);
 		cur.screenUpdateFlags(Update::Force);
 	}
@@ -794,10 +820,10 @@ void InsetMathHull::label(row_type row, docstring const & label)
 }
 
 
-void InsetMathHull::numbered(row_type row, bool num)
+void InsetMathHull::numbered(row_type row, Numbered num)
 {
 	numbered_[row] = num;
-	if (!numbered_[row] && label_[row]) {
+	if (!numbered(row) && label_[row]) {
 		delete label_[row];
 		label_[row] = 0;
 	}
@@ -806,19 +832,32 @@ void InsetMathHull::numbered(row_type row, bool num)
 
 bool InsetMathHull::numbered(row_type row) const
 {
-	return numbered_[row];
+	return numbered_[row] == NUMBER;
 }
 
 
 bool InsetMathHull::ams() const
 {
-	return type_ == hullAlign
-		|| type_ == hullFlAlign
-		|| type_ == hullMultline
-		|| type_ == hullGather
-		|| type_ == hullAlignAt
-		|| type_ == hullXAlignAt
-		|| type_ == hullXXAlignAt;
+	switch (type_) {
+		case hullAlign:
+		case hullFlAlign:
+		case hullMultline:
+		case hullGather:
+		case hullAlignAt:
+		case hullXAlignAt:
+		case hullXXAlignAt:
+			return true;
+		case hullNone:
+		case hullSimple:
+		case hullEquation:
+		case hullEqnArray:
+		case hullRegexp:
+			break;
+	}
+	for (size_t row = 0; row < numbered_.size(); ++row)
+		if (numbered_[row] == NOTAG)
+			return true;
+	return false;
 }
 
 
@@ -840,7 +879,7 @@ bool InsetMathHull::numberedType() const
 	if (type_ == hullRegexp)
 		return false;
 	for (row_type row = 0; row < nrows(); ++row)
-		if (numbered_[row])
+		if (numbered(row))
 			return true;
 	return false;
 }
@@ -894,15 +933,18 @@ void InsetMathHull::header_write(WriteStream & os) const
 
 	case hullSimple:
 		os << '$';
+		os.startOuterRow();
 		if (cell(0).empty())
 			os << ' ';
 		break;
 
 	case hullEquation:
+		os << "\n";
+		os.startOuterRow();
 		if (n)
-			os << "\n\\begin{equation" << star(n) << "}\n";
+			os << "\\begin{equation" << star(n) << "}\n";
 		else
-			os << "\n\\[\n";
+			os << "\\[\n";
 		break;
 
 	case hullEqnArray:
@@ -910,17 +952,23 @@ void InsetMathHull::header_write(WriteStream & os) const
 	case hullFlAlign:
 	case hullGather:
 	case hullMultline:
-		os << "\n\\begin{" << hullName(type_) << star(n) << "}\n";
+		os << "\n";
+		os.startOuterRow();
+		os << "\\begin{" << hullName(type_) << star(n) << "}\n";
 		break;
 
 	case hullAlignAt:
 	case hullXAlignAt:
-		os << "\n\\begin{" << hullName(type_) << star(n) << '}'
+		os << "\n";
+		os.startOuterRow();
+		os << "\\begin{" << hullName(type_) << star(n) << '}'
 		  << '{' << static_cast<unsigned int>((ncols() + 1)/2) << "}\n";
 		break;
 
 	case hullXXAlignAt:
-		os << "\n\\begin{" << hullName(type_) << '}'
+		os << "\n";
+		os.startOuterRow();
+		os << "\\begin{" << hullName(type_) << '}'
 		  << '{' << static_cast<unsigned int>((ncols() + 1)/2) << "}\n";
 		break;
 
@@ -929,7 +977,9 @@ void InsetMathHull::header_write(WriteStream & os) const
 		break;
 
 	default:
-		os << "\n\\begin{unknown" << star(n) << "}\n";
+		os << "\n";
+		os.startOuterRow();
+		os << "\\begin{unknown" << star(n) << "}\n";
 		break;
 	}
 }
@@ -949,10 +999,12 @@ void InsetMathHull::footer_write(WriteStream & os) const
 		break;
 
 	case hullEquation:
+		os << "\n";
+		os.startOuterRow();
 		if (n)
-			os << "\n\\end{equation" << star(n) << "}\n";
+			os << "\\end{equation" << star(n) << "}\n";
 		else
-			os << "\n\\]\n";
+			os << "\\]\n";
 		break;
 
 	case hullEqnArray:
@@ -962,11 +1014,15 @@ void InsetMathHull::footer_write(WriteStream & os) const
 	case hullXAlignAt:
 	case hullGather:
 	case hullMultline:
-		os << "\n\\end{" << hullName(type_) << star(n) << "}\n";
+		os << "\n";
+		os.startOuterRow();
+		os << "\\end{" << hullName(type_) << star(n) << "}\n";
 		break;
 
 	case hullXXAlignAt:
-		os << "\n\\end{" << hullName(type_) << "}\n";
+		os << "\n";
+		os.startOuterRow();
+		os << "\\end{" << hullName(type_) << "}\n";
 		break;
 
 	case hullRegexp:
@@ -975,12 +1031,36 @@ void InsetMathHull::footer_write(WriteStream & os) const
 		break;
 
 	default:
-		os << "\n\\end{unknown" << star(n) << "}\n";
+		os << "\n";
+		os.startOuterRow();
+		os << "\\end{unknown" << star(n) << "}\n";
 		break;
 	}
 }
 
 
+bool InsetMathHull::isTable() const
+{
+	switch (type_) {
+	case hullEqnArray:
+	case hullAlign:
+	case hullAlignAt:
+	case hullXAlignAt:
+	case hullXXAlignAt:
+	case hullFlAlign:
+	case hullMultline:
+	case hullGather:
+		return true;
+	case hullNone:
+	case hullSimple:
+	case hullEquation:
+	case hullRegexp:
+		break;
+	}
+	return false;
+}
+
+
 bool InsetMathHull::rowChangeOK() const
 {
 	return
@@ -1010,14 +1090,14 @@ void InsetMathHull::addRow(row_type row)
 	docstring number = empty_docstring();
 	if (type_ == hullMultline) {
 		if (row + 1 == nrows())  {
-			numbered_[row] = false;
+			numbered_[row] = NONUMBER;
 			swap(label, label_[row]);
 			swap(number, numbers_[row]);
 		} else
 			numbered = false;
 	}
 
-	numbered_.insert(numbered_.begin() + row + 1, numbered);
+	numbered_.insert(numbered_.begin() + row + 1, numbered ? NUMBER : NONUMBER);
 	numbers_.insert(numbers_.begin() + row + 1, number);
 	label_.insert(label_.begin() + row + 1, label);
 	InsetMathGrid::addRow(row);
@@ -1030,14 +1110,7 @@ void InsetMathHull::swapRow(row_type row)
 		return;
 	if (row + 1 == nrows())
 		--row;
-	// gcc implements the standard std::vector<bool> which is *not* a container:
-	//   http://www.gotw.ca/publications/N1185.pdf
-	// As a results, it doesn't like this:
-	//	swap(numbered_[row], numbered_[row + 1]);
-	// so we do it manually:
-	bool const b = numbered_[row];
-	numbered_[row] = numbered_[row + 1];
-	numbered_[row + 1] = b;
+	swap(numbered_[row], numbered_[row + 1]);
 	swap(numbers_[row], numbers_[row + 1]);
 	swap(label_[row], label_[row + 1]);
 	InsetMathGrid::swapRow(row);
@@ -1049,9 +1122,7 @@ void InsetMathHull::delRow(row_type row)
 	if (nrows() <= 1 || !rowChangeOK())
 		return;
 	if (row + 1 == nrows() && type_ == hullMultline) {
-		bool const b = numbered_[row - 1];
-		numbered_[row - 1] = numbered_[row];
-		numbered_[row] = b;
+		swap(numbered_[row - 1], numbered_[row]);
 		swap(numbers_[row - 1], numbers_[row]);
 		swap(label_[row - 1], label_[row]);
 		InsetMathGrid::delRow(row);
@@ -1087,7 +1158,7 @@ void InsetMathHull::delCol(col_type col)
 
 docstring InsetMathHull::nicelabel(row_type row) const
 {
-	if (!numbered_[row])
+	if (!numbered(row))
 		return docstring();
 	docstring const & val = numbers_[row];
 	if (!label_[row])
@@ -1324,21 +1395,24 @@ docstring InsetMathHull::eolString(row_type row, bool fragile, bool latex,
 {
 	docstring res;
 	if (numberedType()) {
-		if (label_[row] && numbered_[row]) {
+		if (label_[row] && numbered(row)) {
 			docstring const name =
 				latex ? escape(label_[row]->getParam("name"))
 				      : label_[row]->getParam("name");
 			res += "\\label{" + name + '}';
 		}
-		if (!numbered_[row] && (type_ != hullMultline))
-			res += "\\nonumber ";
+		if (type_ != hullMultline) {
+			if (numbered_[row]  == NONUMBER)
+				res += "\\nonumber ";
+			else if (numbered_[row]  == NOTAG)
+				res += "\\notag ";
+		}
 	}
 	// Never add \\ on the last empty line of eqnarray and friends
 	last_eoln = false;
 	return res + InsetMathGrid::eolString(row, fragile, latex, last_eoln);
 }
 
-
 void InsetMathHull::write(WriteStream & os) const
 {
 	ModeSpecifier specifier(os, MATH_MODE);
@@ -1475,7 +1549,7 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 		if (type_ == hullSimple || type_ == hullEquation) {
 			cur.recordUndoInset();
 			bool const align =
-				cur.bv().buffer().params().use_package("amsmath") == BufferParams::package_on;
+				cur.bv().buffer().params().use_package("amsmath") != BufferParams::package_off;
 			mutate(align ? hullAlign : hullEqnArray);
 			// mutate() may change labels and such.
 			cur.forceBufferUpdate();
@@ -1540,7 +1614,7 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 			// if there is an argument, find the corresponding label, else
 			// check whether there is at least one label.
 			for (row = 0; row != nrows(); ++row)
-				if (numbered_[row] && label_[row]
+				if (numbered(row) && label_[row]
 					  && (cmd.argument().empty() || label(row) == cmd.argument()))
 					break;
 		}
@@ -1637,12 +1711,25 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_MATH_DISPLAY: {
 		cur.recordUndoInset();
 		mutate(type_ == hullSimple ? hullEquation : hullSimple);
-		cur.idx() = 0;
-		cur.pos() = cur.lastpos();
-		//cur.dispatched(FINISHED);
+		// if the cursor is in a cell that got merged, move it to
+		// start of the hull inset.
+		if (cur.idx() > 0) {
+			cur.idx() = 0;
+			cur.pos() = 0;
+		}
+		if (cur.pos() > cur.lastpos())
+			cur.pos() = cur.lastpos();
+
 		break;
 	}
 
+	case LFUN_TABULAR_FEATURE:
+		if (!isTable())
+			cur.undispatched();
+		else
+			InsetMathGrid::doDispatch(cur, cmd);
+		break;
+
 	default:
 		InsetMathGrid::doDispatch(cur, cmd);
 		break;
@@ -1700,6 +1787,7 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 				enable = display() != Inline;
 		}
 		status.setEnabled(enable);
+		status.setOnOff(display() != Inline);
 		return true;
 	}
 
@@ -1733,12 +1821,12 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 			// if there is no argument and we're inside math, we retrieve
 			// the row number from the cursor position.
 			row = (type_ == hullMultline) ? nrows() - 1 : cur.row();
-			enabled = numberedType() && label_[row] && numbered_[row];
+			enabled = numberedType() && label_[row] && numbered(row);
 		} else {
 			// if there is an argument, find the corresponding label, else
 			// check whether there is at least one label.
 			for (row_type row = 0; row != nrows(); ++row) {
-				if (numbered_[row] && label_[row] &&
+				if (numbered(row) && label_[row] &&
 					(cmd.argument().empty() || label(row) == cmd.argument())) {
 						enabled = true;
 						break;
@@ -1756,13 +1844,10 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 		}
 		return InsetMathGrid::getStatus(cur, cmd, status);
 
-	case LFUN_INSET_MODIFY: {
-		istringstream is(to_utf8(cmd.argument()));
-		string s;
-		is >> s;
-		if (s != "tabular")
-			return InsetMathGrid::getStatus(cur, cmd, status);
-		is >> s;
+	case LFUN_TABULAR_FEATURE: {
+		if (!isTable())
+			return false;
+		string s = cmd.getArg(0);
 		if (!rowChangeOK()
 		    && (s == "append-row"
 			|| s == "delete-row"
@@ -1783,16 +1868,6 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 			status.setEnabled(false);
 			return true;
 		}
-		if ((type_ == hullSimple
-		  || type_ == hullEquation
-		  || type_ == hullNone) &&
-		    (s == "add-hline-above" || s == "add-hline-below")) {
-			status.message(bformat(
-				from_utf8(N_("Can't add horizontal grid lines in '%1$s'")),
-				hullName(type_)));
-			status.setEnabled(false);
-			return true;
-		}
 		if (s == "add-vline-left" || s == "add-vline-right") {
 			status.message(bformat(
 				from_utf8(N_("Can't add vertical grid lines in '%1$s'")),
@@ -1968,7 +2043,9 @@ bool InsetMathHull::searchForward(BufferView * bv, string const & str,
 void InsetMathHull::write(ostream & os) const
 {
 	odocstringstream oss;
-	WriteStream wi(oss, false, false, WriteStream::wsDefault);
+	TexRow texrow(false);
+	otexrowstream ots(oss,texrow);
+	WriteStream wi(ots, false, false, WriteStream::wsDefault);
 	oss << "Formula ";
 	write(wi);
 	os << to_utf8(oss.str());
@@ -2010,8 +2087,10 @@ int InsetMathHull::plaintext(odocstringstream & os,
 	}
 
 	odocstringstream oss;
+	TexRow texrow(false);
+	otexrowstream ots(oss,texrow);
 	Encoding const * const enc = encodings.fromLyXName("utf8");
-	WriteStream wi(oss, false, true, WriteStream::wsDefault, enc);
+	WriteStream wi(ots, false, true, WriteStream::wsDefault, enc);
 
 	// Fix Bug #6139
 	if (type_ == hullRegexp)
@@ -2024,7 +2103,8 @@ int InsetMathHull::plaintext(odocstringstream & os,
 			// and do not include the newline.
 			if (op.for_toc || op.for_tooltip || oss.str().size() >= max_length)
 				break;
-			wi << "\n";
+			if (r < nrows() - 1)
+				wi << "\n";
 		}
 	}
 	docstring const str = oss.str();
@@ -2050,12 +2130,14 @@ int InsetMathHull::docbook(odocstream & os, OutputParams const & runparams) cons
 	++ms.tab(); ms.cr(); ms.os() << '<' << bname << '>';
 
 	odocstringstream ls;
+	TexRow texrow;
+	otexstream ols(ls, texrow);
 	if (runparams.flavor == OutputParams::XML) {
 		ms << MTag("alt role='tex' ");
 		// Workaround for db2latex: db2latex always includes equations with
 		// \ensuremath{} or \begin{display}\end{display}
 		// so we strip LyX' math environment
-		WriteStream wi(ls, false, false, WriteStream::wsDefault, runparams.encoding);
+		WriteStream wi(ols, false, false, WriteStream::wsDefault, runparams.encoding);
 		InsetMathGrid::write(wi);
 		ms << from_utf8(subst(subst(to_utf8(ls.str()), "&", "&amp;"), "<", "&lt;"));
 		ms << ETag("alt");
@@ -2065,9 +2147,6 @@ int InsetMathHull::docbook(odocstream & os, OutputParams const & runparams) cons
 		InsetMathGrid::mathmlize(ms);
 		ms << ETag("math");
 	} else {
-		TexRow texrow;
-		texrow.reset();
-		otexstream ols(ls, texrow);
 		ms << MTag("alt role='tex'");
 		latex(ols, runparams);
 		res = texrow.rows();
@@ -2099,7 +2178,7 @@ bool InsetMathHull::haveNumbers() const
 	if (getType() == hullSimple)
 		return havenumbers;
 	for (size_t i = 0; i != numbered_.size(); ++i) {
-		if (numbered_[i]) {
+		if (numbered(i)) {
 			havenumbers = true;
 			break;
 		}
@@ -2317,7 +2396,9 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 		// Unfortunately, we cannot use latexString() because we do not want
 		// $...$ or whatever.
 		odocstringstream ls;
-		WriteStream wi(ls, false, true, WriteStream::wsPreview);
+		TexRow texrow(false);
+		otexrowstream ots(ls,texrow);
+		WriteStream wi(ots, false, true, WriteStream::wsPreview);
 		ModeSpecifier specifier(wi, MATH_MODE);
 		mathAsLatex(wi);
 		docstring const latex = ls.str();
@@ -2344,11 +2425,13 @@ void InsetMathHull::toString(odocstream & os) const
 }
 
 
-void InsetMathHull::forToc(docstring & os, size_t) const
+void InsetMathHull::forOutliner(docstring & os, size_t const, bool const) const
 {
 	odocstringstream ods;
 	OutputParams op(0);
 	op.for_toc = true;
+	// FIXME: this results in spilling TeX into the LyXHTML output since the
+	// outliner is used to generate the LyXHTML list of figures/etc.
 	plaintext(ods, op);
 	os += ods.str();
 }
