@@ -31,6 +31,15 @@ class RenderPreview;
 /// This provides an interface between "LyX insets" and "LyX math insets"
 class InsetMathHull : public InsetMathGrid {
 public:
+	/// How a line is numbered
+	enum Numbered {
+		/// not numbered, LaTeX code \\nonumber if line differs from inset
+		NONUMBER,
+		/// numbered, LaTeX code \\number if line differs from inset
+		NUMBER,
+		/// not numbered, LaTeX code \\notag if line differs from inset
+		NOTAG
+	};
 	///
 	InsetMathHull(Buffer * buf);
 	///
@@ -42,7 +51,8 @@ public:
 	///
 	void updateBuffer(ParIterator const &, UpdateType);
 	///
-	void addToToc(DocIterator const & di, bool output_active) const;
+	void addToToc(DocIterator const & di, bool output_active,
+				  UpdateType utype) const;
 	///
 	InsetMathHull & operator=(InsetMathHull const &);
 	///
@@ -66,7 +76,9 @@ public:
 	///
 	ColorCode backgroundColor(PainterInfo const &) const;
 	///
-	void numbered(row_type row, bool num);
+	void numbered(row_type row, bool num) { numbered(row, num ? NUMBER : NONUMBER); }
+	///
+	void numbered(row_type row, Numbered num);
 	///
 	bool numbered(row_type row) const;
 	///
@@ -101,8 +113,7 @@ public:
 	///
 	char defaultColAlign(col_type col);
 	///
-	///
-	char displayColAlign(col_type col, row_type row) const;
+	char displayColAlign(idx_type idx) const;
 	///
 	bool idxFirst(Cursor &) const;
 	///
@@ -141,7 +152,7 @@ public:
 	/// 
 	void toString(odocstream &) const;
 	///
-	void forToc(docstring &, size_t) const;
+	void forOutliner(docstring &, size_t const, bool const) const;
 
 	/// get notification when the cursor leaves this inset
 	bool notifyCursorLeaves(Cursor const & old, Cursor & cur);
@@ -225,6 +236,8 @@ private:
 	ColorCode standardColor() const;
 	/// consistency check
 	void check() const;
+	/// does it understand tabular-feature commands?
+	bool isTable() const;
 	/// can this change its number of rows?
 	bool rowChangeOK() const;
 	/// can this change its number of cols?
@@ -235,7 +248,7 @@ private:
 	/// "none", "simple", "display", "eqnarray",...
 	HullType type_;
 	///
-	std::vector<bool> numbered_;
+	std::vector<Numbered> numbered_;
 	///
 	std::vector<docstring> numbers_;
 	///
