@@ -64,6 +64,7 @@ following hack as starting point to write some macros:
 #include "InsetMathString.h"
 #include "InsetMathTabular.h"
 #include "MathMacroTemplate.h"
+#include "MathExtern.h"
 #include "MathFactory.h"
 #include "MathMacroArgument.h"
 #include "MathSupport.h"
@@ -228,6 +229,8 @@ bool addCol(InsetMathGrid & grid, InsetMathGrid::col_type & cellcol)
  * \endverbatim
  * will result in a grid with 3 rows (+ the dummy row that is always present),
  * because the last '\\' opens a new row.
+ * Do never delete a row that contains a multicolumn, even if all cells empty,
+ * since the multicolumn information would get lost otherwise.
  * Note that this is only needed for inner-hull grid types, such as array
  * or aligned, but not for outer-hull grid types, such as eqnarray or align.
  */
@@ -235,7 +238,9 @@ void delEmptyLastRow(InsetMathGrid & grid)
 {
 	InsetMathGrid::row_type const row = grid.nrows() - 1;
 	for (InsetMathGrid::col_type col = 0; col < grid.ncols(); ++col) {
-		if (!grid.cell(grid.index(row, col)).empty())
+		InsetMathGrid::idx_type const idx = grid.index(row, col);
+		if (!grid.cell(idx).empty() ||
+		    grid.cellinfo(idx).multi_ != InsetMathGrid::CELL_NORMAL)
 			return;
 	}
 	// Copy the row information of the empty row (which would contain the
@@ -1374,7 +1379,6 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 		}
 
-#if 0
 		else if (t.cs() == "multicolumn") {
 			// extract column count and insert dummy cells
 			MathData count;
@@ -1382,30 +1386,30 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			int cols = 1;
 			if (!extractNumber(count, cols)) {
 				success_ = false;
-				lyxerr << " can't extract number of cells from " << count << endl;
+				error("can't extract number of multicolumn cells");
 			}
 			// resize the table if necessary
-			for (int i = 0; i < cols; ++i) {
+			size_t first = grid.index(cellrow, cellcol);
+			for (int i = 1; i < cols; ++i) {
 				if (addCol(grid, cellcol)) {
-					cell = &grid.cell(grid.index(
-							cellrow, cellcol));
-					// mark this as dummy
-					grid.cellinfo(grid.index(
-						cellrow, cellcol)).dummy_ = true;
+					size_t const idx = grid.index(cellrow, cellcol);
+					grid.cellinfo(idx).multi_ =
+						InsetMathGrid::CELL_PART_OF_MULTICOLUMN;
 				}
 			}
-			// the last cell is the real thing, not a dummy
-			grid.cellinfo(grid.index(cellrow, cellcol)).dummy_ = false;
+
+			// the first cell is the real thing, not a dummy
+			cell = &grid.cell(first);
+			grid.cellinfo(first).multi_ = InsetMathGrid::CELL_BEGIN_OF_MULTICOLUMN;
 
 			// read special alignment
 			MathData align;
 			parse(align, FLAG_ITEM, mode);
-			//grid.cellinfo(grid.index(cellrow, cellcol)).align_ = extractString(align);
+			grid.cellinfo(first).align_ = asString(align);
 
 			// parse the remaining contents into the "real" cell
 			parse(*cell, FLAG_ITEM, mode);
 		}
-#endif
 
 		else if (t.cs() == "limits" || t.cs() == "nolimits") {
 			CatCode const cat = nextToken().cat();
@@ -1417,15 +1421,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 		}
 
-		else if (t.cs() == "nonumber") {
-			if (grid.asHullInset())
-				grid.asHullInset()->numbered(cellrow, false);
-		}
+		// \notag is the same as \nonumber if amsmath is used
+		else if ((t.cs() == "nonumber" || t.cs() == "notag") &&
+		         grid.asHullInset())
+			grid.asHullInset()->numbered(cellrow, false);
 
-		else if (t.cs() == "number") {
-			if (grid.asHullInset())
-				grid.asHullInset()->numbered(cellrow, true);
-		}
+		else if (t.cs() == "number" && grid.asHullInset())
+			grid.asHullInset()->numbered(cellrow, true);
 
 		else if (t.cs() == "hline") {
 			grid.rowinfo(cellrow).lines_ ++;
@@ -1543,6 +1545,18 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 		}
 
+		else if (t.cs() == "xhookrightarrow" || t.cs() == "xhookleftarrow" ||
+			     t.cs() == "xRightarrow" || t.cs() == "xLeftarrow" ||
+				 t.cs() == "xleftrightarrow" || t.cs() == "xLeftrightarrow" ||
+				 t.cs() == "xrightharpoondown" || t.cs() == "xrightharpoonup" ||
+				 t.cs() == "xleftharpoondown" || t.cs() == "xleftharpoonup" ||
+				 t.cs() == "xleftrightharpoons" || t.cs() == "xrightleftharpoons" ||
+				 t.cs() == "xmapsto") {
+			cell->push_back(createInsetMath(t.cs(), buf));
+			parse(cell->back().nucleus()->cell(1), FLAG_OPTION, mode);
+			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+		}
+
 		else if (t.cs() == "ref" || t.cs() == "eqref" || t.cs() == "prettyref"
 			  || t.cs() == "pageref" || t.cs() == "vpageref" || t.cs() == "vref") {
 			cell->push_back(MathAtom(new InsetMathRef(buf, t.cs())));
@@ -1738,7 +1752,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						lyxerr << "found math environment `"
 						       << to_utf8(name)
 						       << "' in symbols file with unsupported inset `"
-						       << to_utf8(l->inset)
+						       << l->inset
 						       << "'." << endl;
 					}
 					// create generic environment inset
@@ -1898,16 +1912,22 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			skipSpaces();
 			if (nextToken().asInput() == "[") {
 				// Since the phantom inset cannot handle optional arguments
-				// we must not create an InsetMathPhantom (bug 8967).
+				// other than b and t, we must not create an InsetMathPhantom
+				// if opt is different from b and t (bug 8967).
 				docstring const opt = parse_verbatim_option();
-				docstring const arg = parse_verbatim_item();
-				cell->push_back(MathAtom(new MathMacro(buf, t.cs())));
-				MathData ar;
-				mathed_parse_cell(ar, '[' + opt + ']', mode_);
-				cell->append(ar);
-				ar = MathData();
-				mathed_parse_cell(ar, '{' + arg + '}', mode_);
-				cell->append(ar);
+				if (opt == "t" || opt == "b") {
+					cell->push_back(createInsetMath(t.cs() + opt, buf));
+					parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+				} else {
+					docstring const arg = parse_verbatim_item();
+					cell->push_back(MathAtom(new MathMacro(buf, t.cs())));
+					MathData ar;
+					mathed_parse_cell(ar, '[' + opt + ']', mode_);
+					cell->append(ar);
+					ar = MathData();
+					mathed_parse_cell(ar, '{' + arg + '}', mode_);
+					cell->append(ar);
+				}
 			}
 			else {
 				cell->push_back(createInsetMath(t.cs(), buf));
